@@ -39,6 +39,8 @@ import pooler
 import netsvc
 from osv import fields
 
+import zipfile
+
 logger = netsvc.Logger()
 
 opj = os.path.join
@@ -57,7 +59,7 @@ class Graph(dict):
 			father.addChild(name)
 		else:
 			Node(name, self)
-	
+
 	def __iter__(self):
 		level = 0
 		done = Set(self.keys())
@@ -67,7 +69,7 @@ class Graph(dict):
 				done.remove(name)
 				yield module
 			level += 1
-	
+
 class Singleton(object):
 
 	def __new__(cls, name, graph):
@@ -87,7 +89,7 @@ class Node(Singleton):
 			self.childs = []
 		if not hasattr(self, 'depth'):
 			self.depth = 0
-	
+
 	def addChild(self, name):
 		node = Node(name, self.graph)
 		node.depth = self.depth + 1
@@ -97,7 +99,7 @@ class Node(Singleton):
 			if hasattr(self, attr):
 				setattr(node, attr, True)
 		self.childs.sort(lambda x,y: cmp(x.name, y.name))
-	
+
 	def hasChild(self, name):
 		return Node(name, self.graph) in self.childs or \
 				bool([c for c in self.childs if c.hasChild(name)])
@@ -124,23 +126,35 @@ class Node(Singleton):
 			s += '%s`-> %s' % ('   ' * depth, c._pprint(depth+1))
 		return s
 
-def create_graph(module_list, force=[]):
+def create_graph(module_list, force=None):
+	if not force:
+		force=[]
 	graph = Graph()
 	packages = []
 
 	for module in module_list:
+		if module[-4:]=='.zip':
+			module = module[:-4]
 		terp_file = opj(ad, module, '__terp__.py')
-		if os.path.isfile(terp_file):
-			info = eval(file(terp_file).read())
+		mod_path = opj(ad, module)
+		if os.path.isfile(terp_file) or zipfile.is_zipfile(mod_path+'.zip'):
+			try:
+				info = eval(tools.file_open(terp_file).read())
+			except:
+				logger.notifyChannel('init', netsvc.LOG_ERROR, 'addon:%s:eval file %s' % (module, terp_file))
+				raise
 			if info.get('installable', True):
 				packages.append((module, info.get('depends', []), info))
 
 	current,later = Set([p for p, dep, data in packages]), Set()
 	while packages and current > later:
 		package, deps, datas = packages[0]
-		
+
 		# if all dependencies of 'package' are already in the graph, add 'package' in the graph
 		if reduce(lambda x,y: x and y in graph, deps, True):
+			if not package in current:
+				packages.pop(0)
+				continue
 			later.clear()
 			current.remove(package)
 			graph.addNode(package, deps)
@@ -152,25 +166,27 @@ def create_graph(module_list, force=[]):
 		else:
 			later.add(package)
 			packages.append((package, deps, datas))
-		packages = packages[1:]
+		packages.pop(0)
 	
 	for package in later:
 		logger.notifyChannel('init', netsvc.LOG_ERROR, 'addon:%s:Unmet dependency' % package)
-	
+
 	return graph
 
 def init_module_objects(cr, module_name, obj_list):
 	pool = pooler.get_pool(cr.dbname)
 	logger.notifyChannel('init', netsvc.LOG_INFO, 'addon:%s:creating or updating database tables' % module_name)
 	for obj in obj_list:
-		#CHECKME: is this test useful? all objects are supposed to have an _auto_init method, right?
-		#if hasattr(obj, '_auto_init'):
 		if hasattr(obj, 'init'):
 			obj.init(cr)
 		obj._auto_init(cr)
 		cr.commit()
 
-def load_module_graph(cr, graph, status={}):
+def load_module_graph(cr, graph, status=None, **kwargs):
+	# **kwargs is passed directly to convert_xml_import
+	if not status:
+		status={}
+	status = status.copy()
 	package_todo = []
 	statusi = 0
 	for package in graph:
@@ -194,29 +210,28 @@ def load_module_graph(cr, graph, status={}):
 					logger.notifyChannel('init', netsvc.LOG_INFO, 'addon:%s:loading %s' % (m, filename))
 					name, ext = os.path.splitext(filename)
 					if ext == '.csv':
-						tools.convert_csv_import(cr, m, filename, tools.file_open(opj(m, filename)).read(), idref, mode=mode)
+						tools.convert_csv_import(cr, m, os.path.basename(filename), tools.file_open(opj(m, filename)).read(), idref, mode=mode)
 					elif ext == '.sql':
 						queries = tools.file_open(opj(m, filename)).read().split(';')
 						for query in queries:
 							new_query = ' '.join(query.split())
 							if new_query:
 								cr.execute(new_query)
-						cr.commit()
 					else:
-						tools.convert_xml_import(cr, m, tools.file_open(opj(m, filename)).read(), idref, mode=mode)
+						tools.convert_xml_import(cr, m, tools.file_open(opj(m, filename)).read(), idref, mode=mode, **kwargs)
 			if hasattr(package, 'demo') or (package_demo and package_state != 'installed'):
 				status['progress'] = (float(statusi)+0.75)/len(graph)
 				for xml in package.datas.get('demo_xml', []):
 					name, ext = os.path.splitext(xml)
 					logger.notifyChannel('init', netsvc.LOG_INFO, 'addon:%s:loading %s' % (m, xml))
 					if ext == '.csv':
-						tools.convert_csv_import(cr, m, os.path.basename(xml), tools.file_open(opj(m, xml)).read(), idref)
+						tools.convert_csv_import(cr, m, os.path.basename(xml), tools.file_open(opj(m, xml)).read(), idref, noupdate=True)
 					else:
-						tools.convert_xml_import(cr, m, tools.file_open(opj(m, xml)).read(), idref)
+						tools.convert_xml_import(cr, m, tools.file_open(opj(m, xml)).read(), idref, noupdate=True, **kwargs)
 				cr.execute('update ir_module_module set demo=%s where name=%s', (True, package.name))
 			package_todo.append(package.name)
 			cr.execute("update ir_module_module set state='installed' where state in ('to upgrade', 'to install') and name=%s", (package.name,))
-			cr.commit()
+		cr.commit()
 		statusi+=1
 
 	pool = pooler.get_pool(cr.dbname)
@@ -229,22 +244,69 @@ def register_classes():
 		m = package.name
 		logger.notifyChannel('init', netsvc.LOG_INFO, 'addon:%s:registering classes' % m)
 		sys.stdout.flush()
-		imp.load_module(m, *imp.find_module(m))
 
-def load_modules(db, force_demo=False, status={}, update_module=False):
+		if not os.path.isfile(opj(ad, m+'.zip')):
+			# XXX must restrict to only addons paths
+			imp.load_module(m, *imp.find_module(m))
+		else:
+			import zipimport
+			mod_path = opj(ad, m+'.zip')
+			try:
+				zimp = zipimport.zipimporter(mod_path)
+				zimp.load_module(m)
+			except zipimport.ZipImportError:
+				logger.notifyChannel('init', netsvc.LOG_ERROR, 'Couldn\'t find module %s' % m)
+
+def load_modules(db, force_demo=False, status=None, update_module=False):
+	if not status:
+		status={}
+	cr = db.cursor()
 	force = []
 	if force_demo:
 		force.append('demo')
-	cr = db.cursor()
 	if update_module:
-		cr.execute("select name from ir_module_module where state in ('installed', 'to install', 'to upgrade')")
+		cr.execute("select name from ir_module_module where state in ('installed', 'to install', 'to upgrade','to remove')")
 	else:
-		cr.execute("select name from ir_module_module where state = 'installed'")
+		cr.execute("select name from ir_module_module where state in ('installed', 'to upgrade', 'to remove')")
 	module_list = [name for (name,) in cr.fetchall()]
 	graph = create_graph(module_list, force)
-	load_module_graph(cr, graph, status)
+	report = tools.assertion_report()
+	load_module_graph(cr, graph, status, report=report)
+	if report.get_report():
+		logger.notifyChannel('init', netsvc.LOG_INFO, 'assert:%s' % report)
+
 	for kind in ('init', 'demo', 'update'):
 		tools.config[kind]={}
+
 	cr.commit()
+	if update_module:
+		cr.execute("select id,name from ir_module_module where state in ('to remove')")
+		for mod_id, mod_name in cr.fetchall():
+			pool = pooler.get_pool(cr.dbname)
+			cr.execute('select model,res_id from ir_model_data where not noupdate and module=%s order by id desc', (mod_name,))
+			for rmod,rid in cr.fetchall():
+				#
+				# TO BE Improved:
+				#   I can not use the class_pool has _table could be defined in __init__
+				#   and I can not use the pool has the module could not be loaded in the pool
+				#
+				uid = 1
+				pool.get(rmod).unlink(cr, uid, [rid])
+			cr.commit()
+		#
+		# TODO: remove menu without actions of childs
+		#
+		cr.execute('''delete from
+				ir_ui_menu
+			where
+				(id not in (select parent_id from ir_ui_menu where parent_id is not null))
+			and
+				(id not in (select res_id from ir_values where model='ir.ui.menu'))
+			and
+				(id not in (select res_id from ir_model_data where model='ir.ui.menu'))''')
+
+		cr.execute("update ir_module_module set state=%s where state in ('to remove')", ('uninstalled', ))
+		cr.commit()
+		pooler.restart_pool(cr.dbname)
 	cr.close()
 
