@@ -1,7 +1,7 @@
 {
 	"name" : "Accounting and financial management",
 	"version" : "1.0",
-	"depends" : ["product"],
+	"depends" : ["product", "base"],
 	"author" : "Tiny",
 	"description": """Financial and accounting module that covers:
 	General accounting
@@ -17,24 +17,28 @@
 	"demo_xml" : [
 		"account_demo.xml",
 		"project/project_demo.xml",
-		"project/account.analytic.account.csv"
+		"project/analytic_account_demo.xml",
+		"account_unit_test.xml",
 	],
 	"update_xml" : [
 		"account_wizard.xml",
 		"account_view.xml",
 		"account_end_fy.xml",
-		"account_view_transfer.xml",
 		"account_invoice_view.xml",
 		"account_report.xml",
 		"partner_view.xml",
 		"data/account_invoice.xml",
-		"data/account_data.xml",
+		"data/account_data1.xml",
 		"data/account_minimal.xml",
+		"data/account_data2.xml",
 		"account_invoice_workflow.xml",
+		"project/project_wizard.xml",
 		"project/project_view.xml",
 		"project/project_report.xml",
 		"product_data.xml",
 		"product_view.xml",
+		
+		"account_assert_test.xml",
 	],
 #	"translations" : {
 #		"fr": "i18n/french_fr.csv"
