@@ -196,7 +196,7 @@ class account_bank_statement(osv.osv):
 					for newline in move.reconcile_id.line_new_ids:
 						amount += newline.amount
 
-				torec.append(account_move_line_obj.create(cr, uid, {
+				val = {
 					'name': move.name,
 					'date': move.date,
 					'ref': move.ref,
@@ -208,7 +208,25 @@ class account_bank_statement(osv.osv):
 					'statement_id': st.id,
 					'journal_id': st.journal_id.id,
 					'period_id': st.period_id.id,
-				}, context=context))
+					'currency_id': st.currency.id,
+				}
+
+				print 'AMT', move.amount
+				amount = res_currency_obj.compute(cr, uid, st.currency.id,
+						company_currency_id, move.amount, context=context)
+				print 'AMT', amount
+
+				if move.account_id and move.account_id.currency_id:
+					val['currency_id'] = move.account_id.currency_id.id
+					if company_currency_id==move.account_id.currency_id.id:
+						amount_cur = move.amount
+					else:
+						amount_cur = res_currency_obj.compute(cr, uid, company_currency_id,
+								move.account_id.currency_id.id, amount, context=context)
+					val['amount_currency'] = -amount_cur
+					print 'AMT VAL', amount_cur
+
+				torec.append(account_move_line_obj.create(cr, uid, val , context=context))
 
 				if move.reconcile_id and move.reconcile_id.line_new_ids:
 					for newline in move.reconcile_id.line_new_ids:
@@ -226,8 +244,6 @@ class account_bank_statement(osv.osv):
 							'period_id': st.period_id.id,
 						}, context=context)
 
-				amount = res_currency_obj.compute(cr, uid, st.currency.id,
-						company_currency_id, move.amount, context=context)
 				if amount >= 0:
 					account_id = st.journal_id.default_credit_account_id.id
 				else:
@@ -268,8 +284,9 @@ class account_bank_statement(osv.osv):
 				if move.reconcile_id and move.reconcile_id.line_ids:
 					torec += map(lambda x: x.id, move.reconcile_id.line_ids)
 					try:
-						account_move_line_obj.reconcile(cr, uid, torec, 'statement', context)
-					except:
+						account_move_line_obj.reconcile(cr, uid, torec, 'statement', writeoff_period_id=st.period_id.id, writeoff_journal_id=st.journal_id.id, writeoff_acc_id=move.account_id.id ,context=context)
+					except Exception, e:
+						print e
 						raise osv.except_osv('Error !', 'Unable to reconcile entry "%s": %.2f'%(move.name, move.amount))
 
 			done.append(st.id)
@@ -448,7 +465,7 @@ class account_bank_statement_reconcile(osv.osv):
 			 'reconcile_id', 'Bank Statement Line'),
 	}
 	_defaults = {
-		'name': lambda *a: time.strftime('%Y-%m-%d'),
+		'name': lambda self, cr, uid, ctx: ctx.get('date', time.strftime('%Y-%m-%d')),
 		'partner_id': lambda obj, cursor, user, context=None: \
 				context.get('partner', False),
 		'total_amount': _default_amount,
