@@ -38,20 +38,45 @@ class account_move_line(osv.osv):
 	_description = "Entry lines"
 
 	def _query_get(self, cr, uid, obj='l', context={}):
-		fiscalyear_obj = self.pool.get('account.fiscalyear')
-		if not context.get('fiscalyear', False):
-			fiscalyear_ids = fiscalyear_obj.search(cr, uid, [('state', '=', 'draft')])
-			if fiscalyear_ids:
-				fiscalyear_clause = ','.join([str(x) for x in fiscalyear_ids])
+		# Case date_to or / and date_from
+		query_get=obj+".active AND "+obj+".state<>'draft'"
+		
+		if context.has_key('date_from'):
+			query_get=query_get+" AND "+obj+".date >= %s "%(context['date_from'])
+		if context.has_key('date_to'):
+			query_get=query_get+ "AND "+obj+".date <= %s "%(context['date_to'])
+		
+				
+		# We have specified a fiscal year take it
+		# Make this : ".period_id in (SELECT id from account_period WHERE fiscalyear_id in (%s) AND id in (%s))"
+		second_part_query=''
+		if context.get('fiscalyear', False):
+			fiscalyear_clause = context['fiscalyear']
+			# second_part_query =  " AND "+obj+".period_id in (SELECT id from account_period WHERE fiscalyear_id in (%s)"%(context['fiscalyear'])
+		# Else, no fiscal year define
+		else:
+			# If date_to or date_from choosen, don't take fiscalyear
+			if not(context.has_key('date_from') or context.has_key('date_to')):
+				fiscalyear_obj = self.pool.get('account.fiscalyear')
+				fiscalyear_ids = fiscalyear_obj.search(cr, uid, [('state', '=', 'draft')])
+				if fiscalyear_ids:
+					fiscalyear_clause = ','.join([str(x) for x in fiscalyear_ids])
+				else:
+					fiscalyear_clause = 'NULL'
+				# second_part_query = " AND "+obj+".period_id in (SELECT id from account_period WHERE fiscalyear_id in (%s)"%(fiscalyear_clause)
 			else:
 				fiscalyear_clause = 'NULL'
-		else:
-			fiscalyear_clause = '%s' % context['fiscalyear']
+				# second_part_query= " AND "+obj+".period_id in (SELECT id from account_period WHERE "
+				
+		# have we define period ?
 		if context.get('periods', False):
 			ids = ','.join([str(x) for x in context['periods']])
-			return obj+".active AND "+obj+".state<>'draft' AND "+obj+".period_id in (SELECT id from account_period WHERE fiscalyear_id in (%s) AND id in (%s))" % (fiscalyear_clause, ids)
+			second_part_query=" AND "+obj+".period_id in (SELECT id from account_period WHERE fiscalyear_id in (%s) AND id in (%s))"%(fiscalyear_clause,ids)
 		else:
-			return obj+".active AND "+obj+".state<>'draft' AND "+obj+".period_id in (SELECT id from account_period WHERE fiscalyear_id in (%s))" % (fiscalyear_clause,)
+			second_part_query=" AND "+obj+".period_id in (SELECT id from account_period WHERE fiscalyear_id in (%s))"%(fiscalyear_clause)
+			
+		query_get=query_get+second_part_query
+		return query_get
 
 	def default_get(self, cr, uid, fields, context={}):
 		data = self._default_get(cr, uid, fields, context)
@@ -334,7 +359,8 @@ class account_move_line(osv.osv):
 		id_set = ','.join(map(str, ids))
 		lines = self.read(cr, uid, ids, context=context)
 		unrec_lines = filter(lambda x: not x['reconcile_id'], lines)
-		credit = debit = 0
+		credit = debit = 0.0
+		currency = 0.0
 		account_id = False
 		partner_id = False
 		for line in unrec_lines:
@@ -343,10 +369,17 @@ class account_move_line(osv.osv):
 						'Entry "%s" is not valid !' % line.name)
 			credit += line['credit']
 			debit += line['debit']
+			currency += line['amount_currency'] or 0.0
 			account_id = line['account_id'][0]
 			partner_id = (line['partner_id'] and line['partner_id'][0]) or False
 		writeoff = debit - credit
-		date = time.strftime('%Y-%m-%d')
+		
+		
+		# Ifdate_p in context => take this date
+		if context.has_key('date_p') and context['date_p']:
+			date=context['date_p']
+		else:
+			date = time.strftime('%Y-%m-%d')
 
 		cr.execute('SELECT account_id, reconcile_id \
 				FROM account_move_line \
@@ -362,7 +395,8 @@ class account_move_line(osv.osv):
 		if r[0][1] != None:
 			raise osv.except_osv('Error', 'Some entries are already reconciled !')
 
-		if not self.pool.get('res.currency').is_zero(cr, uid, account.company_id.currency_id, writeoff):
+		if (not self.pool.get('res.currency').is_zero(cr, uid, account.company_id.currency_id, writeoff)) or \
+		   (account.currency_id and (not self.pool.get('res.currency').is_zero(cr, uid, account.currency_id, currency))):
 			if not writeoff_acc_id:
 				raise osv.except_osv('Warning', 'You have to provide an account for the write off entry !')
 			if writeoff > 0:
@@ -375,10 +409,31 @@ class account_move_line(osv.osv):
 				credit = -writeoff
 				self_credit = 0.0
 				self_debit = -writeoff
-
+			# If comment exist in context, take it
+			if context.has_key('comment') and context['comment']:
+				libelle=context['comment']
+			else:
+				libelle='Write-Off'
+			
 			writeoff_lines = [
-				(0, 0, {'name':'Write-Off', 'debit':self_debit, 'credit':self_credit, 'account_id':account_id, 'date':date, 'partner_id':partner_id}),
-				(0, 0, {'name':'Write-Off', 'debit':debit, 'credit':credit, 'account_id':writeoff_acc_id, 'date':date, 'partner_id':partner_id})
+				(0, 0, {
+					'name':libelle,
+					'debit':self_debit,
+					'credit':self_credit,
+					'account_id':account_id,
+					'date':date,
+					'partner_id':partner_id,
+					'currency_id': account.currency_id.id or False,
+					'amount_currency': account.currency_id.id and -currency or 0.0
+				}),
+				(0, 0, {
+					'name':libelle,
+					'debit':debit,
+					'credit':credit,
+					'account_id':writeoff_acc_id,
+					'date':date,
+					'partner_id':partner_id
+				})
 			]
 
 			name = 'Write-Off'
@@ -391,7 +446,6 @@ class account_move_line(osv.osv):
 				'name': name,
 				'period_id': writeoff_period_id,
 				'journal_id': writeoff_journal_id,
-
 				'state': 'draft',
 				'line_id': writeoff_lines
 			})
@@ -569,20 +623,28 @@ class account_move_line(osv.osv):
 					raise osv.except_osv('No piece number !', 'Can not create an automatic sequence for this piece !\n\nPut a sequence in the journal definition for automatic numbering or create a sequence manually for this piece.')
 
 		ok = not (journal.type_control_ids or journal.account_control_ids)
-		if ('account_id' in vals) and journal.type_control_ids:
-			type = account_obj.browse(cr, uid, vals['account_id']).type
-			for t in journal.type_control_ids:
-				if type==t.code:
-					ok = True
-					break
-		if ('account_id' in vals) and journal.account_control_ids and not ok:
-			for a in journal.account_control_ids:
-				if a.id==vals['account_id']:
-					ok = True
-					break
+		if ('account_id' in vals):
+			account = account_obj.browse(cr, uid, vals['account_id'])
+			if journal.type_control_ids:
+				type = account.type
+				for t in journal.type_control_ids:
+					if type==t.code:
+						ok = True
+						break
+			if journal.account_control_ids and not ok:
+				for a in journal.account_control_ids:
+					if a.id==vals['account_id']:
+						ok = True
+						break
+			if (account.currency_id) and 'amount_currency' not in vals:
+				vals['currency_id'] = account.currency_id.id
+				cur_obj = self.pool.get('res.currency')
+				ctx = {}
+				if 'date' in vals:
+					ctx['date'] = vals['date']
+				vals['amount_currency'] = cur_obj.compute(cr, uid, account.company_id.currency_id.id, account.currency_id.id, vals.get('debit', 0.0)-vals.get('credit', 0.0), context=ctx)
 		if not ok:
 			raise osv.except_osv('Bad account !', 'You can not use this general account in this journal !')
-
 		result = super(osv.osv, self).create(cr, uid, vals, context)
 		if check:
 			self.pool.get('account.move').validate(cr, uid, [vals['move_id']], context)
