@@ -499,7 +499,7 @@ class account_invoice(osv.osv):
 			'credit':x['price']<0 and -x['price'],
 			'account_id':x['account_id'],
 			'analytic_lines':x.get('analytic_lines', []),
-			'amount_currency':x.get('amount_currency', False),
+			'amount_currency':x['price']>0 and abs(x.get('amount_currency', False)) or -abs(x.get('amount_currency', False)),
 			'currency_id':x.get('currency_id', False),
 			'tax_code_id': x.get('tax_code_id', False),
 			'tax_amount': x.get('tax_amount', False),
@@ -613,7 +613,7 @@ class account_invoice(osv.osv):
 				line['account_analytic_id'] = line.get('account_analytic_id', False) and line['account_analytic_id'][0]
 		return map(lambda x: (0,0,x), lines)
 
-	def refund(self, cr, uid, ids):
+	def refund(self, cr, uid, ids, date=None, period_id=None, description=None):
 		invoices = self.read(cr, uid, ids, ['name', 'type', 'number', 'reference', 'comment', 'date_due', 'partner_id', 'address_contact_id', 'address_invoice_id', 'partner_contact', 'partner_insite', 'partner_ref', 'payment_term', 'account_id', 'currency_id', 'invoice_line', 'tax_line', 'journal_id'])
 
 		new_ids = []
@@ -634,44 +634,58 @@ class account_invoice(osv.osv):
 			tax_lines = self.pool.get('account.invoice.tax').read(cr, uid, invoice['tax_line'])
 			tax_lines = filter(lambda l: l['manual'], tax_lines)
 			tax_lines = self._refund_cleanup_lines(tax_lines)
-
+			if not date :
+				date = time.strftime('%Y-%m-%d')
 			invoice.update({
 				'type': type_dict[invoice['type']],
-				'date_invoice': time.strftime('%Y-%m-%d'),
+				'date_invoice': date,
 				'state': 'draft',
 				'number': False,
 				'invoice_line': invoice_lines,
 				'tax_line': tax_lines
 			})
-
+			if period_id :
+				invoice.update({
+					'period_id': period_id,
+				})
+			if description :
+				invoice.update({
+					'name': description,
+				})
 			# take the id part of the tuple returned for many2one fields
 			for field in ('address_contact_id', 'address_invoice_id', 'partner_id',
 					'account_id', 'currency_id', 'payment_term', 'journal_id'):
 				invoice[field] = invoice[field] and invoice[field][0]
-
 			# create the new invoice
 			new_ids.append(self.create(cr, uid, invoice))
 		return new_ids
 
-	def pay_and_reconcile(self, cr, uid, ids, pay_amount, pay_account_id, period_id, pay_journal_id, writeoff_acc_id, writeoff_period_id, writeoff_journal_id, context={}):
+	def pay_and_reconcile(self, cr, uid, ids, pay_amount, pay_account_id, period_id, pay_journal_id, writeoff_acc_id, writeoff_period_id, writeoff_journal_id, context={}, name=''):
 		#TODO check if we can use different period for payment and the writeoff line
 		assert len(ids)==1, "Can only pay one invoice at a time"
 		invoice = self.browse(cr, uid, ids[0])
 		src_account_id = invoice.account_id.id
 		journal = self.pool.get('account.journal').browse(cr, uid, pay_journal_id)
+		# Take the seq as name for move
 		if journal.sequence_id:
-			name = self.pool.get('ir.sequence').get_id(cr, uid, journal.sequence_id.id)
+			seq = self.pool.get('ir.sequence').get_id(cr, uid, journal.sequence_id.id)
 		else:
 			raise osv.except_osv('No piece number !', 'Can not create an automatic sequence for this piece !\n\nPut a sequence in the journal definition for automatic numbering or create a sequence manually for this piece.')
 		types = {'out_invoice': -1, 'in_invoice': 1, 'out_refund': 1, 'in_refund': -1}
 		direction = types[invoice.type]
+		#take the choosen date
+		if context.has_key('date_p') and context['date_p']:
+			date=context['date_p']
+		else:
+			date=time.strftime('%Y-%m-%d')
 		l1 = {
 			'name': name,
 			'debit': direction * pay_amount>0 and direction * pay_amount,
 			'credit': direction * pay_amount<0 and - direction * pay_amount,
 			'account_id': src_account_id,
 			'partner_id': invoice.partner_id.id,
-			'date': time.strftime('%Y-%m-%d'),
+			'date': date,
+			'ref':invoice.number,
 		}
 		l2 = {
 			'name':name,
@@ -679,10 +693,12 @@ class account_invoice(osv.osv):
 			'credit': direction * pay_amount>0 and direction * pay_amount,
 			'account_id': pay_account_id,
 			'partner_id': invoice.partner_id.id,
-			'date': time.strftime('%Y-%m-%d'),
+			'date': date,
+			'ref':invoice.number,
 		}
+
 		lines = [(0, 0, l1), (0, 0, l2)]
-		move = {'name': name, 'line_id': lines, 'journal_id': pay_journal_id, 'period_id': period_id}
+		move = {'name': seq, 'line_id': lines, 'journal_id': pay_journal_id, 'period_id': period_id}
 		move_id = self.pool.get('account.move').create(cr, uid, move)
 
 		line_ids = []
@@ -827,12 +843,10 @@ class account_invoice_line(osv.osv):
 
 				if inv.type in ('out_invoice', 'in_invoice'):
 					tax_code_id = tax['base_code_id']
-					tax_amount = tax['price_unit'] * \
-							line['quantity'] * tax['base_sign']
+					tax_amount = line.price_subtotal * tax['base_sign']
 				else:
 					tax_code_id = tax['ref_base_code_id']
-					tax_amount = tax['price_unit'] * \
-							line['quantity'] * tax['ref_base_sign']
+					tax_amount = line.price_subtotal * tax['ref_base_sign']
 
 				if tax_code_found:
 					if not tax_code_id:
