@@ -42,8 +42,8 @@ class journal_print(report_sxw.rml_parse):
 	def _sum_lines(self, journal_id):
 		self.cr.execute('select sum(amount) from account_analytic_line where journal_id=%d', (journal_id,))
 		return self.cr.fetchone()[0] or 0.0
-	def __init__(self, name, table, rml):
-		super(journal_print, self).__init__(name, table, rml)
+	def __init__(self, cr, uid, name, context):
+		super(journal_print, self).__init__(cr, uid, name, context)
 		self.localcontext = {
 			'time': time,
 			'lines': self.lines,
