@@ -30,5 +30,6 @@ import wizard_account_analytic_balance_report
 import wizard_account_analytic_inverted_balance_report
 import wizard_account_analytic_cost_ledger_report
 import wizard_account_analytic_cost_ledger_for_journal_report
-import wizard_account_analytic_year_to_date_check
-
+import wizard_account_analytic_analytic_check
+import wizard_account_analytic_budget_spread
+import wizard_account_analytic_budget_report
