@@ -31,8 +31,8 @@ import general_journal
 import account_journal
 import account_balance
 import partner_balance
-import grand_livre
-import grand_livre_tiers
+import general_ledger
+import third_party_ledger
 import invoice
 import rappel
 import aged_trial_balance
