@@ -25,73 +25,65 @@
 #
 ##############################################################################
 
-import pooler
 import time
 from report import report_sxw
 
 class account_balance(report_sxw.rml_parse):
 	def __init__(self, cr, uid, name, context):
 		super(account_balance, self).__init__(cr, uid, name, context)
+		self.sum_debit = 0.0
+		self.sum_credit = 0.0
 		self.localcontext.update({
 			'time': time,
 			'lines': self.lines,
 			'sum_debit': self._sum_debit,
 			'sum_credit': self._sum_credit,
-			'sum_sdebit': self._sum_sdebit,
-			'sum_scredit': self._sum_scredit
 		})
 		self.context = context
 
-	def lines(self, ids=None, done=None, level=0):
-		ids = ids or self.ids
-		done = done or {}
-		if not self.ids:
+	def lines(self, form, ids={}, done=None, level=1):
+		if not ids:
+			ids = self.ids
+		if not ids:
 			return []
+		if not done:
+			done={}
 		result = []
-		for account in self.pool.get('account.account').browse(self.cr, self.uid, ids, self.context):
+		ctx = self.context.copy()
+		ctx['fiscalyear'] = form['fiscalyear']
+		ctx['periods'] = form['periods'][0][2]
+		accounts = self.pool.get('account.account').browse(self.cr, self.uid, ids, ctx)
+		def cmp_code(x, y):
+			return cmp(x.code, y.code)
+		accounts.sort(cmp_code)
+		for account in accounts:
 			if account.id in done:
 				continue
 			done[account.id] = 1
 			res = {
 				'code': account.code,
 				'name': account.name,
+				'level': level,
 				'debit': account.debit,
 				'credit': account.credit,
-				'level': level,
-				'sdebit': account.debit > account.credit and account.debit - account.credit,
-				'scredit': account.debit < account.credit and account.credit - account.debit,
 				'balance': account.balance
 			}
+			self.sum_debit += account.debit
+			self.sum_credit += account.credit
 			if not (res['credit'] or res['debit']) and not account.child_id:
 				continue
 			result.append(res)
-			ids2 = [(x.code,x.id) for x in account.child_id]
-			ids2.sort()
-			result += self.lines([x[1] for x in ids2], done, level+1)
-		self.ids = done
+			if account.child_id:
+				ids2 = [(x.code,x.id) for x in account.child_id]
+				ids2.sort()
+				result += self.lines(form, [x[1] for x in ids2], done, level+1)
 		return result
 
-	def _sum_debit(self):
-		if not self.ids:
-			return 0.0
-
-		self.cr.execute('select sum(debit) from account_move_line where account_id in (' + ','.join(map(str, self.ids)) + ') and date>=%s and date<=%s and state<>\'draft\'', (self.datas['form']['date1'], self.datas['form']['date2']))
-		return self.cr.fetchone()[0] or 0.0
-
 	def _sum_credit(self):
-		if not self.ids:
-			return 0.0
+		return self.sum_credit
 
-		self.cr.execute('select sum(credit) from account_move_line where account_id in (' + ','.join(map(str, self.ids)) + ') and date>=%s and date<=%s and state<>\'draft\'', (self.datas['form']['date1'], self.datas['form']['date2']))
-		return self.cr.fetchone()[0] or 0.0
-	
-	def _sum_sdebit(self):
-		debit, credit = self._sum_debit(), self._sum_credit()
-		return debit > credit and debit - credit
-		
-	def _sum_scredit(self):
-		debit, credit = self._sum_debit(), self._sum_credit()
-		return credit > debit and credit - debit
-	
-report_sxw.report_sxw('report.account.account.balance', 'account.account', 'addons/account/report/account_balance.rml', parser=account_balance)
+	def _sum_debit(self):
+		return self.sum_debit
+
+report_sxw.report_sxw('report.account.account.balance', 'account.account', 'addons/account/report/account_balance.rml', parser=account_balance, header=False)
 
