@@ -46,7 +46,7 @@ class wizard_budget_spread(wizard.interface):
 	def _spread(self, cr, uid, data, context):
 		service = netsvc.LocalService("object_proxy")
 		form = data['form']
-		res = service.execute(uid, 'account.budget.post', 'spread', data['ids'], form['fiscalyear'], form['quantity'], form['amount'])
+		res = service.execute(cr.dbname, uid, 'account.budget.post', 'spread', data['ids'], form['fiscalyear'], form['quantity'], form['amount'])
 		return {}
 		
 	states = {
