@@ -70,6 +70,11 @@ def _trans_rec_reconcile(self, cr, uid, data, context):
 	return {}
 
 def _wo_check(self, cr, uid, data, context):
+	pool = pooler.get_pool(cr.dbname)
+	line = pool.get('account.move.line').browse(cr, uid, data['ids'][0], context)
+	account = line.account_id
+	if account.currency_id:
+		return 'addendum'
 	if data['form']['writeoff'] == 0:
 		return 'reconcile'
 	return 'addendum'
