@@ -151,6 +151,8 @@ class account_analytic_account(osv.osv):
 						and account_analytic_journal.type='general' \
 					GROUP BY account_analytic_line.account_id"%acc_set)
 			for account_id, sum in cr.fetchall():
+				if not sum:
+					sum = 0.0
 				res[account_id] = round(sum,2)
 		for obj_id in ids:
 			res.setdefault(obj_id, 0.0)
