@@ -43,10 +43,11 @@ class report_rappel(report_sxw.rml_parse):
 			'getLines' : self._lines_get,
 		})
 
-	def _ids_to_objects(self, partners_ids):
+	def _ids_to_objects(self, partners_ids):	
+		p_id = pooler.get_pool(self.cr.dbname).get('account_followup.stat').browse(self.cr, self.uid, partners_ids[0]).name
 		pool = pooler.get_pool(self.cr.dbname)
-		partners = pool.get('res.partner').browse(self.cr, self.uid, partners_ids)
-		return partners
+		partners = pool.get('res.partner').browse(self.cr, self.uid, p_id.id)
+		return [partners]
 
 	def _adr_get(self, partner, type):
 		res_partner = pooler.get_pool(self.cr.dbname).get('res.partner')
