@@ -46,8 +46,8 @@ def search_entries(self, cr, uid, data, context):
 	payment = order_obj.browse(cr, uid, data['id'],
 			context=context)
 	ctx = ''
-	if payment.mode:
-		ctx = '''context="{'journal_id': %d}"''' % payment.mode.journal.id
+	# if payment.mode:
+	# 	ctx = '''context="{'journal_id': %d}"''' % payment.mode.journal.id
 
 	# Search for move line to pay:
 	line_ids = line_obj.search(cr, uid, [
@@ -57,7 +57,8 @@ def search_entries(self, cr, uid, data, context):
 	FORM.string = '''<?xml version="1.0"?>
 <form string="Populate Payment:">
 	<field name="entries" colspan="4" height="300" width="800" nolabel="1"
-		domain="[('id', 'in', [%s])]" %s/>
+		domain="[('id', 'in', [%s])]" %s>
+	</field>
 </form>''' % (','.join([str(x) for x in line_ids]), ctx)
 	return {}
 
