@@ -23,14 +23,18 @@
 		"module/module_wizard.xml",
 		"module/module_view.xml",
 		"module/module_report.xml",
+#		"module/move_module_wizard.xml",
 		"res/res_request_view.xml",
 		"res/partner/partner_report.xml",
 		"res/partner/partner_view.xml",
 		"res/partner/partner_wizard.xml",
+		"res/bank_view.xml",
+		"res/country_view.xml",
 		"res/res_currency_view.xml",
 		"res/partner/crm_view.xml",
 		"res/partner/partner_data.xml",
 		"res/ir_property_view.xml",
 	],
-	"installable": True
+	"active": True,
+	"installable": True,
 }
