@@ -57,7 +57,7 @@ class ir_model_fields(osv.osv):
 # faire une jointure a chaque fois qu'on recherche vu que le client ne connait que le nom
 # de l'objet et pas son id
 		'relation': fields.char('Model Relation', size=64),
-		'model_id': fields.many2one('ir.model', 'Model id', required=True, select=True),
+		'model_id': fields.many2one('ir.model', 'Model id', required=True, select=True, ondelete='cascade'),
 # in fact, this is the field label
 		'field_description': fields.char('Field Description', size=256),
 		'ttype': fields.char('Field Type', size=64),
@@ -254,6 +254,13 @@ class ir_model_data(osv.osv):
 		if xml_id:
 			if res_id:
 				self.loads[(module, xml_id)] = (model, res_id)
+				if model_obj._inherits:
+					for table in model_obj._inherits:
+						inherit_field = model_obj._inherits[table]
+						inherit_id = model_obj.read(cr, uid, res_id,
+								[inherit_field])[inherit_field]
+						self.loads[(module, xml_id + '_' + \
+								table.replace('.', '_'))] = (table, inherit_id)
 		return res_id
 
 	def _unlink(self, cr, uid, model, ids, direct=False):
