@@ -64,7 +64,7 @@ class ir_sequence(osv.osv):
 		return (s or '') % {'year':time.strftime('%Y'), 'month': time.strftime('%m'), 'day':time.strftime('%d')}
 
 	def get_id(self, cr, uid, sequence_id, test='id=%d'):
-		cr.execute('lock table ir_sequence')
+		cr.execute('lock table ir_sequence') # Check if we can remove this
 		cr.execute('select id,number_next,number_increment,prefix,suffix,padding from ir_sequence where '+test+' and active=True', (sequence_id,))
 		res = cr.dictfetchone()
 		if res:
