@@ -33,6 +33,7 @@ import tools
 from osv import fields, osv, orm
 import zipfile
 import release
+import zipimport
 
 ver_regexp = re.compile("^(\\d+)((\\.\\d+)*)([a-z]?)((_(pre|p|beta|alpha|rc)\\d*)*)(-r(\\d+))?$")
 suffix_regexp = re.compile("^(alpha|beta|rc|pre|p)(\\d*)$")
@@ -499,6 +500,9 @@ class module(osv.osv):
 				[]))
 			self._update_category(cr, uid, mod.id, terp.get('category',
 				'Uncategorized'))
+			# Import module
+			zimp = zipimport.zipimporter(fname)
+			zimp.load_module(mod.name)
 		return res
 
 	def _update_dependencies(self, cr, uid, id, depends=[]):
@@ -542,7 +546,7 @@ class module_dependency(osv.osv):
 
 	_columns = {
 		'name': fields.char('Name',  size=128),
-		'module_id': fields.many2one('ir.module.module', 'Module', select=True),
+		'module_id': fields.many2one('ir.module.module', 'Module', select=True, ondelete='cascade'),
 		'state': fields.function(_state, method=True, type='selection', selection=[
 			('uninstallable','Uninstallable'),
 			('uninstalled','Not Installed'),
