@@ -56,5 +56,7 @@ class ir_module_reference_print(report_sxw.rml_parse):
 		res = modobj.fields_get(self.cr, self.uid).items()
 		return res
 
-report_sxw.report_sxw('report.ir.module.reference', 'ir.module.module', 'addons/base/module/report/ir_module_reference.rml', parser=ir_module_reference_print)
+report_sxw.report_sxw('report.ir.module.reference', 'ir.module.module',
+		'addons/base/module/report/ir_module_reference.rml',
+		parser=ir_module_reference_print, header=False)
 
