@@ -28,6 +28,8 @@
 
 import tools
 
+import country
+import bank
 import partner
 import res_currency
 import res_company
