@@ -32,7 +32,7 @@ import netsvc
 class wizard_clear_ids(wizard.interface):
 	def _clear_ids(self, cr, uid, data, context):
 		service = netsvc.LocalService("object_proxy")
-		service.execute(uid, 'res.partner', 'write', data['ids'], {'ref': False})
+		service.execute(cr.dbname, uid, 'res.partner', 'write', data['ids'], {'ref': False})
 		return {}
 		
 	states = {
