@@ -37,6 +37,11 @@ from tools.misc import currency
 import mx.DateTime
 from mx.DateTime import RelativeDateTime, now, DateTime, localtime
 
+import decimal
+
+ctx = decimal.getcontext()
+ctx.rounding=decimal.ROUND_HALF_UP
+
 class res_currency(osv.osv):
 	def _current_rate(self, cr, uid, ids, name, arg, context={}):
 		res={}
@@ -70,7 +75,16 @@ class res_currency(osv.osv):
 	_order = "code"
 
 	def round(self, cr, uid, currency, amount):
-		return round(amount / currency.rounding) * currency.rounding
+		if currency.rounding == 0.0:
+		    return 0.0
+		try:
+			# python >= 2.4
+			import decimal
+			ctx = decimal.getcontext()
+			ctx.rounding=decimal.ROUND_HALF_UP
+			return float(decimal.Decimal(str(amount / currency.rounding)).to_integral()) * currency.rounding
+		except:
+			return round(amount / currency.rounding) * currency.rounding
 
 	def is_zero(self, cr, uid, currency, amount):
 		return abs(self.round(cr, uid, currency, amount)) < currency.rounding
