@@ -33,7 +33,32 @@ class groups(osv.osv):
 	_name = "res.groups"
 	_columns = {
 		'name': fields.char('Group Name', size=64, required=True),
+		'model_access': fields.one2many('ir.model.access', 'group_id', 'Access Controls'),
+		'rule_groups': fields.many2many('ir.rule.group', 'group_rule_group_rel',
+			'group_id', 'rule_group_id', 'Rules', domain="[('global', '<>', True)]"),
+		'menu_access': fields.many2many('ir.ui.menu', 'ir_ui_menu_group_rel', 'gid', 'menu_id', 'Access Menu'),
 	}
+	_sql_constraints = [
+		('name_uniq', 'unique (name)', 'The name of the group must be unique !')
+	]
+
+	def write(self, cr, uid, ids, vals, context=None):
+		if 'name' in vals:
+			if vals['name'].startswith('-'):
+				raise osv.except_osv('Error',
+						'The name of the group can not start with "-"')
+		res = super(groups, self).write(cr, uid, ids, vals, context=context)
+		# Restart the cache on the company_get method
+		self.pool.get('ir.rule').domain_get()
+		return res
+
+	def create(self, cr, uid, vals, context=None):
+		if 'name' in vals:
+			if vals['name'].startswith('-'):
+				raise osv.except_osv('Error',
+						'The name of the group can not start with "-"')
+		return super(groups, self).create(cr, uid, vals, context=context)
+
 groups()
 
 
@@ -52,7 +77,7 @@ class roles(osv.osv):
 		cr.execute('select parent_id from res_roles where id=%d', (role_id,))
 		roles = cr.fetchone()[0]
 		if roles:
-			return self.check(cr, uid, ids, roles[0])
+			return self.check(cr, uid, ids, roles)
 		return False
 roles()
 
@@ -63,10 +88,12 @@ class lang(osv.osv):
 		'code': fields.char('Code', size=5, required=True),
 		'translatable': fields.boolean('Translatable'),
 		'active': fields.boolean('Active'),
+		'direction': fields.selection([('ltr', 'Left-to-right'), ('rtl', 'Right-to-left')], 'Direction',resuired=True),
 	}
 	_defaults = {
 		'active': lambda *a: 1,
 		'translatable': lambda *a: 0,
+		'direction': lambda *a: 'ltr',
 	}
 lang()
 
@@ -80,10 +107,12 @@ class users(osv.osv):
 		'signature': fields.text('Signature', size=64),
 		'address_id': fields.many2one('res.partner.address', 'Address'),
 		'active': fields.boolean('Active'),
-		'action_id': fields.many2one('ir.actions.actions', 'Action'),
-		'groups_id': fields.many2many('res.groups', 'res_groups_users_rel', 'uid', 'gid', 'groups'),
+		'action_id': fields.many2one('ir.actions.actions', 'Home Action'),
+		'menu_id': fields.many2one('ir.actions.actions', 'Menu Action'),
+		'groups_id': fields.many2many('res.groups', 'res_groups_users_rel', 'uid', 'gid', 'Groups'),
 		'roles_id': fields.many2many('res.roles', 'res_roles_users_rel', 'uid', 'rid', 'Roles'),
 		'company_id': fields.many2one('res.company', 'Company'),
+		'rule_groups': fields.many2many('ir.rule.group', 'user_rule_group_rel', 'user_id', 'rule_group_id', 'Rules', domain="[('global', '<>', True)]"),
 	}
 	_sql_constraints = [
 		('login_key', 'UNIQUE (login)', 'You can not have two users with the same login !')
@@ -99,8 +128,9 @@ class users(osv.osv):
 
 	def write(self, cr, uid, *args, **argv):
 		res = super(users, self).write(cr, uid, *args, **argv)
-		# Restart the cache on the company_get method
 		self.company_get()
+		# Restart the cache on the company_get method
+		self.pool.get('ir.rule').domain_get()
 		return res
 
 	def unlink(self, cr, uid, ids):
@@ -108,10 +138,16 @@ class users(osv.osv):
 			raise osv.except_osv('Can not remove root user !', 'You can not remove the root user as it is used internally for resources created by Tiny ERP (updates, module installation, ...)')
 		return super(users, self).unlink(cr, uid, ids)
 		
-	def name_search(self, cr, user, name, args=[], operator='ilike', context={}):
-		ids = self.search(cr, user, [('login','=',name)]+ args)
+	def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=80):
+		if not args:
+			args=[]
+		if not context:
+			context={}
+		ids = []
+		if name:
+			ids = self.search(cr, user, [('login','=',name)]+ args, limit=limit)
 		if not ids:
-			ids = self.search(cr, user, [('name',operator,name)]+ args)
+			ids = self.search(cr, user, [('name',operator,name)]+ args, limit=limit)
 		return self.name_get(cr, user, ids)
 		
 	def copy(self, cr, uid, id, default=None, context={}):
@@ -120,3 +156,10 @@ class users(osv.osv):
 		return super(users, self).copy(cr, uid, id, default, context)
 users()
 
+class groups2(osv.osv):
+	_inherit = 'res.groups'
+	_columns = {
+		'users': fields.many2many('res.users', 'res_groups_users_rel', 'gid', 'uid', 'Users'),
+	}
+groups2()
+
