@@ -274,7 +274,7 @@ class crm_case(osv.osv):
 		'date': fields.datetime('Date'),
 		'create_date': fields.datetime('Created' ,readonly=True),
 		'date_deadline': fields.date('Deadline'),
-		'date_closed': fields.datetime('Closed', readonly=True),
+		'date_closed': fields.datetime('Closed'),
 		'canal_id': fields.many2one('res.partner.canal', 'Channel'),
 		'user_id': fields.many2one('res.users', 'User Responsible'),
 		'history_line': fields.one2many('crm.case.history', 'case_id', 'Communication'),
