@@ -7,8 +7,10 @@
 	"author" : "Tiny",
 	"category" : "Generic Modules/Others",
 	"website": "http://www.tinyerp.com",
+	"description": "Sample custom module where you can put your customer specific developments.",
 	"depends" : ["base"],
 	"init_xml" : [],
 	"update_xml" : ["custom_view.xml"],
-	"active": False
+	"active": False,
+	"installable": True
 }
