@@ -9,14 +9,14 @@ class sale_order(osv.osv):
 	_description = "Sale Order"
 
 	_columns = {
- 		'carrier_id':fields.many2one("delivery.carrier","Carrier"),
+ 		'carrier_id':fields.many2one("delivery.carrier","Delivery method", help="Complete this field if you plan to invoice the shipping based on packings made."),
 	}
 
 	def onchange_partner_id(self, cr, uid, ids, part):
 		result = super(sale_order, self).onchange_partner_id(cr, uid, ids, part)
 		if part:
-			dtype = self.pool.get('res.partner').browse(cr, uid, part).property_delivery_carrier
-			result['value']['carrier_id'] = dtype and dtype[0]
+			dtype = self.pool.get('res.partner').browse(cr, uid, part).property_delivery_carrier.id
+			result['value']['carrier_id'] = dtype
 		return result
 
 	def action_ship_create(self, cr, uid, ids, *args):
