@@ -28,8 +28,7 @@
 import time
 import wizard
 import ir
-import pooler 
-from osv.osv import except_osv
+import pooler
 
 delivery_form = """<?xml version="1.0"?>
 <form string="Create deliveries">
@@ -48,7 +47,7 @@ def _delivery_default(self, cr, uid, data, context):
 	if not order.state in ('draft'):
 		raise wizard.except_wizard('Order not in draft state !', 'The order state have to be draft to add delivery lines.')
 
-	carrier_id = order.partner_id.property_delivery_carrier
+	carrier_id = order.partner_id.property_delivery_carrier.id
 	return {'carrier_id': carrier_id}
 
 def _delivery_set(self, cr, uid, data, context):
@@ -56,27 +55,19 @@ def _delivery_set(self, cr, uid, data, context):
 	line_obj = pooler.get_pool(cr.dbname).get('sale.order.line')
 	order_objs = order_obj.browse(cr, uid, data['ids'], context)
 
-	for order in order_objs:		
-		dt = time.strftime('%Y-%m-%d')
-		for line in order.order_line:
-			if line.date_planned > dt:
-				dt = line.date_planned
-
+	for order in order_objs:
 		grid_id = pooler.get_pool(cr.dbname).get('delivery.carrier').grid_get(cr, uid, [data['form']['carrier_id']],order.partner_shipping_id.id)
-		if not grid_id: 
-			raise except_osv('No grid avaible !', 'No grid matching for this carrier !')
+		if not grid_id:
+			raise wizard.except_wizard('No grid avaible !', 'No grid matching for this carrier !')
 		grid = pooler.get_pool(cr.dbname).get('delivery.grid').browse(cr, uid, [grid_id])[0]
 
-
 		line_obj.create(cr, uid, {
 			'order_id': order.id,
 			'name': grid.carrier_id.name,
 			'product_uom_qty': 1,
 			'product_uom': grid.carrier_id.product_id.uom_id.id,
 			'product_id': grid.carrier_id.product_id.id,
-#			'price_unit': pooler.get_pool(cr.dbname).get('delivery.grid').get_price(cr, uid, order, data, dt),
-			'price_unit': grid.get_price(cr, uid, grid.id, order, dt, context), 
-			'date_planned': dt,
+			'price_unit': grid.get_price(cr, uid, grid.id, order, time.strftime('%Y-%m-%d'), context), 
 			'tax_id': [(6,0,[ x.id for x in grid.carrier_id.product_id.taxes_id])],
 			'type': 'make_to_stock'
 			})
