@@ -25,6 +25,8 @@
 #
 ##############################################################################
 
-from sign_in_out import wiz_si_so
 from print_byweek import wiz_byweek
 from print_bymonth import wiz_bymonth
+import print_attendance_error
+
+import sign_in_out
