@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2004-2006 TINY SPRL. (http://tiny.be) All Rights Reserved.
+# Copyright (c) 2004-2007 TINY SPRL. (http://tiny.be) All Rights Reserved.
 #
 # $Id: account.py 1005 2005-07-25 08:41:42Z nicoe $
 #
@@ -46,42 +46,58 @@ class invoice_create(wizard.interface):
 		account_ids = cr.fetchall()
 		return {'accounts': [x[0] for x in account_ids]}
 
-	#
-	# Sould use method from the partner object
-	#
-	def _get_partner_address_id(self, cr, uid, partner_id, type='default'):
-		id = pooler.get_pool(cr.dbname).get('res.partner.address').search(cr, uid, [('partner_id','=',partner_id),('type','=',type)])
-		if not id:
-			return pooler.get_pool(cr.dbname).get('res.partner.address').search(cr, uid, [('partner_id','=',partner_id),('type','=','default')])[0]
-		return id[0]
-
 	def _do_create(self, cr, uid, data, context):
 		pool = pooler.get_pool(cr.dbname)
+		analytic_account_obj = pool.get('account.analytic.account')
+		res_partner_obj = pool.get('res.partner')
+		account_payment_term_obj = pool.get('account.payment.term')
+
 		account_ids = data['form']['accounts'][0][2]
 		invoices = []
-		for account in pool.get('account.analytic.account').browse(cr, uid, account_ids, context):
+		for account in analytic_account_obj.browse(cr, uid, account_ids, context):
 			partner = account.partner_id
 			if (not partner) or not (account.pricelist_id):
-				raise wizard.except_wizard('Analytic account incomplete', 'Please fill in the partner and pricelist field in the analytic account:\n%s' % (account.name,))
+				raise wizard.except_wizard('Analytic account incomplete',
+						'Please fill in the partner and pricelist field '
+						'in the analytic account:\n%s' % (account.name,))
+
+			date_due = False
+			if partner.property_payment_term:
+				pterm_list= account_payment_term_obj.compute(cr, uid,
+						partner.property_payment_term.id, value=1,
+						date_ref=time.strftime('%Y-%m-%d'))
+				if pterm_list:
+					pterm_list = [line[0] for line in pterm_list]
+					pterm_list.sort()
+					date_due = pterm_list[-1]
 
 			curr_invoice = {
 				'name': time.strftime('%D')+' - '+account.name,
 				'partner_id': account.partner_id.id,
-				'address_contact_id': self._get_partner_address_id(cr, uid, account.partner_id.id, 'contact'),
-				'address_invoice_id': self._get_partner_address_id(cr, uid, account.partner_id.id, 'invoice'),
-				'payment_term': (partner.property_payment_term and partner.property_payment_term[0]) or False,
-				'account_id': partner.property_account_receivable[0],
+				'address_contact_id': res_partner_obj.address_get(cr, uid,
+					[account.partner_id.id], adr_pref=['contact'])['contact'],
+				'address_invoice_id': res_partner_obj.address_get(cr, uid,
+					[account.partner_id.id], adr_pref=['invoice'])['invoice'],
+				'payment_term': partner.property_payment_term.id or False,
+				'account_id': partner.property_account_receivable.id,
 				'currency_id': account.pricelist_id.currency_id.id,
-				'project_id': account.id
+				'date_due': date_due,
 			}
 			last_invoice = pool.get('account.invoice').create(cr, uid, curr_invoice)
 			invoices.append(last_invoice)
 
 			context2=context.copy()
 			context2['lang'] = partner.lang
-			cr.execute("SELECT product_id,to_invoice,sum(unit_amount) FROM account_analytic_line as line WHERE account_id = %d and id IN (%s) GROUP BY product_id,to_invoice" % (account.id, ','.join(map(str,data['ids']))))
+			cr.execute("SELECT product_id, to_invoice, sum(unit_amount) " \
+					"FROM account_analytic_line as line " \
+					"WHERE account_id = %d " \
+						"AND id IN (" + ','.join([str(x) for x in data['ids']]) + ") " \
+						"AND to_invoice IS NOT NULL " \
+					"GROUP BY product_id,to_invoice", (account.id,))
 			for product_id,factor_id,qty in cr.fetchall():
 				product = pool.get('product.product').browse(cr, uid, product_id, context2)
+				if not product:
+					raise wizard.except_wizard('Error', 'At least on line have no product !')
 				factor_name = ''
 				factor = pool.get('hr_timesheet_invoice.factor').browse(cr, uid, factor_id, context2)
 				if factor.customer_name:
@@ -96,15 +112,16 @@ class invoice_create(wizard.interface):
 
 				taxes = product.taxes_id
 				taxep = account.partner_id.property_account_tax
-				if not taxep:
+				if not taxep.id:
 					tax = [x.id for x in taxes or []]
 				else:
-					tax = [taxep[0]]
-					tp = pool.get('account.tax').browse(cr, uid, taxep[0])
+					tax = [taxep.id]
 					for t in taxes:
-						if not t.tax_group==tp.tax_group:
+						if not t.tax_group==taxep.tax_group:
 							tax.append(t.id)
 
+				account_id = product.product_tmpl_id.property_account_income.id or product.categ_id.property_account_income_categ.id
+
 				curr_line = {
 					'price_unit': price,
 					'quantity': qty,
@@ -112,8 +129,11 @@ class invoice_create(wizard.interface):
 					'invoice_line_tax_id': [(6,0,tax )],
 					'invoice_id': last_invoice,
 					'name': factor_name,
-					'product_id': product_id,
-					'invoice_line_tax_id': [(6,0,tax)]
+					'product_id': data['form']['product'] or product_id,
+					'invoice_line_tax_id': [(6,0,tax)],
+					'uos_id': product.uom_id.id,
+					'account_id': account_id,
+					'account_analytic_id': account.id,
 				}
 
 				#
@@ -123,9 +143,6 @@ class invoice_create(wizard.interface):
 				line_ids = cr.dictfetchall()
 				note = []
 				for line in line_ids:
-					curr_line['uos_id'] = line['product_uom_id']
-					curr_line['account_id'] = line['general_account_id']
-
 					# set invoice_line_note
 					details = []
 					if data['form']['date']:
@@ -163,6 +180,8 @@ class invoice_create(wizard.interface):
 		<field name="price"/>
 		<separator string="Choose accounts you want to invoice" colspan="4"/>
 		<field name="accounts" colspan="4"/>
+		<separator string="Choose a product for intermediary invoice" colspan="4"/>
+		<field name="product"/>
 	</form>"""
 
 	_create_fields = {
@@ -171,6 +190,7 @@ class invoice_create(wizard.interface):
 		'time': {'string':'Time spent', 'type':'boolean'},
 		'name': {'string':'Name of entry', 'type':'boolean'},
 		'price': {'string':'Cost', 'type':'boolean'},
+		'product': {'string':'Product', 'type':'many2one', 'relation': 'product.product'},
 	}
 
 	states = {
