@@ -3,7 +3,7 @@
 	"version" : "1.1",
 	"author" : "Tiny",
 	"category" : "Localisation/Account charts",
-	"depends" : ["base", "account", "account_report", "base_vat", "base_iban",
+	"depends" : ["account", "account_report", "base_vat", "base_iban",
 		"account_chart"],
 	"init_xml" : [],
 	"demo_xml" : ["account_demo.xml","account.report.report.csv"],
