@@ -397,6 +397,10 @@ class mrp_production(osv.osv):
 	#     between the end of the picking list and the call to this function
 	def action_ready(self, cr, uid, ids):
 		self.write(cr, uid, ids, {'state':'ready'})
+		for production in self.browse(cr, uid, ids):
+			if production.move_prod_id:
+				self.pool.get('stock.move').write(cr, uid, [production.move_prod_id.id],
+						{'location_id':production.location_dest_id.id})
 		return True
 
 	#TODO Review materials in function in_prod and prod_end.
@@ -508,9 +512,6 @@ class mrp_production(osv.osv):
 				'auto_picking': True,
 			})
 			toconfirm = True
-			
-			if production.move_prod_id:
-				self.pool.get('stock.move').write(cr, uid, [production.move_prod_id.id], {'location_id':production.location_dest_id.id})
 
 			source = production.product_id.product_tmpl_id.property_stock_production.id
 			data = {
@@ -642,7 +643,7 @@ class mrp_procurement(osv.osv):
 		'product_id': fields.many2one('product.product', 'Product', required=True),
 		'product_qty': fields.float('Quantity', required=True),
 		'product_uom': fields.many2one('product.uom', 'Product UOM', required=True),
-		'move_id': fields.many2one('stock.move', 'Reservation', ondelete='cascade'),
+		'move_id': fields.many2one('stock.move', 'Reservation', ondelete='set null'),
 		'close_move': fields.boolean('Close Move at end', required=True),
 		'location_id': fields.many2one('stock.location', 'Location', required=True),
 		'procure_method': fields.selection([('make_to_stock','from stock'),('make_to_order','on order')], 'Procurement Method', states={'draft':[('readonly',False)], 'confirmed':[('readonly',False)]}, readonly=True),
@@ -692,6 +693,12 @@ class mrp_procurement(osv.osv):
 			return proc.move_id.product_uos_qty
 		return False
 
+	def _uom_compute_get(self, cr, uid, proc, context={}):
+		if proc.product_id.type=='product':
+			if proc.move_id.product_uos:
+				return proc.move_id.product_uos.id
+		return False
+
 	#
 	# Return the quantity of product shipped/produced/served, wich may be
 	# different from the planned quantity
@@ -703,6 +710,13 @@ class mrp_procurement(osv.osv):
 			result = proc.product_qty
 		return result
 
+	def uom_get(self, cr, uid, id, context=None):
+		proc = self.browse(cr, uid, id, context)
+		result = self._uom_compute_get(cr, uid, proc, context)
+		if not result:
+			result = proc.product_uom.id
+		return result
+
 	def check_produce_service(self, cr, uid, procurement, context=[]):
 		return True
 
@@ -852,7 +866,7 @@ class mrp_procurement(osv.osv):
 				'product_uom': uom_id,
 				'price_unit': price,
 				'date_planned': newdate.strftime('%Y-%m-%d'),
-				'taxes_id': [(6, 0, [x.id for x in procurement.product_id.product_tmpl_id.taxes_id])],
+				'taxes_id': [(6, 0, [x.id for x in procurement.product_id.product_tmpl_id.supplier_taxes_id])],
 				'move_dest_id': res_id,
 			}
 
