@@ -41,7 +41,7 @@ class price_type(osv.osv):
 		sale and purchase prices based on some fields of the product.
 	"""
 	def _price_field_get(self, cr, uid, context={}):
-		cr.execute('select name, field_description from ir_model_fields where model in (%s,%s) and ttype=%s', ('product.product', 'product.template', 'float'))
+		cr.execute('select name, field_description from ir_model_fields where model in (%s,%s) and ttype=%s order by name', ('product.product', 'product.template', 'float'))
 		return cr.fetchall()
 	def _get_currency(self, cr, uid, ctx):
 		comp = self.pool.get('res.users').browse(cr,uid,uid).company_id
@@ -189,22 +189,18 @@ class product_pricelist(osv.osv):
 					if partner:
 						where = [('name', '=', partner) ] 
 					sinfo = supplierinfo_obj.search(cr, uid,
-							[('product_id', '=', prod_id)] + where)
-					if not sinfo:
-						result[id] = 0
-						continue
-					cr.execute('SELECT * ' \
-							'FROM pricelist_partnerinfo ' \
-							'WHERE suppinfo_id IN (' + \
-								','.join(map(str, sinfo)) + ') ' \
-								'AND min_quantity <= %f ' \
-							'ORDER BY min_quantity DESC LIMIT 1', (qty,))
-					res = cr.dictfetchone()
-					if res:
-						result[id] = res['price']
-					else:
-						result[id] = 0
-					continue
+							[('product_id', '=', tmpl_id)] + where)
+					price = 0.0
+					if sinfo:
+						cr.execute('SELECT * ' \
+								'FROM pricelist_partnerinfo ' \
+								'WHERE suppinfo_id IN (' + \
+									','.join(map(str, sinfo)) + ') ' \
+									'AND min_quantity <= %f ' \
+								'ORDER BY min_quantity DESC LIMIT 1', (qty,))
+						res2 = cr.dictfetchone()
+						if res2:
+							price = res2['price']
 				else:
 					price_type = price_type_obj.browse(cr, uid, res['base'])
 					price = currency_obj.compute(cr, uid,
