@@ -3,7 +3,7 @@
 	"version":"1.0",
 	"author":"Tiny",
 	"category":"Profile",
-	"depends":["base", "mrp", "product", "stock", "crm", "sale", "delivery"],
+	"depends":["mrp", "crm", "sale", "delivery","board_manufacturing"],
 	"demo_xml":[],
 	"update_xml":[],
 	"active":False,
