@@ -7,8 +7,11 @@
 	"category" : "Generic Modules/Accounting",
 	"description": "A report on analytic lines, costs by products, months and accounts.",
 	"init_xml" : [],
-	"demo_xml" : [],
-	"update_xml" : ["report_analytic_line_view.xml"],
+	"demo_xml" : [
+	],
+	"update_xml" : [
+		"report_analytic_line_view.xml",
+	],
 	"active": False,
 	"installable": True
 }
