@@ -57,7 +57,7 @@ class report_sale_order_product(osv.osv):
 			create or replace view report_sale_order_product as (
 				select
 					min(l.id) as id,
-					substring(s.date_order for 7)||'-'||'01' as name,
+					to_char(s.date_order, 'YYYY-MM')||'-'||'01' as name,
 					s.state,
 					l.product_id,
 					sum(l.product_uom_qty*u.factor) as quantity,
@@ -67,7 +67,7 @@ class report_sale_order_product(osv.osv):
 				from sale_order s
 					right join sale_order_line l on (s.id=l.order_id)
 					left join product_uom u on (u.id=l.product_uom)
-				group by l.product_id, substring(s.date_order for 7),s.state
+				group by l.product_id, to_char(s.date_order, 'YYYY-MM'),s.state
 			)
 		""")
 # Done in the _auto_init
@@ -109,7 +109,7 @@ class report_sale_order_category(osv.osv):
 			create or replace view report_sale_order_category as (
 				select
 					min(l.id) as id,
-					substring(s.date_order for 7)||'-'||'01' as name,
+					to_char(s.date_order, 'YYYY-MM')||'-'||'01' as name,
 					s.state,
 					t.categ_id as category_id,
 					sum(l.product_uom_qty*u.factor) as quantity,
@@ -121,7 +121,7 @@ class report_sale_order_category(osv.osv):
 					left join product_product p on (p.id=l.product_id)
 					left join product_template t on (t.id=p.product_tmpl_id)
 					left join product_uom u on (u.id=l.product_uom)
-				group by t.categ_id, substring(s.date_order for 7),s.state
+				group by t.categ_id, to_char(s.date_order, 'YYYY-MM'),s.state
 			)
 		""")
 report_sale_order_category()
