@@ -91,7 +91,7 @@ class sale_order(osv.osv):
 
 	def _amount_total(self, cr, uid, ids, field_name, arg, context):
 		res = {}
-		untax = self._amount_untaxed(cr, uid, ids, field_name, arg, context) 
+		untax = self._amount_untaxed(cr, uid, ids, field_name, arg, context)
 		tax = self._amount_tax(cr, uid, ids, field_name, arg, context)
 		cur_obj=self.pool.get('res.currency')
 		for id in ids:
@@ -242,8 +242,8 @@ class sale_order(osv.osv):
 	def button_dummy(self, cr, uid, ids, context={}):
 		return True
 
-#FIXME: the method should return the list of invoices created (invoice_ids) 
-# and not the id of the last invoice created (res). The problem is that we 
+#FIXME: the method should return the list of invoices created (invoice_ids)
+# and not the id of the last invoice created (res). The problem is that we
 # cannot change it directly since the method is called by the sale order
 # workflow and I suppose it expects a single id...
 	def _inv_get(self, cr, uid, order, context={}):
@@ -541,11 +541,6 @@ sale_order()
 # - update it on change product and unit price
 # - use it in report if there is a uos
 class sale_order_line(osv.osv):
-	def copy(self, cr, uid, id, default=None, context={}):
-		if not default: default = {}
-		default.update( {'invoice_lines':[]})
-		return super(sale_order_line, self).copy(cr, uid, id, default, context)
-
 	def _amount_line_net(self, cr, uid, ids, field_name, arg, context):
 		res = {}
 		for line in self.browse(cr, uid, ids):
@@ -569,7 +564,7 @@ class sale_order_line(osv.osv):
 			except:
 				res[line.id] = 1
 		return res
-	
+
 	def _get_1st_packaging(self, cr, uid, context={}):
 		cr.execute('select id from product_packaging order by id asc limit 1')
 		res = cr.fetchone()
