@@ -33,16 +33,20 @@ import time
 
 def compute_burndown(cr, uid, tasks_id, date_start, date_stop):
 	latest = False
-	cr.execute('select id,create_date,state,planned_hours from project_task where id in ('+','.join(map(str,tasks_id))+') order by create_date')
-	tasks = cr.fetchall()
+	if len(tasks_id):
+		cr.execute('select id,create_date,state,planned_hours from project_task where id in ('+','.join(map(str,tasks_id))+') order by create_date')
+		tasks = cr.fetchall()
 
-	cr.execute('select w.date,w.hours from project_task_work w left join project_task t on (t.id=w.task_id) where t.id in ('+','.join(map(str,tasks_id))+') and t.state in (\'open\',\'progress\') order by date')
+		cr.execute('select w.date,w.hours from project_task_work w left join project_task t on (t.id=w.task_id) where t.id in ('+','.join(map(str,tasks_id))+') and t.state in (\'open\',\'progress\') order by date')
 
-	tasks2 = cr.fetchall()
+		tasks2 = cr.fetchall()
 
-	cr.execute('select date_close,planned_hours from project_task where id in ('+','.join(map(str,tasks_id))+') and state in (\'cancelled\',\'done\') order by date_close')
-	tasks2 += cr.fetchall()
-	tasks2.sort()
+		cr.execute('select date_close,planned_hours from project_task where id in ('+','.join(map(str,tasks_id))+') and state in (\'cancelled\',\'done\') order by date_close')
+		tasks2 += cr.fetchall()
+		tasks2.sort()
+	else:
+		tasks = []
+		tasks2 = []
 
 	current_date = date_start
 	total = 0
