@@ -28,7 +28,6 @@
 ##############################################################################
 
 import StringIO
-import sql_db
 
 from report.render import render
 from report.interface import report_int
@@ -76,10 +75,9 @@ def burndown_chart(cr, uid, tasks_id, date_start, date_stop):
 	return result
 
 class report_tasks(report_int):
-	def create(self, uid, ids, datas, context={}):
+	def create(self, cr, uid, ids, datas, context={}):
 		io = StringIO.StringIO()
 
-		cr = sql_db.db.cursor()
 		if 'date_start' not in datas:
 			cr.execute('select min(date_start) from project_task where id in ('+','.join(map(str,ids))+')')
 			dt = cr.fetchone()[0]
