@@ -27,6 +27,8 @@
 ##############################################################################
 
 from stock import *
+import partner
+import product
 
 import report
 import wizard
