@@ -29,5 +29,5 @@
 #import lot_overview
 import product_stock
 import product_location
-import stock_by_location
+import lot_location
 import ups
