@@ -32,6 +32,6 @@ import wizard_picking_make
 import wizard_replacement
 import wizard_return
 import wizard_split_lot_line
-import wizard_split_track_line
 import wizard_track_line
 import wizard_ups
+import wizard_invoice_onshipping
