@@ -29,12 +29,12 @@
 
 import wizard
 import netsvc
+import pooler
 
 import time
 from osv import osv
 from tools.misc import UpdateableStr
 
-#FIXME: this is not concurrency-safe !!!
 arch = UpdateableStr()
 fields = {}
 
@@ -44,7 +44,7 @@ def make_default(val):
 	return fct
 
 def _get_moves(self, cr, uid, data, context):
-	pick_obj = osv.osv_pools.get('stock.picking')
+	pick_obj = pooler.get_pool(cr.dbname).get('stock.picking')
 	pick = pick_obj.browse(cr, uid, [data['id']])[0]
 	res = {}
 	fields.clear()
@@ -59,25 +59,27 @@ def _get_moves(self, cr, uid, data, context):
 	return res
 
 def _split_lines(self, cr, uid, data, context):
-	move_obj = osv.osv_pools.get('stock.move')
+	move_obj = pooler.get_pool(cr.dbname).get('stock.move')
 	for move in move_obj.browse(cr, uid, data['form']['moves']):
 		quantity = data['form']['move%s' % move.id]
 		if 0 < quantity < move.product_qty:
-			new_obj = move_obj.copy(cr, uid, move.id, {'product_qty' : move.product_qty - quantity, 'state':move.state })
-			move_obj.write(cr, uid, [move.id], {'product_qty' : quantity})
+			new_qty = move.product_qty - quantity
+			new_uos_qty = new_qty / move.product_qty * move.product_uos_qty
+			new_obj = move_obj.copy(cr, uid, move.id, {'product_qty' : new_qty, 'product_uos_qty': new_uos_qty, 'state':move.state})
+			uos_qty = quantity / move.product_qty * move.product_uos_qty
+			move_obj.write(cr, uid, [move.id], {'product_qty' : quantity, 'product_uos_qty': uos_qty})
 	return {}
 
 class wizard_split_move(wizard.interface):
 	states = {
 		'init': {
 			'actions': [_get_moves],
-			'result': {'type':'form', 'arch':arch, 'fields':fields, 'state':[('end','Exit'),('split','Split')]}
+			'result': {'type':'form', 'arch':arch, 'fields':fields, 'state':[('end','Cancel'),('split','Split')]}
 		},
 		'split': {
 			'actions': [_split_lines],
 			'result': {'type':'state', 'state':'end'}
 		}
 	}
-
 wizard_split_move('stock.move.split')
 
