@@ -47,7 +47,7 @@
 #
 #
 
-from xml import dom
+from xml import dom, xpath
 from xml.parsers import expat
 import string
 import pooler
@@ -392,6 +392,15 @@ class orm(object):
 									cr.execute("UPDATE \"%s\" SET \"%s\"=NULL" % (self._table, k))
 								else:
 									cr.execute("UPDATE \"%s\" SET \"%s\"='%s'" % (self._table, k, default))
+							if isinstance(f,fields.function):
+								cr.execute('select id from '+self._table)
+								ids_lst = map(lambda x: x[0], cr.fetchall())
+								while ids_lst:
+									iids = ids_lst[:40]
+									ids_lst = ids_lst[40:]
+									res = f.get(cr, self, iids, k, 1, {})
+									for r in res.items():
+										cr.execute("UPDATE \"%s\" SET \"%s\"='%s' where id=%d"% (self._table, k, r[1],r[0]))
 
 							# and add constraints if needed
 							if isinstance(f, fields.many2one):
@@ -430,14 +439,16 @@ class orm(object):
 								# columns with the name 'type' cannot be changed for an unknown reason?!
 								if k != 'type':
 									if f_pg_size > f.size:
-										logger.notifyChannel('init', netsvc.LOG_WARNING, "column '%s' in table '%s' has changed size (DB = %d, def = %d), strings will be truncated !" % (k, self._table, f_pg_size, f.size))
-#TODO: check si y a des donnees qui vont poser probleme (select char_length(...))
-#TODO: issue a log message even if f_pg_size < f.size
-									cr.execute("ALTER TABLE \"%s\" RENAME COLUMN \"%s\" TO temp_change_size" % (self._table,k))
-									cr.execute("ALTER TABLE \"%s\" ADD COLUMN \"%s\" VARCHAR(%d)" % (self._table,k,f.size))
-									cr.execute("UPDATE \"%s\" SET \"%s\"=temp_change_size::VARCHAR(%d)" % (self._table,k,f.size))
-									cr.execute("ALTER TABLE \"%s\" DROP COLUMN temp_change_size" % (self._table,))
-									cr.commit()
+										logger.notifyChannel('init', netsvc.LOG_WARNING, "column '%s' in table '%s' has changed size (DB = %d, def = %d), DB size will be kept !" % (k, self._table, f_pg_size, f.size))
+									# If actual DB size is < than new
+									# We update varchar size, otherwise, we keep DB size
+									# to avoid truncated string...
+									if f_pg_size < f.size:
+										cr.execute("ALTER TABLE \"%s\" RENAME COLUMN \"%s\" TO temp_change_size" % (self._table,k))
+										cr.execute("ALTER TABLE \"%s\" ADD COLUMN \"%s\" VARCHAR(%d)" % (self._table,k,f.size))
+										cr.execute("UPDATE \"%s\" SET \"%s\"=temp_change_size::VARCHAR(%d)" % (self._table,k,f.size))
+										cr.execute("ALTER TABLE \"%s\" DROP COLUMN temp_change_size" % (self._table,))
+										cr.commit()
 							# if the field is required and hasn't got a NOT NULL constraint
 							if f.required and f_pg_notnull == 0:
 								# set the field to the default value if any
@@ -1418,20 +1429,25 @@ class orm(object):
 			context={}
 		def _inherit_apply(src, inherit):
 			def _find(node, node2):
-				if node.nodeType==node.ELEMENT_NODE and node.localName==node2.localName:
-					res = True
-					for attr in node2.attributes.keys():
-						if attr=='position':
-							continue
-						if node.hasAttribute(attr):
-							if node.getAttribute(attr)==node2.getAttribute(attr):
+				# Check if xpath query or normal inherit (with field matching)
+				if node2.nodeType==node2.ELEMENT_NODE and node2.localName=='xpath':
+					res = xpath.Evaluate(node2.getAttribute('expr'), node)
+					return res and res[0]
+				else:
+					if node.nodeType==node.ELEMENT_NODE and node.localName==node2.localName:
+						res = True
+						for attr in node2.attributes.keys():
+							if attr=='position':
 								continue
-						res = False
-					if res:
-						return node
-				for child in node.childNodes:
-					res = _find(child, node2)
-					if res: return res
+							if node.hasAttribute(attr):
+								if node.getAttribute(attr)==node2.getAttribute(attr):
+									continue
+							res = False
+						if res:
+							return node
+					for child in node.childNodes:
+						res = _find(child, node2)
+						if res: return res
 				return None
 
 			doc_src = dom.minidom.parseString(src)
@@ -1631,11 +1647,18 @@ class orm(object):
 			elif field._type=='one2many':
 				field_obj = self.pool.get(field._obj)
 
-				# get the ids of the records of the "distant" resource
-				ids2 = [x[0] for x in field_obj.name_search(cr, user, args[i][2], [], args[i][1])]
+				if isinstance(args[i][2], basestring):
+					# get the ids of the records of the "distant" resource
+					ids2 = [x[0] for x in field_obj.name_search(cr, user, args[i][2], [], args[i][1])]
+				else:
+					ids2 = args[i][2]
 				if not ids2:
 					args[i] = ('id','=','0')
 				else:
+					try:
+						iter(ids2)
+					except Exception, e:
+						ids2 = [ids2]
 					cr.execute('select "'+field._fields_id+'" from "'+field_obj._table+'" where id in ('+','.join(map(str,ids2))+')')
 					ids3 = [x[0] for x in cr.fetchall()]
 
