@@ -29,15 +29,19 @@ import sql_db
 import osv.osv
 import tools
 import addons
+import netsvc
 
 db_dic = {}
 pool_dic = {}
 
-def get_db_and_pool(db_name, force_demo=False, status={}, update_module=False):
+def get_db_and_pool(db_name, force_demo=False, status=None, update_module=False):
+	if not status:
+		status={}
 	if db_name in db_dic:
 		db = db_dic[db_name]
 	else:
-		print 'Pooler Connecting to', db_name
+		logger = netsvc.Logger()
+		logger.notifyChannel('pooler', netsvc.LOG_INFO, 'Connecting to %s' % (db_name))
 		db = sql_db.db_connect(db_name)
 		db_dic[db_name] = db
 	
@@ -48,10 +52,10 @@ def get_db_and_pool(db_name, force_demo=False, status={}, update_module=False):
 		pool_dic[db_name] = pool
 		addons.load_modules(db, force_demo, status, update_module)
 
-		import report
-		report.interface.register_all(db)
-
-		pool.get('ir.cron')._poolJobs(db.dbname)
+		if not update_module:
+			import report
+			report.interface.register_all(db)
+			pool.get('ir.cron')._poolJobs(db.dbname)
 	return db, pool
 
 def restart_pool(db_name, force_demo=False, update_module=False):
@@ -78,7 +82,7 @@ def get_db(db_name):
 #	print "get_db", db_name
 	return get_db_and_pool(db_name)[0]
 
-def get_pool(db_name, force_demo=False, status={}, update_module=False):
+def get_pool(db_name, force_demo=False, status=None, update_module=False):
 #	print "get_pool", db_name
 	pool = get_db_and_pool(db_name, force_demo, status, update_module)[1]
 #	addons.load_modules(db_name, False)
