@@ -60,9 +60,18 @@ class report_printscreen_list(report_int):
 		dom = minidom.parseString(view)
 		return self._parse_node(dom)
 
-	def create(self, cr, uid, ids, datas, context={}):
+	def create(self, cr, uid, ids, datas, context=None):
+		if not context:
+			context={}
 		datas['ids'] = ids
-		model = pooler.get_pool(cr.dbname).get(datas['model'])
+		pool = pooler.get_pool(cr.dbname)
+		model_id = pool.get('ir.model').search(cr, uid, [('model','=',model._name)])
+		if model_id:
+			model_desc = pool.get('ir.model').browse(cr, uid, model_id, context).name
+		else:
+			model_desc = model._description
+
+		model = pool.get(datas['model'])
 		result = model.fields_view_get(cr, uid, view_type='tree', context=context)
 
 		fields_order = self._parse_string(result['arch'])
@@ -114,7 +123,7 @@ class report_printscreen_list(report_int):
 
 		for f in fields_order:
 			field = new_doc.createElement("field")
-			field_txt = new_doc.createTextNode('%s' % (fields[f]['string'],))
+			field_txt = new_doc.createTextNode(str(fields[f]['string']))
 			field.appendChild(field_txt)
 			header.appendChild(field)
 		
@@ -139,8 +148,6 @@ class report_printscreen_list(report_int):
 			lines.appendChild(node_line)
 		new_doc.childNodes[0].appendChild(lines)
 
-		file('/tmp/terp.xml','w+').write(new_doc.toxml())
-
 		styledoc = libxml2.parseFile(os.path.join(tools.config['root_path'],'addons/base/report/custom_new.xsl'))
 		style = libxslt.parseStylesheetDoc(styledoc)
 		doc = libxml2.parseDoc(new_doc.toxml())
