@@ -175,6 +175,7 @@ class _rml_doc(object):
 		return result
 
 	def render(self, out):
+		self.canvas = canvas.Canvas(out)
 		el = self.dom.documentElement.getElementsByTagName('docinit')
 		if el:
 			self.docinit(el)
@@ -191,7 +192,6 @@ class _rml_doc(object):
 			pt_obj = _rml_template(out, el[0], self, images=self.images, path=self.path)
 			pt_obj.render(self.dom.documentElement.getElementsByTagName('story'))
 		else:
-			self.canvas = canvas.Canvas(out)
 			pd = self.dom.documentElement.getElementsByTagName('pageDrawing')[0]
 			pd_obj = _rml_canvas(self.canvas, None, self, self.images, path=self.path)
 			pd_obj.render(pd)
@@ -413,7 +413,8 @@ class _rml_flowable(object):
 					node.insertBefore(newNode, n)
 					node.removeChild(n)
 				elif n.localName == 'pageNumber':
-					rc += '<pageNumber/>'            # TODO: change this !
+					rc += str(self.doc.canvas.getPageNumber())
+					#rc += '<pageNumber/>'            # TODO: change this !
 				else:
 					#CHECKME: I wonder if this is useful since we don't stock the result. Maybe for the getName tag?
 					self._textual(n)
@@ -453,6 +454,8 @@ class _rml_flowable(object):
 			colwidths = [utils.unit_get(f.strip()) for f in node.getAttribute('colWidths').split(',')]
 		if node.hasAttribute('rowHeights'):
 			rowheights = [utils.unit_get(f.strip()) for f in node.getAttribute('rowHeights').split(',')]
+                        if len(rowheights) == 1:
+                            rowheights = rowheights[0]
 		table = platypus.LongTable(data = data, colWidths=colwidths, rowHeights=rowheights, **(utils.attr_get(node, ['splitByRow'] ,{'repeatRows':'int','repeatCols':'int'})))
 		if node.hasAttribute('style'):
 			table.setStyle(self.styles.table_styles[node.getAttribute('style')])
@@ -477,7 +480,8 @@ class _rml_flowable(object):
 	def _flowable(self, node):
 		if node.localName=='para':
 			style = self.styles.para_style_get(node)
-			return platypus.Paragraph(self._textual(node), style, **(utils.attr_get(node, [], {'bulletText':'str'})))
+			p = platypus.Paragraph(self._textual(node), style, **(utils.attr_get(node, [], {'bulletText':'str'})))
+			return p
 		elif node.localName=='barCode':
 			try:
 				from reportlab.extensions.barcode import code128
