@@ -8,20 +8,22 @@ vertical versions.
 
 Stacked and percentile bar charts to follow...
 """
-__version__=''' $Id: barcharts.py 4213 2006-10-11 07:43:32Z nicoe $ '''
+__version__=''' $Id: barcharts.py 2647 2005-07-26 13:47:51Z rgbecker $ '''
 
 import string, copy
 from types import FunctionType, StringType
 
 from reportlab.lib import colors
-from reportlab.lib.validators import isNumber, isColor, isColorOrNone, isListOfStrings, SequenceOf, isBoolean, isNoneOrShape
+from reportlab.lib.validators import isNumber, isColor, isColorOrNone, isString,\
+            isListOfStrings, SequenceOf, isBoolean, isNoneOrShape, isStringOrNone,\
+            NoneOr
+from reportlab.graphics.widgets.markers import uSymbol2Symbol, isSymbol
 from reportlab.lib.formatters import Formatter
 from reportlab.lib.attrmap import AttrMap, AttrMapValue
 from reportlab.pdfbase.pdfmetrics import stringWidth
 from reportlab.graphics.widgetbase import Widget, TypedPropertyCollection, PropHolder
 from reportlab.graphics.shapes import Line, Rect, Group, Drawing, NotImplementedError
-from reportlab.graphics.charts.axes import XCategoryAxis, YValueAxis
-from reportlab.graphics.charts.axes import YCategoryAxis, XValueAxis
+from reportlab.graphics.charts.axes import XCategoryAxis, YValueAxis, YCategoryAxis, XValueAxis
 from reportlab.graphics.charts.textlabels import BarChartLabel, NA_Label, NoneOrInstanceOfNA_Label
 from reportlab.graphics.charts.areas import PlotArea
 
@@ -31,6 +33,8 @@ class BarChartProperties(PropHolder):
         fillColor = AttrMapValue(isColorOrNone, desc='Color of the bar interior area.'),
         strokeWidth = AttrMapValue(isNumber, desc='Width of the bar border.'),
         symbol = AttrMapValue(None, desc='A widget to be used instead of a normal bar.'),
+        name = AttrMapValue(isString, desc='Text to be associated with a bar (eg seriesname)'),
+        swatchMarker = AttrMapValue(NoneOr(isSymbol), desc="None or makeMarker('Diamond') ..."),
         )
 
     def __init__(self):
@@ -61,6 +65,24 @@ class BarChart(PlotArea):
         annotations = AttrMapValue(None, desc='list of callables, will be called with self, xscale, yscale.'),
         )
 
+    def makeSwatchSample(self, rowNo, x, y, width, height):
+        baseStyle = self.bars
+        styleIdx = rowNo % len(baseStyle)
+        style = baseStyle[styleIdx]
+        strokeColor = getattr(style, 'strokeColor', getattr(baseStyle,'strokeColor',None))
+        fillColor = getattr(style, 'fillColor', getattr(baseStyle,'fillColor',None))
+        strokeDashArray = getattr(style, 'strokeDashArray', getattr(baseStyle,'strokeDashArray',None))
+        strokeWidth = getattr(style, 'strokeWidth', getattr(style, 'strokeWidth',None))
+        swatchMarker = getattr(style, 'swatchMarker', getattr(baseStyle, 'swatchMarker',None))
+        if swatchMarker:
+            return uSymbol2Symbol(swatchMarker,x+width/2.,y+height/2.,fillColor)
+        return Rect(x,y,width,height,strokeWidth=strokeWidth,strokeColor=strokeColor,
+                    strokeDashArray=strokeDashArray,fillColor=fillColor)
+
+    def getSeriesName(self,i,default=None):
+        '''return series name i or default'''
+        return getattr(self.bars[i],'name',default)
+
     def __init__(self):
         assert self.__class__.__name__ not in ('BarChart','BarChart3D'), 'Abstract Class %s Instantiated' % self.__class__.__name__
 
@@ -313,10 +335,17 @@ class BarChart(PlotArea):
 
     def _labelXY(self,label,x,y,width,height):
         'Compute x, y for a label'
+        nudge = label.nudge
+        anti = getattr(label,'boxTarget','normal')=='anti'
+        if anti: nudge = -nudge
         if self._flipXY:
-            return x + width + (width>=0 and 1 or -1) * label.nudge, y + 0.5*height
+            value = width
+            if anti: value = 0
+            return x + value + (width>=0 and 1 or -1)*nudge, y + 0.5*height
         else:
-            return x + 0.5*width, y + height + (height>=0 and 1 or -1) * label.nudge
+            value = height
+            if anti: value = 0
+            return x + 0.5*width, y + value + (height>=0 and 1 or -1)*nudge
 
     def _addBarLabel(self, g, rowNo, colNo, x, y, width, height):
         text = self._getLabelText(rowNo,colNo)
