@@ -4,7 +4,7 @@
 """
 This modules defines a very preliminary Line Chart example.
 """
-__version__=''' $Id: linecharts.py 4213 2006-10-11 07:43:32Z nicoe $ '''
+__version__=''' $Id: linecharts.py 2493 2004-12-22 16:14:25Z rgbecker $ '''
 
 import string
 from types import FunctionType, StringType
@@ -12,7 +12,7 @@ from types import FunctionType, StringType
 from reportlab.lib import colors
 from reportlab.lib.validators import isNumber, isColor, isColorOrNone, isListOfStrings, \
                                     isListOfStringsOrNone, SequenceOf, isBoolean, NoneOr, \
-                                    isListOfNumbersOrNone
+                                    isListOfNumbersOrNone, isStringOrNone
 from reportlab.lib.attrmap import *
 from reportlab.lib.formatters import Formatter
 from reportlab.graphics.widgetbase import Widget, TypedPropertyCollection, PropHolder
@@ -29,9 +29,48 @@ class LineChartProperties(PropHolder):
         strokeColor = AttrMapValue(isColorOrNone, desc='Color of a line.'),
         strokeDashArray = AttrMapValue(isListOfNumbersOrNone, desc='Dash array of a line.'),
         symbol = AttrMapValue(NoneOr(isSymbol), desc='Widget placed at data points.'),
+        shader = AttrMapValue(None, desc='Shader Class.'),
+        filler = AttrMapValue(None, desc='Filler Class.'),
+        name = AttrMapValue(isStringOrNone, desc='Name of the line.'),
         )
 
-class LineChart(PlotArea):
+class AbstractLineChart(PlotArea):
+
+    def makeSwatchSample(self,rowNo, x, y, width, height):
+        baseStyle = self.lines
+        styleIdx = rowNo % len(baseStyle)
+        style = baseStyle[styleIdx]
+        color = style.strokeColor
+        y = y+height/2.
+        if self.joinedLines:
+            dash = getattr(style, 'strokeDashArray', getattr(baseStyle,'strokeDashArray',None))
+            strokeWidth= getattr(style, 'strokeWidth', getattr(style, 'strokeWidth',None))
+            L = Line(x,y,x+width,y,strokeColor=color,strokeLineCap=0)
+            if strokeWidth: L.strokeWidth = strokeWidth
+            if dash: L.strokeDashArray = dash
+        else:
+            L = None
+
+        if hasattr(style, 'symbol'):
+            S = style.symbol
+        elif hasattr(baseStyle, 'symbol'):
+            S = baseStyle.symbol
+        else:
+            S = None
+
+        if S: S = uSymbol2Symbol(S,x+width/2.,y,color)
+        if S and L:
+            g = Group()
+            g.add(L)
+            g.add(S)
+            return g
+        return S or L
+
+    def getSeriesName(self,i,default=None):
+        '''return series name i or default'''
+        return getattr(self.lines[i],'name',default)
+
+class LineChart(AbstractLineChart):
     pass
 
 # This is conceptually similar to the VerticalBarChart.
