@@ -1,7 +1,7 @@
 #Copyright ReportLab Europe Ltd. 2000-2004
 #see license.txt for license details
 #history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/graphics/widgetbase.py
-__version__=''' $Id: widgetbase.py 4213 2006-10-11 07:43:32Z nicoe $ '''
+__version__=''' $Id: widgetbase.py 2668 2005-09-05 10:23:51Z rgbecker $ '''
 import string
 
 from reportlab.graphics import shapes
@@ -174,6 +174,9 @@ class TypedPropertyCollection(PropHolder):
     The last line should be taken as a prescription of how to
     create wedge no. 3 if one is needed; no error is raised if
     there are only two data points.
+
+    We try and make sensible use of tuple indeces.
+        line[(3,x)] is backed by line[(3,)], line[3] & line
     """
 
     def __init__(self, exampleClass):
@@ -181,6 +184,24 @@ class TypedPropertyCollection(PropHolder):
         self.__dict__['_value'] = exampleClass()
         self.__dict__['_children'] = {}
 
+    def wKlassFactory(self,Klass):
+        class WKlass(Klass):
+            def __getattr__(self,name):
+                try:
+                    return self.__class__.__bases__[0].__getattr__(self,name)
+                except:
+                    i = self._index
+                    if i:
+                        c = self._parent._children
+                        if c.has_key(i) and c[i].__dict__.has_key(name):
+                            return getattr(c[i],name)
+                        elif len(i)==1:
+                            i = i[0]
+                            if c.has_key(i) and c[i].__dict__.has_key(name):
+                                return getattr(c[i],name)
+                    return getattr(self._parent,name)
+        return WKlass
+
     def __getitem__(self, index):
         try:
             return self._children[index]
@@ -189,16 +210,7 @@ class TypedPropertyCollection(PropHolder):
             if _ItemWrapper.has_key(Klass):
                 WKlass = _ItemWrapper[Klass]
             else:
-                class WKlass(Klass):
-                    def __getattr__(self,name):
-                        try:
-                            return self.__class__.__bases__[0].__getattr__(self,name)
-                        except:
-                            if self._index and self._parent._children.has_key(self._index):
-                                if self._parent._children[self._index].__dict__.has_key(name):
-                                    return getattr(self._parent._children[self._index],name)
-                            return getattr(self._parent,name)
-                _ItemWrapper[Klass] = WKlass
+                _ItemWrapper[Klass] = WKlass = self.wKlassFactory(Klass)
 
             child = WKlass()
             child._parent = self
