@@ -1,12 +1,20 @@
-#Copyright ReportLab Europe Ltd. 2000-2004
+#Copyright ReportLab Europe Ltd. 2000-2006
 #see license.txt for license details
-#history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/lib/utils.py
-__version__=''' $Id: utils.py 4213 2006-10-11 07:43:32Z nicoe $ '''
+# $URI:$
+__version__=''' $Id: utils.py 2892 2006-05-19 14:16:02Z rgbecker $ '''
 
 import string, os, sys, imp
-from types import *
 from reportlab.lib.logger import warnOnce
+from types import *
+from rltempfile import get_rl_tempfile, get_rl_tempdir, _rl_getuid
 SeqTypes = (ListType,TupleType)
+if sys.hexversion<0x2020000:
+    def isSeqType(v):
+        return type(v) in SeqTypes
+else:
+    def isSeqType(v):
+        return isinstance(v,(tuple,list))
+
 if sys.hexversion<0x2030000:
     True = 1
     False = 0
@@ -137,17 +145,17 @@ except:
 import glob, fnmatch
 try:
     _isFSD = not __loader__
-    _archive = __loader__.archive
+    _archive = os.path.normcase(os.path.normpath(__loader__.archive))
     _archivepfx = _archive + os.sep
-    _archivedir = os.path.dirname(__loader__.archive)
+    _archivedir = os.path.dirname(_archive)
     _archivedirpfx = _archivedir + os.sep
     _archivepfxlen = len(_archivepfx)
     _archivedirpfxlen = len(_archivedirpfx)
     def __startswith_rl(fn,
-                    _archivepfx=os.path.normcase(_archivepfx),
-                    _archivedirpfx=os.path.normcase(_archivedirpfx),
-                    _archive=os.path.normcase(_archive),
-                    _archivedir=os.path.normcase(_archivedir),
+                    _archivepfx=_archivepfx,
+                    _archivedirpfx=_archivedirpfx,
+                    _archive=_archive,
+                    _archivedir=_archivedir,
                     os_path_normpath=os.path.normpath,
                     os_path_normcase=os.path.normcase,
                     os_getcwd=os.getcwd,
@@ -213,7 +221,7 @@ except ImportError:
     _tz_re = re.compile('0+$')
     del re
     def fp_str(*a):
-        if len(a)==1 and type(a[0]) in SeqTypes: a = a[0]
+        if len(a)==1 and isSeqType(a[0]): a = a[0]
         s = []
         A = s.append
         for i in a:
@@ -238,32 +246,12 @@ if ',' in fp_str(0.25):
     def fp_str(*a):
         return string.replace(apply(_FP_STR,a),',','.')
 
-_rl_tempdir=None
-def get_rl_tempdir(*subdirs):
-    global _rl_tempdir
-    if _rl_tempdir is None:
-        import tempfile
-        _rl_tempdir = os.path.join(tempfile.gettempdir(),'ReportLab_tmp%s' % (sys.platform=='unix' and `os.getuid()` or ''))
-    d = _rl_tempdir
-    if subdirs: d = os.path.join(*((d,)+subdirs))
-    try:
-        os.makedirs(d)
-    except:
-        pass
-    return d
-
-def get_rl_tempfile(fn=None):
-    if not fn:
-        import tempfile
-        fn = tempfile.mktemp()
-    return os.path.join(get_rl_tempdir(),fn)
-
 def recursiveImport(modulename, baseDir=None, noCWD=0, debug=0):
     """Dynamically imports possible packagized module, or raises ImportError"""
     normalize = lambda x: os.path.normcase(os.path.abspath(os.path.normpath(x)))
     path = map(normalize,sys.path)
     if baseDir:
-        if type(baseDir) not in SeqTypes:
+        if not isSeqType(baseDir):
             tp = [baseDir]
         else:
             tp = filter(None,list(baseDir))
@@ -502,12 +490,21 @@ def rl_get_module(name,dir):
         del om
         if f: f.close()
 
+def _isPILImage(im):
+    try:
+        from PIL.Image import Image
+        return isinstance(im,Image)
+    except ImportError:
+        return 0
+
 class ImageReader:
     "Wraps up either PIL or Java to get data from bitmaps"
     def __init__(self, fileName):
-        if not haveImages:
-            warnOnce('Imaging Library not available, unable to import bitmaps')
+        if isinstance(fileName,ImageReader):
+            self.__dict__ = fileName.__dict__   #borgize
             return
+        if not haveImages:
+            raise RuntimeError('Imaging Library not available, unable to import bitmaps')
         #start wih lots of null private fields, to be populated by
         #the relevant engine.
         self.fileName = fileName
@@ -516,15 +513,41 @@ class ImageReader:
         self._height = None
         self._transparent = None
         self._data = None
-        self.fp = open_for_read(fileName,'b')
-
-        #detect which library we are using and open the image
-        if sys.platform[0:4] == 'java':
-            from javax.imageio import ImageIO
-            self._image = ImageIO.read(self.fp)
+        if _isPILImage(fileName):
+            self._image = fileName
+            self.fp = fileName.fp
+            try:
+                self.fileName = im.fileName
+            except AttributeError:
+                self.fileName = 'PILIMAGE_%d' % id(self)
         else:
-            import PIL.Image
-            self._image = PIL.Image.open(self.fp)
+            try:
+                self.fp = open_for_read(fileName,'b')
+                #detect which library we are using and open the image
+                if sys.platform[0:4] == 'java':
+                    from javax.imageio import ImageIO
+                    self._image = ImageIO.read(self.fp)
+                else:
+                    import PIL.Image
+                    self._image = PIL.Image.open(self.fp)
+                    if self._image=='JPEG': self.jpeg_fh = self._jpeg_fp
+            except:
+                et,ev,tb = sys.exc_info()
+                if hasattr(ev,'args'):
+                    a = str(ev.args[-1])+(' fileName='+fileName)
+                    ev.args= ev.args[:-1]+(a,)
+                    raise et,ev,tb
+                else:
+                    raise
+
+
+    def _jpeg_fh(self):
+        fp = self.fp
+        fp.seek(0)
+        return fp
+
+    def jpeg_fh(self):
+        return None
 
     def getSize(self):
         if (self._width is None or self._height is None):
@@ -555,9 +578,14 @@ class ImageReader:
                     a(chr((rgb>>8)&0xff))
                     a(chr(rgb&0xff))
                 self._data = ''.join(pixels)
+                self.mode = 'RGB'
             else:
-                rgb = self._image.convert('RGB')
-                self._data = rgb.tostring()
+                im = self._image
+                mode = self.mode = im.mode
+                if mode not in ('L','RGB','CMYK'):
+                    im = im.convert('RGB')
+                    self.mode = 'RGB'
+                self._data = im.tostring()
         return self._data
 
     def getImageData(self):
@@ -581,7 +609,10 @@ class ImageReader:
 
 def getImageData(imageFileName):
     "Get width, height and RGB pixels from image file.  Wraps Java/PIL"
-    return ImageReader.getImageData(imageFileName)
+    try:
+        return imageFileName.getImageData()
+    except AttributeError:
+        return ImageReader(imageFileName).getImageData()
 
 class DebugMemo:
     '''Intended as a simple report back encapsulator
@@ -643,6 +674,7 @@ class DebugMemo:
                         'api_version': getattr(sys,'api_version','????'),
                         'version_info': getattr(sys,'version_info','????'),
                         'winver': getattr(sys,'winver','????'),
+                        'environment': os.environ,
                         })
         for M,A in (
                 (sys,('getwindowsversion','getfilesystemencoding')),
@@ -759,7 +791,7 @@ class DebugMemo:
 
 def _flatten(L,a):
     for x in L:
-        if type(x) in SeqTypes: _flatten(x,a)
+        if isSeqType(x): _flatten(x,a)
         else: a(x)
 
 def flatten(L):
@@ -774,3 +806,22 @@ def find_locals(func,depth=0):
         _ = func(sys._getframe(depth).f_locals)
         if _: return _
         depth += 1
+
+class _FmtSelfDict:
+    def __init__(self,obj,overrideArgs):
+        self.obj = obj
+        self._overrideArgs = overrideArgs
+    def __getitem__(self,k):
+        try:
+            return self._overrideArgs[k]
+        except KeyError:
+            try:
+                return self.obj.__dict__[k]
+            except KeyError:
+                return getattr(self.obj,k)
+
+class FmtSelfDict:
+    '''mixin to provide the _fmt method'''
+    def _fmt(self,fmt,**overrideArgs):
+        D = _FmtSelfDict(self, overrideArgs)
+        return fmt % D
