@@ -2,7 +2,7 @@
 #see license.txt for license details
 #history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/pdfbase/pdfmetrics.py
 #$Header $
-__version__=''' $Id: pdfmetrics.py 4213 2006-10-11 07:43:32Z nicoe $ '''
+__version__=''' $Id: pdfmetrics.py 2873 2006-05-17 10:59:59Z rgbecker $ '''
 __doc__="""
 This provides a database of font metric information and
 efines Font, Encoding and TypeFace classes aimed at end users.
@@ -23,28 +23,47 @@ import string, os
 from types import StringType, ListType, TupleType
 from reportlab.pdfbase import _fontdata
 from reportlab.lib.logger import warnOnce
-from reportlab.lib.utils import rl_isfile, open_and_read, open_and_readlines 
+from reportlab.lib.utils import rl_isfile, rl_glob, rl_isdir, open_and_read, open_and_readlines 
 from reportlab.rl_config import defaultEncoding
+import rl_codecs
 
+rl_codecs.RL_Codecs.register()
 standardFonts = _fontdata.standardFonts
 standardEncodings = _fontdata.standardEncodings
 
-_dummyEncoding=' _not an encoding_ '
-# conditional import - try both import techniques, and set a flag
-try:
-    import _rl_accel
-    try:
-        _stringWidth = _rl_accel.stringWidth
-        _rl_accel.defaultEncoding(_dummyEncoding)
-    except:
-        _stringWidth = None
-except ImportError:
-    _stringWidth = None
-
 _typefaces = {}
 _encodings = {}
 _fonts = {}
 
+def _py_unicode2T1(utext,fonts):
+    '''return a list of (font,string) pairs representing the unicode text'''
+    #print 'unicode2t1(%s, %s): %s' % (utext, fonts, type(utext))
+    #if type(utext) 
+    R = []
+    font, fonts = fonts[0], fonts[1:]
+    enc = font.encName
+    if 'UCS-2' in enc:
+        enc = 'UTF16'
+    while utext:
+        try:
+            R.append((font,utext.encode(enc)))
+            break
+        except UnicodeEncodeError, e:
+            i0, il = e.args[2:4]
+            if i0:
+                R.append((font,utext[:i0].encode(enc)))
+            if fonts:
+                R.extend(_py_unicode2T1(utext[i0:il],fonts))
+            else:
+                R.append((_notdefFont,_notdefChar*(il-i0)))
+            utext = utext[il:]
+    return R
+
+try:
+    from _rl_accel import unicode2T1
+except ImportError:
+    unicode2T1 = _py_unicode2T1
+
 class FontError(Exception):
     pass
 class FontNotFoundError(Exception):
@@ -61,7 +80,7 @@ def parseAFMFile(afmFileName):
     lines = open_and_readlines(afmFileName, 'r')
     if len(lines)<=1:
         #likely to be a MAC file
-        lines = string.split(lines,'\r')
+        if lines: lines = string.split(lines[0],'\r')
         if len(lines)<=1:
             raise ValueError, 'AFM file %s hasn\'t enough data' % afmFileName
     topLevel = {}
@@ -186,25 +205,31 @@ class TypeFace:
             warnOnce("Can't find %s for face '%s'" % (ext, self.name))
         return r
 
+def bruteForceSearchForFile(fn,searchPath=None):
+    if searchPath is None: from reportlab.rl_config import T1SearchPath as searchPath
+    if rl_isfile(fn): return fn
+    bfn = os.path.basename(fn)
+    for dirname in searchPath:
+        if not rl_isdir(dirname): continue
+        tfn = os.path.join(dirname,bfn)
+        if rl_isfile(tfn): return tfn
+    return fn
+
 def bruteForceSearchForAFM(faceName):
     """Looks in all AFM files on path for face with given name.
 
     Returns AFM file name or None.  Ouch!"""
-    import glob
     from reportlab.rl_config import T1SearchPath
 
     for dirname in T1SearchPath:
-        if not os.path.isdir(dirname):
-            continue
-        possibles = glob.glob(dirname + os.sep + '*.[aA][fF][mM]')
+        if not rl_isdir(dirname): continue
+        possibles = rl_glob(dirname + os.sep + '*.[aA][fF][mM]')
         for possible in possibles:
             (topDict, glyphDict) = parseAFMFile(possible)
             if topDict['FontName'] == faceName:
                 return possible
     return None
 
-
-
 #for faceName in standardFonts:
 #    registerTypeFace(TypeFace(faceName))
 
@@ -320,6 +345,7 @@ class Encoding:
 #for encName in standardEncodings:
 #    registerEncoding(Encoding(encName))
 
+standardT1SubstitutionFonts = []
 class Font:
     """Represents a font (i.e combination of face and encoding).
 
@@ -330,13 +356,23 @@ class Font:
     composition)"""
     def __init__(self, name, faceName, encName):
         self.fontName = name
-        self.face = getTypeFace(faceName)
+        face = self.face = getTypeFace(faceName)
         self.encoding= getEncoding(encName)
+        self.encName = encName
+        if face.builtIn and face.requiredEncoding is None:
+            _ = standardT1SubstitutionFonts
+        else:
+            _ = []
+        self.substitutionFonts = _
         self._calcWidths()
 
         # multi byte fonts do their own stringwidth calculations.
         # signal this here.
         self._multiByte = 0
+        
+
+    def __repr__(self):
+        return "<%s %s>" % (self.__class__.__name__, self.face.name)
 
     def _calcWidths(self):
         """Vector of widths for stringWidth function"""
@@ -358,16 +394,13 @@ class Font:
                         pass
         self.widths = w
 
-    if not _stringWidth:
-        def stringWidth(self, text, size):
-            """This is the "purist" approach to width.  The practical one
-            is to use the stringWidth one which may be optimized
-            in C."""
-            w = 0
-            widths = self.widths
-            for ch in text:
-                w = w + widths[ord(ch)]
-            return w * 0.001 * size
+    def _py_stringWidth(self, text, size, encoding='utf8'):
+        """This is the "purist" approach to width.  The practical approach
+        is to use the stringWidth function, which may be swapped in for one
+        written in C."""
+        if not isinstance(text,unicode): text = text.decode(encoding)
+        return sum([sum(map(f.widths.__getitem__,map(ord,t))) for f, t in unicode2T1(text,[self]+self.substitutionFonts)])*0.001*size
+    stringWidth = _py_stringWidth
 
     def _formatWidths(self):
         "returns a pretty block in PDF Array format to aid inspection"
@@ -452,6 +485,7 @@ class EmbeddedType1Face(TypeFace):
     def _loadGlyphs(self, pfbFileName):
         """Loads in binary glyph data, and finds the four length
         measurements needed for the font descriptor"""
+        pfbFileName = bruteForceSearchForFile(pfbFileName)
         assert rl_isfile(pfbFileName), 'file %s not found' % pfbFileName
         d = open_and_read(pfbFileName, 'b')
         s1, l1 = _pfbCheck(0,d,PFB_ASCII,pfbFileName)
@@ -469,6 +503,7 @@ class EmbeddedType1Face(TypeFace):
     def _loadMetrics(self, afmFileName):
         """Loads in and parses font metrics"""
         #assert os.path.isfile(afmFileName), "AFM file %s not found" % afmFileName
+        afmFileName = bruteForceSearchForFile(afmFileName)
         (topLevel, glyphData) = parseAFMFile(afmFileName)
 
         self.name = topLevel['FontName']
@@ -576,14 +611,6 @@ def registerFont(font):
         fonts.addMapping(ttname, 1, 0, font.fontName)
         fonts.addMapping(ttname, 0, 1, font.fontName)
         fonts.addMapping(ttname, 1, 1, font.fontName)
-        #cannot accelerate these yet...
-    else:
-        if _stringWidth:
-            _rl_accel.setFontInfo(string.lower(fontName),
-                                  _dummyEncoding,
-                                  font.face.ascent,
-                                  font.face.descent,
-                                  font.widths)
 
 
 def getTypeFace(faceName):
@@ -626,7 +653,19 @@ def getEncoding(encName):
         else:
             raise
 
-def getFont(fontName):
+def findFontAndRegister(fontName):
+    '''search for and register a font given it's name'''
+    #it might have a font-specific encoding e.g. Symbol
+    # or Dingbats.  If not, take the default.
+    face = getTypeFace(fontName)
+    if face.requiredEncoding:
+        font = Font(fontName, fontName, face.requiredEncoding)
+    else:
+        font = Font(fontName, fontName, defaultEncoding)
+    registerFont(font)
+    return font
+
+def _py_getFont(fontName):
     """Lazily constructs known fonts if not found.
 
     Names of form 'face-encoding' will be built if
@@ -636,15 +675,28 @@ def getFont(fontName):
     try:
         return _fonts[fontName]
     except KeyError:
-        #it might have a font-specific encoding e.g. Symbol
-        # or Dingbats.  If not, take the default.
-        face = getTypeFace(fontName)
-        if face.requiredEncoding:
-            font = Font(fontName, fontName, face.requiredEncoding)
-        else:
-            font = Font(fontName, fontName, defaultEncoding)
-        registerFont(font)
-        return font
+        return findFontAndRegister(fontName)
+
+try:
+    from _rl_accel import getFontU as getFont
+except ImportError:
+    getFont = _py_getFont
+
+_notdefFont,_notdefChar = getFont('ZapfDingbats'),chr(110)
+standardT1SubstitutionFonts.extend([getFont('Symbol'),getFont('ZapfDingbats')])
+
+def getAscentDescent(fontName):
+    font = getFont(fontName)
+    try:
+        return font.ascent,font.descent
+    except:
+        return font.face.ascent,font.face.descent
+
+def getAscent(fontName):
+    return getAscentDescent(fontName)[0]
+
+def getDescent(fontName):
+    return getAscentDescent(fontName)[1]
 
 def getRegisteredFontNames():
     "Returns what's in there"
@@ -652,47 +704,21 @@ def getRegisteredFontNames():
     reg.sort()
     return reg
 
-def _slowStringWidth(text, fontName, fontSize):
+def _py_stringWidth(text, fontName, fontSize, encoding='utf8'):
     """Define this anyway so it can be tested, but whether it is used or not depends on _rl_accel"""
-    font = getFont(fontName)
-    return font.stringWidth(text, fontSize)
-    #this is faster, but will need more special-casing for multi-byte fonts.
-    #wid = getFont(fontName).widths
-    #w = 0
-    #for ch in text:
-    #    w = w + wid[ord(ch)]
-    #return 0.001 * w * fontSize
+    return getFont(fontName).stringWidth(text, fontSize, encoding=encoding)
 
+try:
+    from _rl_accel import stringWidthU as stringWidth
+except ImportError:
+    stringWidth = _py_stringWidth
 
-if _stringWidth:
+try:
+    from _rl_accel import _instanceStringWidthU
     import new
-    Font.stringWidth = new.instancemethod(_rl_accel._instanceStringWidth,None,Font)
-    stringWidth = _stringWidth
-
-    #if accelerator present, make sure we at least
-    #register Courier font, since it will fall back to Courier
-    #as its default font.
-    f = getFont('Courier')
-
-
-    def _SWRecover(text, fontName, fontSize, encoding):
-        '''This is called when _rl_accel's database doesn't know about a font.
-        Currently encoding is always a dummy.
-        '''
-        try:
-            font = getFont(fontName)
-            if font._multiByte:
-                return font.stringWidth(text, fontSize)
-            else:
-                registerFont(font)
-                return _stringWidth(text,fontName,fontSize,encoding)
-        except:
-            warnOnce('Font %s:%s not found - using Courier:%s for widths'%(fontName,encoding,encoding))
-            return _stringWidth(text,'courier',fontSize,encoding)
-
-    _rl_accel._SWRecover(_SWRecover)
-else:
-    stringWidth = _slowStringWidth
+    Font.stringWidth = new.instancemethod(_instanceStringWidthU,None,Font)
+except ImportError:
+    pass
 
 def dumpFontData():
     print 'Registered Encodings:'
@@ -717,17 +743,15 @@ def dumpFontData():
         font = _fonts[key]
         print '    %s (%s/%s)' % (font.fontName, font.face.name, font.encoding.name)
 
-
-
 def test3widths(texts):
     # checks all 3 algorithms give same answer, note speed
     import time
     for fontName in standardFonts[0:1]:
-        t0 = time.time()
-        for text in texts:
-            l1 = _stringWidth(text, fontName, 10)
-        t1 = time.time()
-        print 'fast stringWidth took %0.4f' % (t1 - t0)
+##        t0 = time.time()
+##        for text in texts:
+##            l1 = stringWidth(text, fontName, 10)
+##        t1 = time.time()
+##        print 'fast stringWidth took %0.4f' % (t1 - t0)
 
         t0 = time.time()
         w = getFont(fontName).widths
@@ -767,7 +791,6 @@ def test():
 
     dumpFontData()
 
-
 if __name__=='__main__':
     test()
     testStringWidthAlgorithms()
