@@ -57,10 +57,10 @@ Oh, and that 14 up there is font size.)
 Canvas and TextObject have special support for dynamic fonts.
 """
 
-__version__ = '$Id: ttfonts.py 4213 2006-10-11 07:43:32Z nicoe $'
+__version__ = '$Id: ttfonts.py 2865 2006-05-15 16:37:44Z rgbecker $'
 
 import string
-from types import StringType
+from types import StringType, UnicodeType
 from struct import pack, unpack
 from cStringIO import StringIO
 from reportlab.pdfbase import pdfmetrics, pdfdoc
@@ -72,6 +72,9 @@ class TTFError(pdfdoc.PDFError):
     "TrueType font exception"
     pass
 
+
+def SUBSETN(n,table=string.maketrans('0123456789','ABCDEFGHIJ')):
+    return ('%6.6d'%n).translate(table)
 #
 # Helpers
 #
@@ -102,7 +105,7 @@ def makeToUnicodeCMap(fontname, subset):
         "endcodespacerange",
         "%d beginbfchar" % len(subset)
     ] + map(lambda n, subset=subset: "<%02X> <%04X>" % (n, subset[n]),
-            range(len(subset))) + [
+            xrange(len(subset))) + [
         "endbfchar",
         "endcmap",
         "CMapName currentdict /CMap defineresource pop",
@@ -199,32 +202,51 @@ def TTFOpenFile(fn):
 
 class TTFontParser:
     "Basic TTF file parser"
+    ttfVersions = (0x00010000,0x74727565,0x74746366)
+    ttcVersions = (0x00010000,0x00020000)
+    fileKind='TTF'
 
-    def __init__(self, file, validate=0):
+    def __init__(self, file, validate=0,subfontIndex=0):
         """Loads and parses a TrueType font file.  file can be a filename or a
         file object.  If validate is set to a false values, skips checksum
         validation.  This can save time, especially if the font is large.
         """
-
-        # Open the file
-        if type(file) is StringType:
-            self.filename, file = TTFOpenFile(file)
+        self.validate = validate
+        self.readFile(file)
+        isCollection = self.readHeader()
+        if isCollection:
+            self.readTTCHeader()
+            self.getSubfont(subfontIndex)
         else:
-            self.filename = '(ttf)'
+            if self.validate: self.checksumFile()
+            self.readTableDirectory()
+            self.subfontNameX = ''
 
-        self._ttf_data = file.read()
-        self._pos = 0
+    def readTTCHeader(self):
+        self.ttcVersion = self.read_ulong()
+        self.fileKind = 'TTC'
+        self.ttfVersions = self.ttfVersions[:-1]
+        if self.ttcVersion not in self.ttcVersions: 
+            raise TTFError('"%s" is not a %s file: can\'t read version 0x%8.8x' %(self.filename,self.fileKind,self.ttcVersion))
+        self.numSubfonts = self.read_ulong()
+        self.subfontOffsets = []
+        a = self.subfontOffsets.append
+        for i in xrange(self.numSubfonts):
+            a(self.read_ulong())
 
-        # Read header
+    def getSubfont(self,subfontIndex):
+        if self.fileKind!='TTC':
+            raise TTFError('"%s" is not a TTC file: use this method' % (self.filename,self.fileKind))
         try:
-            version = self.read_ulong()
-            if version == 0x4F54544F:
-                raise TTFError, 'OpenType fonts with PostScript outlines are not supported'
-            if version != 0x00010000 and version != 0x74727565:
-                raise TTFError, 'Not a TrueType font'
-        except:
-            raise TTFError, 'Not a TrueType font'
+            pos = self.subfontOffsets[subfontIndex]
+        except IndexError:
+            raise TTFError('TTC file "%s": bad subfontIndex %s not in [0,%d]' % (self.filename,subfontIndex,self.numSubfonts-1))
+        self.seek(pos)
+        self.readHeader()
+        self.readTableDirectory()
+        self.subfontNameX = '-'+str(subfontIndex)
 
+    def readTableDirectory(self):
         try:
             self.numTables = self.read_ushort()
             self.searchRange = self.read_ushort()
@@ -234,7 +256,7 @@ class TTFontParser:
             # Read table directory
             self.table = {}
             self.tables = []
-            for n in range(self.numTables):
+            for n in xrange(self.numTables):
                 record = {}
                 record['tag'] = self.read_tag()
                 record['checksum'] = self.read_ulong()
@@ -243,16 +265,33 @@ class TTFontParser:
                 self.tables.append(record)
                 self.table[record['tag']] = record
         except:
-            raise TTFError, 'Corrupt TrueType font file'
+            raise TTFError('Corrupt %s file "%s" cannot read Table Directory' % (self.fileKind, self.filename))
+        if self.validate: self.checksumTables()
 
-        if not validate:
-            return
+    def readHeader(self):
+        '''read the sfnt header at the current position'''
+        try:
+            self.version = version = self.read_ulong()
+        except:
+            raise TTFError('"%s" is not a %s file: can\'t read version' %(self.filename,self.fileKind))
 
-        # Check the checksums for the whole file
-        checkSum = calcChecksum(self._ttf_data)
-        if add32(_L2U32(0xB1B0AFBAL), -checkSum) != 0:
-            raise TTFError, 'Invalid checksum %s len: %d &3: %d' % (hex32(checkSum),len(self._ttf_data),(len(self._ttf_data)&3))
+        if version==0x4F54544F:
+            raise TTFError('%s file "%s": postscript outlines are not supported'%(self.fileKind,self.filename))
 
+        if version not in self.ttfVersions:
+            raise TTFError('Not a TrueType font: version=0x%8.8X' % version)
+        return version==self.ttfVersions[-1]
+
+    def readFile(self,file):
+        if type(file) is StringType:
+            self.filename, file = TTFOpenFile(file)
+        else:
+            self.filename = '(ttf)'
+
+        self._ttf_data = file.read()
+        self._pos = 0
+
+    def checksumTables(self):
         # Check the checksums for all tables
         for t in self.tables:
             table = self.get_chunk(t['offset'], t['length'])
@@ -261,7 +300,13 @@ class TTFontParser:
                 adjustment = unpack('>l', table[8:8+4])[0]
                 checkSum = add32(checkSum, -adjustment)
             if t['checksum'] != checkSum:
-                raise TTFError, 'Invalid checksum %s table: %s' % (hex32(checkSum),t['tag'])
+                raise TTFError('TTF file "%s": invalid checksum %s table: %s' % (self.filename,hex32(checkSum),t['tag']))
+
+    def checksumFile(self):
+        # Check the checksums for the whole file
+        checkSum = calcChecksum(self._ttf_data)
+        if add32(_L2U32(0xB1B0AFBAL), -checkSum) != 0:
+            raise TTFError('TTF file "%s": invalid checksum %s len: %d &3: %d' % (self.filename,hex32(checkSum),len(self._ttf_data),(len(self._ttf_data)&3)))
 
     def get_table_pos(self, tag):
         "Returns the offset and size of a given TTF table."
@@ -285,32 +330,27 @@ class TTFontParser:
 
     def read_tag(self):
         "Read a 4-character tag"
-        self._pos = self._pos + 4
+        self._pos += 4
         return self._ttf_data[self._pos - 4:self._pos]
 
     def read_ushort(self):
         "Reads an unsigned short"
-        self._pos = self._pos + 2
-        return (ord(self._ttf_data[self._pos - 2]) << 8) + \
-               (ord(self._ttf_data[self._pos - 1]))
+        self._pos += 2
+        return unpack('>H',self._ttf_data[self._pos-2:self._pos])[0]
 
     def read_ulong(self):
         "Reads an unsigned long"
-        self._pos = self._pos + 4
+        self._pos += 4
         return unpack('>l',self._ttf_data[self._pos - 4:self._pos])[0]
 
     def read_short(self):
         "Reads a signed short"
-        us = self.read_ushort()
-        if us >= 0x8000:
-            return us - 0x10000
-        else:
-            return us
+        self._pos += 2
+        return unpack('>h',self._ttf_data[self._pos-2:self._pos])[0]
 
     def get_ushort(self, pos):
         "Return an unsigned short at given position"
-        return (ord(self._ttf_data[pos]) << 8) + \
-               (ord(self._ttf_data[pos + 1]))
+        return unpack('>H',self._ttf_data[pos:pos+2])[0]
 
     def get_ulong(self, pos):
         "Return an unsigned long at given position"
@@ -325,7 +365,6 @@ class TTFontParser:
         pos, length = self.get_table_pos(tag)
         return self._ttf_data[pos:pos+length]
 
-
 class TTFontMaker:
     "Basic TTF file generator"
 
@@ -384,14 +423,14 @@ class TTFontMaker:
 class TTFontFile(TTFontParser):
     "TTF file parser and generator"
 
-    def __init__(self, file, charInfo=1, validate=0):
+    def __init__(self, file, charInfo=1, validate=0,subfontIndex=0):
         """Loads and parses a TrueType font file.
 
         file can be a filename or a file object.  If validate is set to a false
         values, skips checksum validation.  This can save time, especially if
         the font is large.  See TTFontFile.extractInfo for more information.
         """
-        TTFontParser.__init__(self, file, validate=validate)
+        TTFontParser.__init__(self, file, validate=validate,subfontIndex=subfontIndex)
         self.extractInfo(charInfo)
 
     def extractInfo(self, charInfo=1):
@@ -425,7 +464,7 @@ class TTFontFile(TTFontParser):
         names = {1:None,2:None,3:None,4:None,6:None}
         K = names.keys()
         nameCount = len(names)
-        for i in range(numRecords):
+        for i in xrange(numRecords):
             platformId = self.read_ushort()
             encodingId = self.read_ushort()
             languageId = self.read_ushort()
@@ -435,17 +474,21 @@ class TTFontFile(TTFontParser):
             if nameId not in K: continue
             N = None
             if platformId == 3 and encodingId == 1 and languageId == 0x409: # Microsoft, Unicode, US English, PS Name
-                self.seek(string_data_offset + offset)
-                if length % 2 != 0:
-                    raise TTFError, "PostScript name is UTF-16BE string of odd length"
-                length /= 2
-                N = []
-                A = N.append
-                while length > 0:
-                    char = self.read_ushort()
-                    A(chr(char))
-                    length -= 1
-                N = ''.join(N)
+                opos = self._pos
+                try:
+                    self.seek(string_data_offset + offset)
+                    if length % 2 != 0:
+                        raise TTFError, "PostScript name is UTF-16BE string of odd length"
+                    length /= 2
+                    N = []
+                    A = N.append
+                    while length > 0:
+                        char = self.read_ushort()
+                        A(chr(char))
+                        length -= 1
+                    N = ''.join(N)
+                finally:
+                    self._pos = opos
             elif platformId == 1 and encodingId == 0 and languageId == 0: # Macintosh, Roman, English, PS Name
                 # According to OpenType spec, if PS name exists, it must exist
                 # both in MS Unicode and Macintosh Roman formats.  Apparently,
@@ -501,14 +544,14 @@ class TTFontFile(TTFontParser):
             fsType = self.read_ushort()
             if fsType == 0x0002 or (fsType & 0x0300) != 0:
                 raise TTFError, 'Font does not allow subsetting/embedding (%04X)' % fsType
-            self.skip(11*2 + 10 + 4*4 + 4 + 3*2)
+            self.skip(58)   #11*2 + 10 + 4*4 + 4 + 3*2
             sTypoAscender = self.read_short()
             sTypoDescender = self.read_short()
             self.ascent = scale(sTypoAscender)      # XXX: for some reason it needs to be multiplied by 1.24--1.28
             self.descent = scale(sTypoDescender)
 
             if version > 1:
-                self.skip(3*2 + 2*4 + 2)
+                self.skip(16)   #3*2 + 2*4 + 2
                 sCapHeight = self.read_short()
                 self.capHeight = scale(sCapHeight)
             else:
@@ -536,7 +579,7 @@ class TTFontFile(TTFontParser):
             # try to remove this check altogether.
             raise TTFError, 'Unknown post table version %d.%04x' % (ver_maj, ver_min)
         self.italicAngle = self.read_short() + self.read_ushort() / 65536.0
-        self.skip(2*2)
+        self.skip(4)    #2*2
         isFixedPitch = self.read_ulong()
 
         self.flags = FF_SYMBOLIC        # All fonts that contain characters
@@ -584,7 +627,7 @@ class TTFontFile(TTFontParser):
         self.skip(2)
         cmapTableCount = self.read_ushort()
         unicode_cmap_offset = None
-        for n in range(cmapTableCount):
+        for n in xrange(cmapTableCount):
             platformID = self.read_ushort()
             encodingID = self.read_ushort()
             offset = self.read_ulong()
@@ -606,18 +649,18 @@ class TTFontFile(TTFontParser):
         self.skip(2)
         segCount = self.read_ushort() / 2
         self.skip(6)
-        endCount = map(lambda x, self=self: self.read_ushort(), range(segCount))
+        endCount = map(lambda x, self=self: self.read_ushort(), xrange(segCount))
         self.skip(2)
-        startCount = map(lambda x, self=self: self.read_ushort(), range(segCount))
-        idDelta = map(lambda x, self=self: self.read_short(), range(segCount))
+        startCount = map(lambda x, self=self: self.read_ushort(), xrange(segCount))
+        idDelta = map(lambda x, self=self: self.read_short(), xrange(segCount))
         idRangeOffset_start = self._pos
-        idRangeOffset = map(lambda x, self=self: self.read_ushort(), range(segCount))
+        idRangeOffset = map(lambda x, self=self: self.read_ushort(), xrange(segCount))
 
         # Now it gets tricky.
         glyphToChar = {}
         charToGlyph = {}
-        for n in range(segCount):
-            for unichar in range(startCount[n], endCount[n] + 1):
+        for n in xrange(segCount):
+            for unichar in xrange(startCount[n], endCount[n] + 1):
                 if idRangeOffset[n] == 0:
                     glyph = (unichar + idDelta[n]) & 0xFFFF
                 else:
@@ -643,7 +686,7 @@ class TTFontFile(TTFontParser):
         aw = None
         self.charWidths = {}
         self.hmetrics = []
-        for glyph in range(numberOfHMetrics):
+        for glyph in xrange(numberOfHMetrics):
             # advance width and left side bearing.  lsb is actually signed
             # short, but we don't need it anyway (except for subsetting)
             aw, lsb = self.read_ushort(), self.read_ushort()
@@ -654,7 +697,7 @@ class TTFontFile(TTFontParser):
             if glyphToChar.has_key(glyph):
                 for char in glyphToChar[glyph]:
                     self.charWidths[char] = aw
-        for glyph in range(numberOfHMetrics, numGlyphs):
+        for glyph in xrange(numberOfHMetrics, numGlyphs):
             # the rest of the table only lists advance left side bearings.
             # so we reuse aw set by the last iteration of the previous loop
             lsb = self.read_ushort()
@@ -667,10 +710,10 @@ class TTFontFile(TTFontParser):
         self.seek_table('loca')
         self.glyphPos = []
         if indexToLocFormat == 0:
-            for n in range(numGlyphs + 1):
+            for n in xrange(numGlyphs + 1):
                 self.glyphPos.append(self.read_ushort() << 1)
         elif indexToLocFormat == 1:
-            for n in range(numGlyphs + 1):
+            for n in xrange(numGlyphs + 1):
                 self.glyphPos.append(self.read_ulong())
         else:
             raise TTFError, 'Unknown location table format (%d)' % indexToLocFormat
@@ -774,7 +817,7 @@ class TTFontFile(TTFontParser):
 
         # hmtx - Horizontal Metrics
         hmtx = []
-        for n in range(numGlyphs):
+        for n in xrange(numGlyphs):
             originalGlyphIdx = glyphMap[n]
             aw, lsb = self.hmetrics[originalGlyphIdx]
             if n < numberOfHMetrics:
@@ -788,7 +831,7 @@ class TTFontFile(TTFontParser):
         offsets = []
         glyf = []
         pos = 0
-        for n in range(numGlyphs):
+        for n in xrange(numGlyphs):
             offsets.append(pos)
             originalGlyphIdx = glyphMap[n]
             glyphPos = self.glyphPos[originalGlyphIdx]
@@ -866,10 +909,10 @@ class TTFontFace(TTFontFile, pdfmetrics.TypeFace):
     Conceptually similar to a single byte typeface, but the glyphs are
     identified by UCS character codes instead of glyph names."""
 
-    def __init__(self, filename, validate=0):
+    def __init__(self, filename, validate=0, subfontIndex=0):
         "Loads a TrueType font from filename."
         pdfmetrics.TypeFace.__init__(self, None)
-        TTFontFile.__init__(self, filename, validate=validate)
+        TTFontFile.__init__(self, filename, validate=validate, subfontIndex=subfontIndex)
 
     def getCharWidth(self, code):
         "Returns the width of character U+<code>"
@@ -935,32 +978,41 @@ class TTFont:
         def __init__(self):
             self.assignments = {}
             self.nextCode = 0
-            self.subsets = []
             self.internalName = None
             self.frozen = 0
 
-    def __init__(self, name, filename, validate=0):
+            # Let's add the first 128 unicodes to the 0th subset, so ' '
+            # always has code 32 (for word spacing to work) and the ASCII
+            # output is readable
+            subset0 = range(128)
+            self.subsets = [subset0]
+            for n in subset0:
+                self.assignments[n] = n
+            self.nextCode = 128
+
+    def __init__(self, name, filename, validate=0, subfontIndex=0):
         """Loads a TrueType font from filename.
 
         If validate is set to a false values, skips checksum validation.  This
         can save time, especially if the font is large.
         """
         self.fontName = name
-        self.face = TTFontFace(filename, validate=validate)
+        self.face = TTFontFace(filename, validate=validate, subfontIndex=subfontIndex)
         self.encoding = TTEncoding()
         self._multiByte = 1     # We want our own stringwidth
         self._dynamicFont = 1   # We want dynamic subsetting
         self.state = {}
 
-    def stringWidth(self, text, size):
+    def _py_stringWidth(self, text, size, encoding='utf-8'):
         "Calculate text width"
-        width = self.face.getCharWidth
-        w = 0
-        for code in parse_utf8(text):
-            w = w + width(code)
-        return 0.001 * w * size
+        if type(text) is not UnicodeType:
+            text = unicode(text, encoding or 'utf-8')   # encoding defaults to utf-8
+        g = self.face.charWidths.get
+        dw = self.face.defaultWidth
+        return 0.001*size*sum([g(ord(u),dw) for u in text])
+    stringWidth = _py_stringWidth
 
-    def splitString(self, text, doc):
+    def splitString(self, text, doc, encoding='utf-8'):
         """Splits text into a number of chunks, each of which belongs to a
         single subset.  Returns a list of tuples (subset, string).  Use subset
         numbers with getSubsetInternalName.  Doc is needed for distinguishing
@@ -970,14 +1022,21 @@ class TTFont:
         curSet = -1
         cur = []
         results = []
-        for code in parse_utf8(text):
+        if type(text) is not UnicodeType:
+            text = unicode(text, encoding or 'utf-8')   # encoding defaults to utf-8
+        for code in map(ord,text):
             if state.assignments.has_key(code):
                 n = state.assignments[code]
             else:
                 if state.frozen:
                     raise pdfdoc.PDFError, "Font %s is already frozen, cannot add new character U+%04X" % (self.fontName, code)
                 n = state.nextCode
-                state.nextCode = state.nextCode + 1
+                if n & 0xFF == 32:
+                    # make code 32 always be a space character
+                    state.subsets[n >> 8].append(32)
+                    state.nextCode += 1
+                    n = state.nextCode
+                state.nextCode += 1
                 state.assignments[code] = n
                 if (n & 0xFF) == 0:
                     state.subsets.append([])
@@ -1017,10 +1076,10 @@ class TTFont:
         try: state = self.state[doc]
         except KeyError: state = self.state[doc] = TTFont.State()
         state.frozen = 1
-        for n in range(len(state.subsets)):
+        for n in xrange(len(state.subsets)):
             subset = state.subsets[n]
             internalName = self.getSubsetInternalName(n, doc)[1:]
-            baseFontName = "SUBSET+%s+%d" % (self.face.name, n)
+            baseFontName = "%s+%s%s" % (SUBSETN(n),self.face.name,self.face.subfontNameX)
 
             pdfFont = pdfdoc.PDFTrueTypeFont()
             pdfFont.__Comment__ = 'Font %s subset %d' % (self.fontName, n)
@@ -1046,3 +1105,9 @@ class TTFont:
             fontDict = doc.idToObject['BasicFonts'].dict
             fontDict[internalName] = pdfFont
         del self.state[doc]
+try:
+    from _rl_accel import _instanceStringWidthTTF
+    import new
+    TTFont.stringWidth = new.instancemethod(_instanceStringWidthTTF,None,TTFont)
+except ImportError:
+    pass
