@@ -1,7 +1,7 @@
 #Copyright ReportLab Europe Ltd. 2000-2004
 #see license.txt for license details
 #history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/pdfgen/canvas.py
-__version__=''' $Id: canvas.py 4213 2006-10-11 07:43:32Z nicoe $ '''
+__version__=''' $Id: canvas.py 2854 2006-05-10 12:57:21Z rgbecker $ '''
 __doc__="""
 The Canvas object is the primary interface for creating PDF files. See
 doc/userguide.pdf for copious examples.
@@ -10,7 +10,8 @@ ENABLE_TRACKING = 1 # turn this off to do profile testing w/o tracking
 
 import os
 import sys
-from string import join, split, strip, atoi, replace, upper
+import re
+from string import join, split, strip, atoi, replace, upper, digits
 import tempfile
 from types import *
 from math import sin, cos, tan, pi, ceil
@@ -21,10 +22,12 @@ from reportlab.pdfbase import pdfutils
 from reportlab.pdfbase import pdfdoc
 from reportlab.pdfbase import pdfmetrics
 from reportlab.pdfgen  import pdfgeom, pathobject, textobject
-from reportlab.lib.colors import Color, CMYKColor, toColor
 from reportlab.lib.utils import import_zlib
 from reportlab.lib.utils import fp_str
 
+
+digitPat = re.compile('\d')  #used in decimal alignment
+
 zlib = import_zlib()
 _SeqTypes=(TupleType,ListType)
 
@@ -60,7 +63,18 @@ else:
     def _digester(s):
         return join(map(lambda x : "%02x" % ord(x), md5.md5(s).digest()), '')
 
-class Canvas:
+def _annFormat(D,color,thickness,dashArray):
+    from reportlab.pdfbase.pdfdoc import PDFArray
+    if color:
+        D["C"] = PDFArray([color.red, color.green, color.blue])
+    border = [0,0,0]
+    if thickness:
+        border[2] = thickness
+    if dashArray:
+        border.append(PDFArray(dashArray))
+    D["Border"] = PDFArray(border)
+
+class Canvas(textobject._PDFColorSetter):
     """This class is the programmer's interface to the PDF file format.  Methods
     are (or will be) provided here to do just about everything PDF can do.
 
@@ -111,20 +125,21 @@ class Canvas:
                  pagesize=None,
                  bottomup = 1,
                  pageCompression=None,
-                 encoding = None,
                  invariant = None,
                  verbosity=0):
         """Create a canvas of a given size. etc.
+
+        You may pass a file-like object to filename as an alternative to
+        a string.
+        
         Most of the attributes are private - we will use set/get methods
         as the preferred interface.  Default page size is A4."""
         if pagesize is None: pagesize = rl_config.defaultPageSize
-        if encoding is None: encoding = rl_config.defaultEncoding
         if invariant is None: invariant = rl_config.invariant
         self._filename = filename
-        self._encodingName = encoding
-        self._doc = pdfdoc.PDFDocument(encoding,
-                                       compression=pageCompression,
-                                       invariant=invariant)
+
+        self._doc = pdfdoc.PDFDocument(compression=pageCompression,
+                                       invariant=invariant, filename=filename)
 
 
         #this only controls whether it prints 'saved ...' - 0 disables
@@ -169,6 +184,7 @@ class Canvas:
         self._y = 0
         self._fontname = 'Times-Roman'
         self._fontsize = 12
+
         self._dynamicFont = 0
         self._textMode = 0  #track if between BT/ET
         self._leading = 14.4
@@ -234,9 +250,14 @@ class Canvas:
     def setAuthor(self, author):
         """identify the author for invisible embedding inside the PDF document.
            the author annotation will appear in the the text of the file but will
-           not automatically be seen when the document is viewed."""
+           not automatically be seen when the document is viewed, but is visible
+           in document properties etc etc."""
         self._doc.setAuthor(author)
 
+    def setDateFormatter(self, dateFormatter):
+        """accepts a func(yyyy,mm,dd,hh,m,s) used to create embedded formatted date"""
+        self._doc.setDateFormatter(dateFormatter)
+
     def addOutlineEntry(self, title, key, level=0, closed=None):
         """Adds a new entry to the outline at given level.  If LEVEL not specified,
         entry goes at the top level.  If level specified, it must be
@@ -304,7 +325,7 @@ class Canvas:
           each of the string names inside must be bound to a bookmark
           before the document is generated.
         """
-        apply(self._doc.outline.setNames, (self,)+nametree)
+        self._doc.outline.setNames(*((self,)+nametree))
 
     def setTitle(self, title):
         """write a title into the PDF file that won't automatically display
@@ -394,7 +415,7 @@ class Canvas:
         return result
 
     def bookmarkPage(self, key,
-                      fitType="Fit",
+                      fit="Fit",
                       left=None,
                       top=None,
                       bottom=None,
@@ -446,30 +467,30 @@ class Canvas:
         if zoom is None:
             zoom = "null"
 
-        if fitType == "XYZ":
+        if fit == "XYZ":
             dest.xyz(left,top,zoom)
-        elif fitType == "Fit":
+        elif fit == "Fit":
             dest.fit()
-        elif fitType == "FitH":
+        elif fit == "FitH":
             dest.fith(top)
-        elif fitType == "FitV":
+        elif fit == "FitV":
             dest.fitv(left)
-        elif fitType == "FitR":
+        elif fit == "FitR":
             dest.fitr(left,bottom,right,top)
         #Do we need these (version 1.1 / Acrobat 3 versions)?
-        elif fitType == "FitB":
+        elif fit == "FitB":
             dest.fitb()
-        elif fitType == "FitBH":
+        elif fit == "FitBH":
             dest.fitbh(top)
-        elif fitType == "FitBV":
+        elif fit == "FitBV":
             dest.fitbv(left)
         else:
-            raise "Unknown Fit type %s" % (fitType,)
+            raise "Unknown Fit type %s" % (fit,)
 
         dest.setPage(pageref)
         return dest
 
-    def bookmarkHorizontalAbsolute(self, key, yhorizontal):
+    def bookmarkHorizontalAbsolute(self, key, top, left=0, fit='XYZ', **kw):
         """Bind a bookmark (destination) to the current page at a horizontal position.
            Note that the yhorizontal of the book mark is with respect to the default
            user space (where the origin is at the lower left corner of the page)
@@ -478,12 +499,12 @@ class Canvas:
            responsible for making sure the bookmark matches an appropriate item on
            the page."""
         #This method should probably be deprecated since it is just a sub-set of bookmarkPage
-        return self.bookmarkPage(key,fitType="FitH",top=yhorizontal)
+        return self.bookmarkPage(key, fit=fit, top=top, left=left, zoom=0)
 
-    def bookmarkHorizontal(self, key, relativeX, relativeY):
+    def bookmarkHorizontal(self, key, relativeX, relativeY, **kw):
         """w.r.t. the current transformation, bookmark this horizontal."""
-        (xt, yt) = self.absolutePosition(relativeX,relativeY)
-        self.bookmarkHorizontalAbsolute(key, yt)
+        (left, top) = self.absolutePosition(relativeX,relativeY)
+        self.bookmarkHorizontalAbsolute(key, top, left=left, **kw)
 
     #def _inPage0(self):  disallowed!
     #    """declare a page, enable page features"""
@@ -693,26 +714,46 @@ class Canvas:
 
         self._formsinuse.append(rawName)
 
-    def textAnnotation0(self, contents, Rect=None, addtopage=1, name=None, **kw):
-        """Experimental.
+    def _absRect(self,rect,relative=0):
+        if not rect:
+            w,h = self._pagesize
+            rect = (0,0,w,h)
+        elif relative:
+            lx, ly, ux, uy = rect
+            xll,yll = self.absolutePosition(lx,ly)
+            xur,yur = self.absolutePosition(ux, uy)
+            xul,yul = self.absolutePosition(lx, uy)
+            xlr,ylr = self.absolutePosition(ux, ly)
+            xs = xll, xur, xul, xlr
+            ys = yll, yur, yul, ylr
+            xmin, ymin = min(xs), min(ys)
+            xmax, ymax = max(xs), max(ys)
+            rect = xmin, ymin, xmax, ymax
+        return rect
+
+    def freeTextAnnotation(self, contents, DA, Rect=None, addtopage=1, name=None, relative=0, **kw):
+        """DA is the default appearance string???"""
+        Rect = self._absRect(Rect,relative)
+        self._addAnnotation(pdfdoc.FreeTextAnnotation(Rect, contents, DA, **kw), name, addtopage)
+
+    def textAnnotation(self, contents, Rect=None, addtopage=1, name=None, relative=0, **kw):
+        """Experimental, but works.
         """
-        if not Rect:
-            (w,h) = self._pagesize# default to whole page (?)
-            Rect = (0,0,w,h)
-        annotation = apply(pdfdoc.TextAnnotation, (Rect, contents), kw)
-        self._addAnnotation(annotation, name, addtopage)
+        Rect = self._absRect(Rect,relative)
+        self._addAnnotation(pdfdoc.TextAnnotation(Rect, contents, **kw), name, addtopage)
+    textAnnotation0 = textAnnotation    #deprecated
 
-    def inkAnnotation0(self, contents, InkList=None, Rect=None, addtopage=1, name=None, **kw):
+    def inkAnnotation(self, contents, InkList=None, Rect=None, addtopage=1, name=None, relative=0, **kw):
+        raise NotImplementedError
         "Experimental"
-        (w,h) = self._pagesize
-        if not Rect:
-            Rect = (0,0,w,h)
+        Rect = self._absRect(Rect,relative)
         if not InkList:
-            InkList = ( (100,100,100,h-100,w-100,h-100,w-100,100), )
-        annotation = apply(pdfdoc.InkAnnotation, (Rect, contents, InkList), kw)
-        self.addAnnotation(annotation, name, addtopage)
+            InkList = ((100,100,100,h-100,w-100,h-100,w-100,100),)
+        self._addAnnotation(pdfdoc.InkAnnotation(Rect, contents, InkList, **kw), name, addtopage)
+    inkAnnotation0 = inkAnnotation  #deprecated
 
-    def linkAbsolute(self, contents, destinationname, Rect=None, addtopage=1, name=None, **kw):
+    def linkAbsolute(self, contents, destinationname, Rect=None, addtopage=1, name=None,
+            thickness=0, color=None, dashArray=None, **kw):
         """rectangular link annotation positioned wrt the default user space.
            The identified rectangle on the page becomes a "hot link" which
            when clicked will send the viewer to the page and position identified
@@ -727,37 +768,23 @@ class Canvas:
 
            You may want to use the keyword argument Border='[0 0 0]' to
            suppress the visible rectangle around the during viewing link."""
-        destination = self._bookmarkReference(destinationname) # permitted to be undefined... must bind later...
-        (w,h) = self._pagesize
-        if not Rect:
-            Rect = (0,0,w,h)
-        kw["Rect"] = Rect
-        kw["Contents"] = contents
-        kw["Destination"] = destination
-        annotation = apply(pdfdoc.LinkAnnotation, (), kw)
-        self._addAnnotation(annotation, name, addtopage)
+        return self.linkRect(contents, destinationname, Rect, addtopage, name, relative=0,
+                thickness=thickness, color=color, dashArray=dashArray, **kw)
 
-    def linkRect(self, contents, destinationname, Rect=None, addtopage=1, name=None, **kw):
+    def linkRect(self, contents, destinationname, Rect=None, addtopage=1, name=None, relative=0,
+            thickness=0, color=None, dashArray=None, **kw):
         """rectangular link annotation w.r.t the current user transform.
            if the transform is skewed/rotated the absolute rectangle will use the max/min x/y
         """
-        if Rect is None:
-            pass # do whatever linkAbsolute does
-        else:
-            (lx, ly, ux, uy) = Rect
-            (xll,yll) = self.absolutePosition(lx,ly)
-            (xur,yur) = self.absolutePosition(ux, uy)
-            (xul,yul) = self.absolutePosition(lx, uy)
-            (xlr,ylr) = self.absolutePosition(ux, ly)
-            xs = (xll, xur, xul, xlr)
-            ys = (yll, yur, yul, ylr)
-            (xmin, ymin) = (min(xs), min(ys))
-            (xmax, ymax) = (max(xs), max(ys))
-            #(w2, h2) = (xmax-xmin, ymax-ymin)
-            Rect = (xmin, ymin, xmax, ymax)
-        return apply(self.linkAbsolute, (contents, destinationname, Rect, addtopage, name), kw)
+        destination = self._bookmarkReference(destinationname) # permitted to be undefined... must bind later...
+        Rect = self._absRect(Rect,relative)
+        kw["Rect"] = Rect
+        kw["Contents"] = contents
+        kw["Destination"] = destination
+        _annFormat(kw,color,thickness,dashArray)
+        return self._addAnnotation(pdfdoc.LinkAnnotation(**kw), name, addtopage)
 
-    def linkURL(self, url, rect, relative=0, thickness=0, color=None, dashArray=None):
+    def linkURL(self, url, rect, relative=0, thickness=0, color=None, dashArray=None, kind="URI", **kw):
         """Create a rectangular URL 'hotspot' in the given rectangle.
 
         if relative=1, this is in the current coord system, otherwise
@@ -770,42 +797,27 @@ class Canvas:
         from reportlab.pdfbase.pdfdoc import PDFDictionary, PDFName, PDFArray, PDFString
         #tried the documented BS element in the pdf spec but it
         #does not work, and Acrobat itself does not appear to use it!
-        if relative:
-            #adjust coordinates
-            (lx, ly, ux, uy) = rect
-            (xll,yll) = self.absolutePosition(lx,ly)
-            (xur,yur) = self.absolutePosition(ux, uy)
-            (xul,yul) = self.absolutePosition(lx, uy)
-            (xlr,ylr) = self.absolutePosition(ux, ly)
-            xs = (xll, xur, xul, xlr)
-            ys = (yll, yur, yul, ylr)
-            (xmin, ymin) = (min(xs), min(ys))
-            (xmax, ymax) = (max(xs), max(ys))
-            #(w2, h2) = (xmax-xmin, ymax-ymin)
-            rect = (xmin, ymin, xmax, ymax)
 
-        ann = PDFDictionary()
+        ann = PDFDictionary(dict=kw)
         ann["Type"] = PDFName("Annot")
         ann["Subtype"] = PDFName("Link")
-        ann["Rect"] = PDFArray(rect) # the whole page for testing
+        ann["Rect"] = PDFArray(self._absRect(rect,relative)) # the whole page for testing
 
         # the action is a separate dictionary
         A = PDFDictionary()
         A["Type"] = PDFName("Action") # not needed?
-        A["S"] = PDFName("URI")
-        A["URI"] = PDFString(url)
+        uri = PDFString(url)
+        A['S'] = PDFName(kind)
+        if kind=="URI":
+            A["URI"] = uri
+        elif kind=='GoToR':
+            A["F"] = uri
+            A["D"] = "[ 0 /XYZ null null null ]"
+        else:
+            raise ValueError("Unknown linkURI kind '%s'" % kind)
+
         ann["A"] = A
-
-        # now for formatting stuff.
-        if color:
-            ann["C"] = PDFArray([color.red, color.green, color.blue])
-        border = [0,0,0]
-        if thickness:
-            border[2] = thickness
-        if dashArray:
-            border.append(PDFArray(dashArray))
-        ann["Border"] = PDFArray(border)
-
+        _annFormat(ann,color,thickness,dashArray)
         self._addAnnotation(ann)
 
     def _addAnnotation(self, annotation, name=None, addtopage=1):
@@ -1148,20 +1160,6 @@ class Canvas:
         # use PDFTextObject for multi-line text.
         ##################################################
 
-    def setFillColorCMYK(self, c, m, y, k):
-         """set the fill color useing negative color values
-            (cyan, magenta, yellow and darkness value).
-         Takes 4 arguments between 0.0 and 1.0"""
-         self._fillColorCMYK = (c, m, y, k)
-         self._code.append('%s k' % fp_str(c, m, y, k))
-
-    def setStrokeColorCMYK(self, c, m, y, k):
-         """set the stroke color useing negative color values
-            (cyan, magenta, yellow and darkness value).
-            Takes 4 arguments between 0.0 and 1.0"""
-         self._strokeColorCMYK = (c, m, y, k)
-         self._code.append('%s K' % fp_str(c, m, y, k))
-
     def drawString(self, x, y, text):
         """Draws a string in the current text styles."""
         #we could inline this for speed if needed
@@ -1186,15 +1184,53 @@ class Canvas:
     def drawAlignedString(self, x, y, text, pivotChar="."):
         """Draws a string aligned on the first '.' (or other pivot character).
 
-        The centre position of the pivot character will be used as x."""
+        The centre position of the pivot character will be used as x.
+        So, you could draw a straight line down through all the decimals in a
+        column of numbers, and anything without a decimal should be
+        optically aligned with those that have.
+
+        There is one special rule to help with accounting formatting.  Here's
+        how normal numbers should be aligned on the 'dot'. Look at the
+        LAST two:
+           12,345,67
+              987.15
+               42
+           -1,234.56
+             (456.78)
+             (456)
+               27 inches
+               13cm
+        Since the last three do not contain a dot, a crude dot-finding
+        rule would place them wrong. So we test for the special case
+        where no pivot is found, digits are present, but the last character
+        is not a digit.  We then work back from the end of the string
+        This case is a tad slower but hopefully rare.
+        
+        """
         parts = text.split(pivotChar,1)
         pivW = self.stringWidth(pivotChar, self._fontname, self._fontsize)
-        leftText = parts[0]
-        self.drawRightString(x-0.5*pivW, y, leftText)
-        if len(parts) > 1:
-            rightText = pivotChar + parts[1]
+        
+        if len(parts) == 1 and digitPat.search(text) is not None and text[-1] not in digits:
+            #we have no decimal but it ends in a bracket, or 'in' or something.
+            #the cut should be after the last digit.
+            leftText = parts[0][0:-1]
+            rightText = parts[0][-1]
+            #any more?
+            while leftText[-1] not in digits:
+                rightText = leftText[-1] + rightText
+                leftText = leftText[0:-1]
+
+            self.drawRightString(x-0.5*pivW, y, leftText)
             self.drawString(x-0.5*pivW, y, rightText)
 
+        else:
+            #normal case
+            leftText = parts[0]
+            self.drawRightString(x-0.5*pivW, y, leftText)
+            if len(parts) > 1:
+                rightText = pivotChar + parts[1]
+                self.drawString(x-0.5*pivW, y, rightText)
+
     def getAvailableFonts(self):
         """Returns the list of PostScript font names available.
 
@@ -1230,17 +1266,20 @@ class Canvas:
             leading = size * 1.2
         self._leading = leading
         font = pdfmetrics.getFont(self._fontname)
+
         self._dynamicFont = getattr(font, '_dynamicFont', 0)
         if not self._dynamicFont:
             pdffontname = self._doc.getInternalFontName(psfontname)
             self._code.append('BT %s %s Tf %s TL ET' % (pdffontname, fp_str(size), fp_str(leading)))
 
-    def stringWidth(self, text, fontName, fontSize, encoding=None):
+    def setFontSize(self, size=None, leading=None):
+        '''Sets font size or leading without knowing the font face'''
+        if size is None: size = self._fontsize
+        if leading is None: leading = self._leading
+        self.setFont(self._fontname, size, leading)
+
+    def stringWidth(self, text, fontName, fontSize):
         "gets width of a string in the given font and size"
-        if encoding is not None:
-            from reportlab.lib import logger
-            logger.warnOnce('encoding argument to Canvas.stringWidth is deprecated and has no effect!')
-        #if encoding is None: encoding = self._doc.encoding
         return pdfmetrics.stringWidth(text, fontName, fontSize)
 
     # basic graphics modes
@@ -1271,82 +1310,9 @@ class Canvas:
             self._code.append('[%s %s] 0 d' % (array, phase))
         elif type(array) == ListType or type(array) == TupleType:
             assert phase >= 0, "phase is a length in user space"
-            textarray = join(map(str, array))
+            textarray = ' '.join(map(str, array))
             self._code.append('[%s] %s d' % (textarray, phase))
 
-    def setFillColorRGB(self, r, g, b):
-        """Set the fill color using positive color description
-           (Red,Green,Blue).  Takes 3 arguments between 0.0 and 1.0"""
-        self._fillColorRGB = (r, g, b)
-        self._code.append('%s rg' % fp_str(r,g,b))
-
-    def setStrokeColorRGB(self, r, g, b):
-        """Set the stroke color using positive color description
-           (Red,Green,Blue).  Takes 3 arguments between 0.0 and 1.0"""
-        self._strokeColorRGB = (r, g, b)
-        self._code.append('%s RG' % fp_str(r,g,b))
-
-    def setFillColor(self, aColor):
-        """Takes a color object, allowing colors to be referred to by name"""
-        if isinstance(aColor, CMYKColor):
-            d = aColor.density
-            c,m,y,k = (d*aColor.cyan, d*aColor.magenta, d*aColor.yellow, d*aColor.black)
-            self._fillColorCMYK = (c, m, y, k)
-            self._code.append('%s k' % fp_str(c, m, y, k))
-        elif isinstance(aColor, Color):
-            rgb = (aColor.red, aColor.green, aColor.blue)
-            self._fillColorRGB = rgb
-            self._code.append('%s rg' % fp_str(rgb) )
-        elif type(aColor) in _SeqTypes:
-            l = len(aColor)
-            if l==3:
-                self._fillColorRGB = aColor
-                self._code.append('%s rg' % fp_str(aColor) )
-            elif l==4:
-                self.setFillColorCMYK(aColor[0], aColor[1], aColor[2], aColor[3])
-            else:
-                raise 'Unknown color', str(aColor)
-        elif type(aColor) is StringType:
-            self.setFillColor(toColor(aColor))
-        else:
-            raise 'Unknown color', str(aColor)
-
-    def setStrokeColor(self, aColor):
-        """Takes a color object, allowing colors to be referred to by name"""
-        if isinstance(aColor, CMYKColor):
-            d = aColor.density
-            c,m,y,k = (d*aColor.cyan, d*aColor.magenta, d*aColor.yellow, d*aColor.black)
-            self._strokeColorCMYK = (c, m, y, k)
-            self._code.append('%s K' % fp_str(c, m, y, k))
-        elif isinstance(aColor, Color):
-        #if type(aColor) == ColorType:
-            rgb = (aColor.red, aColor.green, aColor.blue)
-            self._strokeColorRGB = rgb
-            self._code.append('%s RG' % fp_str(rgb) )
-        elif type(aColor) in _SeqTypes:
-            l = len(aColor)
-            if l==3:
-                self._strokeColorRGB = aColor
-                self._code.append('%s RG' % fp_str(aColor) )
-            elif l==4:
-                self.setStrokeColorCMYK(aColor[0], aColor[1], aColor[2], aColor[3])
-            else:
-                raise 'Unknown color', str(aColor)
-        elif type(aColor) is StringType:
-            self.setStrokeColor(toColor(aColor))
-        else:
-            raise 'Unknown color', str(aColor)
-
-    def setFillGray(self, gray):
-        """Sets the gray level; 0.0=black, 1.0=white"""
-        self._fillColorRGB = (gray, gray, gray)
-        self._code.append('%s g' % fp_str(gray))
-
-    def setStrokeGray(self, gray):
-        """Sets the gray level; 0.0=black, 1.0=white"""
-        self._strokeColorRGB = (gray, gray, gray)
-        self._code.append('%s G' % fp_str(gray))
-
     # path stuff - the separate path object builds it
 
     def beginPath(self):
@@ -1479,6 +1445,15 @@ class Canvas:
             transDict[key] = value
         self._pageTransition = transDict
 
+    def getCurrentPageContent(self):
+        """Return uncompressed contents of current page buffer.
+
+        This is useful in creating test cases and assertions of what
+        got drawn, without necessarily saving pages to disk"""
+        return '\n'.join(self._code)
+
+
+
 if _instanceEscapePDF:
     import new
     Canvas._escape = new.instancemethod(_instanceEscapePDF,None,Canvas)
