@@ -2,7 +2,7 @@
 #see license.txt for license details
 #history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/platypus/doctemplate.py
 
-__version__=''' $Id: doctemplate.py 4213 2006-10-11 07:43:32Z nicoe $ '''
+__version__=''' $Id: doctemplate.py 2852 2006-05-08 15:04:15Z rgbecker $ '''
 
 __doc__="""
 This module contains the core structure of platypus.
@@ -29,13 +29,17 @@ for the current frame).
 """
 
 from reportlab.platypus.flowables import *
+from reportlab.lib.units import inch
 from reportlab.platypus.paragraph import Paragraph
 from reportlab.platypus.frames import Frame
 from reportlab.rl_config import defaultPageSize, verbose
 import reportlab.lib.sequencer
+from reportlab.pdfgen import canvas
 
 from types import *
 import sys
+import logging
+logger = logging.getLogger("reportlab.platypus")
 
 class LayoutError(Exception):
     pass
@@ -71,25 +75,19 @@ class IndexingFlowable(Flowable):
         """Called after build ends but before isSatisfied"""
         pass
 
-
 class ActionFlowable(Flowable):
     '''This Flowable is never drawn, it can be used for data driven controls
        For example to change a page template (from one column to two, for example)
        use NextPageTemplate which creates an ActionFlowable.
     '''
     def __init__(self,action=()):
+        #must call super init to ensure it has a width and height (of zero),
+        #as in some cases the packer might get called on it...
+        Flowable.__init__(self)
         if type(action) not in (ListType, TupleType):
             action = (action,)
         self.action = tuple(action)
 
-    def wrap(self, availWidth, availHeight):
-        '''Should never be called.'''
-        raise NotImplementedError
-
-    def draw(self):
-        '''Should never be called.'''
-        raise NotImplementedError
-
     def apply(self,doc):
         '''
         This is called by the doc.build processing to allow the instance to
@@ -113,17 +111,32 @@ class ActionFlowable(Flowable):
         return self
 
     def identity(self, maxLen=None):
-        return "ActionFlowable: %s" % str(self.action)
+        return "ActionFlowable: %s%s" % (str(self.action),self._frameName())
+
+class LCActionFlowable(ActionFlowable):
+    locChanger = 1                  #we cause a frame or page change
+
+    def wrap(self, availWidth, availHeight):
+        '''Should never be called.'''
+        raise NotImplementedError
+
+    def draw(self):
+        '''Should never be called.'''
+        raise NotImplementedError
 
 class NextFrameFlowable(ActionFlowable):
     def __init__(self,ix,resume=0):
         ActionFlowable.__init__(self,('nextFrame',ix,resume))
 
-class CurrentFrameFlowable(ActionFlowable):
+class CurrentFrameFlowable(LCActionFlowable):
     def __init__(self,ix,resume=0):
         ActionFlowable.__init__(self,('currentFrame',ix,resume))
 
-class _FrameBreak(ActionFlowable):
+class NullActionFlowable(ActionFlowable):
+    def apply(self):
+        pass
+
+class _FrameBreak(LCActionFlowable):
     '''
     A special ActionFlowable that allows setting doc._nextFrameIndex
 
@@ -135,11 +148,12 @@ class _FrameBreak(ActionFlowable):
         return r
 
     def apply(self,doc):
-        if getattr(self,'_ix',None): doc._nextFrameIndex = self._ix
+        if getattr(self,'_ix',None):
+            doc.handle_nextFrame(self._ix)
         ActionFlowable.apply(self,doc)
 
 FrameBreak = _FrameBreak('frameEnd')
-PageBegin = ActionFlowable('pageBegin')
+PageBegin = LCActionFlowable('pageBegin')
 
 def _evalMeasurement(n):
     if type(n) is type(''):
@@ -148,21 +162,26 @@ def _evalMeasurement(n):
         if type(n) is type(()): n = n[1]
     return n
 
-class Indenter(ActionFlowable):
+class FrameActionFlowable(Flowable):
+    def __init__(self,*arg,**kw):
+        raise NotImplementedError('Abstract Class')
+
+    def frameAction(self,frame):
+        raise NotImplementedError('Abstract Class')
+
+class Indenter(FrameActionFlowable):
     """Increases or decreases left and right margins of frame.
 
     This allows one to have a 'context-sensitive' indentation
     and makes nested lists way easier.
     """
-
     def __init__(self, left=0, right=0):
         self.left = _evalMeasurement(left)
         self.right = _evalMeasurement(right)
 
-    def apply(self, doc):
-        doc.frame._leftExtraIndent = doc.frame._leftExtraIndent + self.left
-        doc.frame._rightExtraIndent = doc.frame._rightExtraIndent + self.right
-
+    def frameAction(self, frame):
+        frame._leftExtraIndent += self.left
+        frame._rightExtraIndent += self.right
 
 class NextPageTemplate(ActionFlowable):
     """When you get to the next page, use the template specified (change to two column, for example)  """
@@ -286,7 +305,10 @@ class BaseDocTemplate:
                     'title':None,
                     'author':None,
                     'invariant':None,
-                    '_pageBreakQuick':1}
+                    'pageCompression':None,
+                    '_pageBreakQuick':1,
+                    'rotation':0,
+                    '_debug':0}
     _invalidInitArgs = ()
     _firstPageTemplateIndex = 0
 
@@ -302,7 +324,6 @@ class BaseDocTemplate:
                     raise ValueError, "Invalid argument %s" % k
                 v = kw[k]
             setattr(self,k,v)
-        #print "pagesize is", self.pagesize
 
         p = self.pageTemplates
         self.pageTemplates = []
@@ -372,7 +393,8 @@ class BaseDocTemplate:
     def handle_pageBegin(self):
         '''Perform actions required at beginning of page.
         shouldn't normally be called directly'''
-        self.page = self.page + 1
+        self.page += 1
+        if self._debug: logger.debug("beginning page %d" % self.page)
         self.pageTemplate.beforeDrawPage(self.canv,self)
         self.pageTemplate.checkPageSize(self.canv,self)
         self.pageTemplate.onPage(self.canv,self)
@@ -383,6 +405,7 @@ class BaseDocTemplate:
         if hasattr(self,'_nextFrameIndex'):
             del self._nextFrameIndex
         self.frame = self.pageTemplate.frames[0]
+        self.frame._debug = self._debug
         self.handle_frameBegin()
 
     def handle_pageEnd(self):
@@ -392,12 +415,14 @@ class BaseDocTemplate:
         '''
         #detect infinite loops...
         if self._curPageFlowableCount == 0:
-            self._emptyPages = self._emptyPages + 1
+            self._emptyPages += 1
         else:
             self._emptyPages = 0
         if self._emptyPages >= self._emptyPagesAllowed:
             if 1:
-                raise LayoutError("More than %d pages generated without content - halting layout.  Likely that a flowable is too large for any frame." % self._emptyPagesAllowed)
+                ident = "More than %d pages generated without content - halting layout.  Likely that a flowable is too large for any frame." % self._emptyPagesAllowed
+                #leave to keep apart from the raise
+                raise LayoutError(ident)
             else:
                 pass    #attempt to restore to good state
         else:
@@ -406,8 +431,18 @@ class BaseDocTemplate:
             self.pageTemplate.afterDrawPage(self.canv, self)
             self.pageTemplate.onPageEnd(self.canv, self)
             self.afterPage()
+            if self._debug: logger.debug("ending page %d" % self.page)
+            self.canv.setPageRotation(getattr(self.pageTemplate,'rotation',self.rotation))
             self.canv.showPage()
-            if hasattr(self,'_nextPageTemplateIndex'):
+
+            if hasattr(self,'_nextPageTemplateCycle'):
+                #they are cycling through pages'; we keep the index
+                cyc = self._nextPageTemplateCycle
+                idx = self._nextPageTemplateIndex
+                self.pageTemplate = cyc[idx]  #which is one of the ones in the list anyway
+                #bump up by 1
+                self._nextPageTemplateIndex = (idx + 1) % len(cyc)
+            elif hasattr(self,'_nextPageTemplateIndex'):
                 self.pageTemplate = self.pageTemplates[self._nextPageTemplateIndex]
                 del self._nextPageTemplateIndex
             if self._emptyPages==0:
@@ -441,7 +476,8 @@ class BaseDocTemplate:
         self._rightExtraIndent = self.frame._rightExtraIndent
 
         if hasattr(self,'_nextFrameIndex'):
-            frame = self.pageTemplate.frames[self._nextFrameIndex]
+            self.frame = self.pageTemplate.frames[self._nextFrameIndex]
+            self.frame._debug = self._debug
             del self._nextFrameIndex
             self.handle_frameBegin(resume)
         elif hasattr(self.frame,'lastFrame') or self.frame is self.pageTemplate.frames[-1]:
@@ -450,23 +486,44 @@ class BaseDocTemplate:
         else:
             f = self.frame
             self.frame = self.pageTemplate.frames[self.pageTemplate.frames.index(f) + 1]
+            self.frame._debug = self._debug
             self.handle_frameBegin()
 
     def handle_nextPageTemplate(self,pt):
-        '''On endPage chenge to the page template with name or index pt'''
+        '''On endPage change to the page template with name or index pt'''
         if type(pt) is StringType:
+            if hasattr(self, '_nextPageTemplateCycle'): del self._nextPageTemplateCycle
             for t in self.pageTemplates:
                 if t.id == pt:
                     self._nextPageTemplateIndex = self.pageTemplates.index(t)
                     return
             raise ValueError, "can't find template('%s')"%pt
         elif type(pt) is IntType:
+            if hasattr(self, '_nextPageTemplateCycle'): del self._nextPageTemplateCycle
             self._nextPageTemplateIndex = pt
+        elif type(pt) in (ListType, TupleType):
+            #used for alternating left/right pages
+            #collect the refs to the template objects, complain if any are bad
+            cycle = []
+            for templateName in pt:
+                found = 0
+                for t in self.pageTemplates:
+                    if t.id == templateName:
+                        cycle.append(t)
+                        found = 1
+                if not found:
+                    raise ValueError("Cannot find page template called %s" % templateName)
+            #double-check all of them are there...
+            
+            first = cycle[0]
+            #ensure we start on the first one
+            self._nextPageTemplateCycle = cycle
+            self._nextPageTemplateIndex = 0  #indexes into the cycle
         else:
-            raise TypeError, "argument pt should be string or integer"
+            raise TypeError, "argument pt should be string or integer or list"
 
-    def handle_nextFrame(self,fx):
-        '''On endFrame chenge to the frame with name or index fx'''
+    def handle_nextFrame(self,fx,resume=0):
+        '''On endFrame change to the frame with name or index fx'''
         if type(fx) is StringType:
             for f in self.pageTemplate.frames:
                 if f.id == fx:
@@ -478,18 +535,10 @@ class BaseDocTemplate:
         else:
             raise TypeError, "argument fx should be string or integer"
 
-    def handle_currentFrame(self,fx):
-        '''chenge to the frame with name or index fx'''
-        if type(fx) is StringType:
-            for f in self.pageTemplate.frames:
-                if f.id == fx:
-                    self._nextFrameIndex = self.pageTemplate.frames.index(f)
-                    return
-            raise ValueError, "can't find frame('%s')"%fx
-        elif type(fx) is IntType:
-            self._nextFrameIndex = fx
-        else:
-            raise TypeError, "argument fx should be string or integer"
+    def handle_currentFrame(self,fx,resume=0):
+        '''change to the frame with name or index fx'''
+        self.handle_nextFrame(fx,resume)
+        self.handle_frameEnd(resume)
 
     def handle_breakBefore(self, flowables):
         '''preprocessing step to allow pageBreakBefore and frameBreakBefore attributes'''
@@ -504,7 +553,7 @@ class BaseDocTemplate:
         # in to the flowables with a defult value of 0
         if hasattr(first,'pageBreakBefore') and first.pageBreakBefore == 1:
             first._skipMeNextTime = 1
-            flowables.insert(0, PageBreak())
+            first.insert(0, PageBreak())
             return
         if hasattr(first,'style') and hasattr(first.style, 'pageBreakBefore') and first.style.pageBreakBefore == 1:
             first._skipMeNextTime = 1
@@ -519,20 +568,27 @@ class BaseDocTemplate:
             flowables.insert(0, FrameBreak())
             return
 
-
     def handle_keepWithNext(self, flowables):
         "implements keepWithNext"
         i = 0
         n = len(flowables)
-        while i<n and flowables[i].getKeepWithNext(): i = i + 1
+        while i<n and flowables[i].getKeepWithNext(): i += 1
         if i:
-            i = i + 1
+            if not getattr(flowables[i],'locChanger',None): i += 1
             K = KeepTogether(flowables[:i])
-            for f in K._flowables:
+            for f in K._content:
                 f.keepWithNext = 0
             del flowables[:i]
             flowables.insert(0,K)
 
+    def _fIdent(self,f,maxLen=None,frame=None):
+        if frame: f._frame = frame
+        try:
+            return f.identity(maxLen)
+        finally:
+            if frame: del f._frame
+
+
     def handle_flowable(self,flowables):
         '''try to handle one flowable from the front of list flowables.'''
 
@@ -543,7 +599,6 @@ class BaseDocTemplate:
         self.handle_breakBefore(flowables)
         self.handle_keepWithNext(flowables)
         f = flowables[0]
-        #print 'handling flowable %s' % f.identity()
         del flowables[0]
         if f is None:
             return
@@ -560,30 +615,32 @@ class BaseDocTemplate:
         else:
             #try to fit it then draw it
             if self.frame.add(f, self.canv, trySplit=self.allowSplitting):
-                self._curPageFlowableCount = self._curPageFlowableCount + 1
+                self._curPageFlowableCount += 1
                 self.afterFlowable(f)
             else:
-                #if isinstance(f, KeepTogether): print 'could not add it to frame'
                 if self.allowSplitting:
                     # see if this is a splittable thing
                     S = self.frame.split(f,self.canv)
-                    #print '%d parts to sequence on page %d' % (len(S), self.page)
                     n = len(S)
                 else:
                     n = 0
-                #if isinstance(f, KeepTogether): print 'n=%d' % n
                 if n:
-                    if self.frame.add(S[0], self.canv, trySplit=0):
-                        self._curPageFlowableCount = self._curPageFlowableCount + 1
-                        self.afterFlowable(S[0])
-                    else:
-                        raise LayoutError("Splitting error(n==%d) on page %d in\n%s" % (n,self.page,f.identity(30)))
-                    del S[0]
-                    for f in xrange(n-1):
-                        flowables.insert(f,S[f])    # put split flowables back on the list
+                    if not isinstance(S[0],(PageBreak,SlowPageBreak,ActionFlowable)):
+                        if self.frame.add(S[0], self.canv, trySplit=0):
+                            self._curPageFlowableCount += 1
+                            self.afterFlowable(S[0])
+                        else:
+                            ident = "Splitting error(n==%d) on page %d in\n%s" % (n,self.page,self._fIdent(f,30,self.frame))
+                            #leave to keep apart from the raise
+                            raise LayoutError(ident)
+                        del S[0]
+                    for i,f in enumerate(S):
+                        flowables.insert(i,f)   # put split flowables back on the list
                 else:
                     if hasattr(f,'_postponed'):
-                        raise LayoutError("Flowable %s too large on page %d" % (f.identity(30), self.page))
+                        ident = "Flowable %s too large on page %d" % (self._fIdent(f,30,self.frame), self.page)
+                        #leave to keep apart from the raise
+                        raise LayoutError(ident)
                     # this ought to be cleared when they are finally drawn!
                     f._postponed = 1
                     flowables.insert(0,f)           # put the flowable back
@@ -604,7 +661,8 @@ class BaseDocTemplate:
         self._calc()
         self.canv = canvasmaker(filename or self.filename,
                                 pagesize=self.pagesize,
-                                invariant=self.invariant)
+                                invariant=self.invariant,
+                                pageCompression=self.pageCompression)
         if self._onPage:
             self.canv.setPageCallBack(self._onPage)
         self.handle_documentBegin()
@@ -648,7 +706,7 @@ class BaseDocTemplate:
                     exc = sys.exc_info()[1]
                     args = list(exc.args)
                     tr = first._traceInfo
-                    args[0] = args[0] + '\n(srcFile %s, line %d char %d to line %d char %d)' % (
+                    args[0] += '\n(srcFile %s, line %d char %d to line %d char %d)' % (
                         tr.srcFile,
                         tr.startLineNo,
                         tr.startLinePos,
@@ -695,16 +753,14 @@ class BaseDocTemplate:
         for thing in story:
             if thing.isIndexing():
                 self._indexingFlowables.append(thing)
-        #print 'scanned story, found these indexing flowables:\n'
-        #print self._indexingFlowables
 
         #better fix for filename is a 'file' problem
         self._doSave = 0
         passes = 0
         while 1:
-            passes = passes + 1
+            passes += 1
             if self._onProgress:
-                self.onProgress('PASS', passes)
+                self._onProgress('PASS', passes)
             if verbose: print 'building pass '+str(passes) + '...',
 
             for fl in self._indexingFlowables:
@@ -790,7 +846,7 @@ class SimpleDocTemplate(BaseDocTemplate):
         self._handle_pageBegin()
         self._handle_nextPageTemplate('Later')
 
-    def build(self,flowables,onFirstPage=_doNothing, onLaterPages=_doNothing):
+    def build(self,flowables,onFirstPage=_doNothing, onLaterPages=_doNothing, canvasmaker=canvas.Canvas):
         """build the document using the flowables.  Annotate the first page using the onFirstPage
                function and later pages using the onLaterPages function.  The onXXX pages should follow
                the signature
@@ -811,7 +867,7 @@ class SimpleDocTemplate(BaseDocTemplate):
             self.pageTemplates[0].beforeDrawPage = self.onFirstPage
         if onLaterPages is _doNothing and hasattr(self,'onLaterPages'):
             self.pageTemplates[1].beforeDrawPage = self.onLaterPages
-        BaseDocTemplate.build(self,flowables)
+        BaseDocTemplate.build(self,flowables, canvasmaker=canvasmaker)
 
 
 def progressCB(typ, value):
