@@ -1,7 +1,7 @@
 #Copyright ReportLab Europe Ltd. 2000-2004
 #see license.txt for license details
 #history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/platypus/flowables.py
-__version__=''' $Id: flowables.py 4213 2006-10-11 07:43:32Z nicoe $ '''
+__version__=''' $Id: flowables.py 2830 2006-04-05 15:18:32Z rgbecker $ '''
 __doc__="""
 A flowable is a "floating element" in a document whose exact position is determined by the
 other elements that precede it, such as a paragraph, a diagram interspersed between paragraphs,
@@ -29,13 +29,14 @@ import string
 from copy import deepcopy
 from types import ListType, TupleType, StringType
 
-from reportlab.pdfgen import canvas
-from reportlab.lib.units import inch
 from reportlab.lib.colors import red, gray, lightgrey
+from reportlab.lib.utils import fp_str
 from reportlab.pdfbase import pdfutils
 
-from reportlab.rl_config import defaultPageSize, _FUZZ
-PAGE_HEIGHT = defaultPageSize[1]
+from reportlab.rl_config import _FUZZ, overlapAttachedSpace
+__all__=('TraceInfo','Flowable','XBox','Preformatted','Image','Spacer','PageBreak','SlowPageBreak',
+        'CondPageBreak','KeepTogether','Macro','CallerMacro','ParagraphAndImage',
+        'FailOnWrap','HRFlowable','PTOContainer','KeepInFrame','UseUpSpace')
 
 
 class TraceInfo:
@@ -75,6 +76,11 @@ class Flowable:
         self._traceInfo = None
         self._showBoundary = None
 
+        #many flowables handle text and must be processed in the
+        #absence of a canvas.  tagging them with their encoding
+        #helps us to get conversions right.  Use Python codec names.
+        self.encoding = None        
+
 
     def _drawOn(self,canv):
         '''ensure canv is set on and then draw'''
@@ -164,12 +170,17 @@ class Flowable:
         if hasattr(self, 'getPlainText'):
             r = self.getPlainText(identify=1)
         elif hasattr(self, 'text'):
-            r = self.text
+            r = str(self.text)
         else:
             r = '...'
         if r and maxLen:
             r = r[:maxLen]
-        return "<%s at %d>%s" % (self.__class__.__name__, id(self), r)
+        return "<%s at %s%s>%s" % (self.__class__.__name__, hex(id(self)), self._frameName(), r)
+
+    def _frameName(self):
+        f = getattr(self,'_frame',None)
+        if f and f.id: return ' frame=%s' % f.id
+        return ''
 
 class XBox(Flowable):
     """Example flowable - a box with an x through it and a caption.
@@ -394,19 +405,24 @@ class Spacer(Flowable):
     def draw(self):
         pass
 
-class PageBreak(Spacer):
-    """Move on to the next page in the document.
-       This works by consuming all remaining space in the frame!"""
+class UseUpSpace(Flowable):
     def __init__(self):
         pass
 
     def __repr__(self):
-        return "PageBreak()"
+        return "%s()" % self.__class__.__name__
 
     def wrap(self, availWidth, availHeight):
         self.width = availWidth
         self.height = availHeight
-        return (availWidth,availHeight)  #step back a point
+        return (availWidth,availHeight-1e-8)  #step back a point
+
+    def draw(self):
+        pass
+
+class PageBreak(UseUpSpace):
+    """Move on to the next page in the document.
+       This works by consuming all remaining space in the frame!"""
 
 class SlowPageBreak(PageBreak):
     pass
@@ -424,39 +440,61 @@ class CondPageBreak(Spacer):
             return (availWidth, availHeight)
         return (0, 0)
 
-def _listWrapOn(F,availWidth,canv,mergeSpace=1):
+def _listWrapOn(F,availWidth,canv,mergeSpace=1,obj=None,dims=None):
     '''return max width, required height for a list of flowables F'''
     W = 0
     H = 0
     pS = 0
-    n = len(F)
-    nm1 = n - 1
-    for i in xrange(n):
-        f = F[i]
+    atTop = 1
+    for f in F:
         w,h = f.wrapOn(canv,availWidth,0xfffffff)
+        if dims is not None: dims.append((w,h))
+        if w<=_FUZZ or h<=_FUZZ: continue
         W = max(W,w)
-        H = H+h
-        if i:
+        H += h
+        if not atTop:
             h = f.getSpaceBefore()
-            if mergeSpace: H += max(h-pS,0) 
-            else: H += h
-        if i!=nm1:
-            pS = f.getSpaceAfter()
-            H += pS
-    return W, H
+            if mergeSpace: h = max(h-pS,0) 
+            H += h
+        else:
+            if obj is not None: obj._spaceBefore = f.getSpaceBefore()
+            atTop = 0
+        pS = f.getSpaceAfter()
+        H += pS
+    if obj is not None: obj._spaceAfter = pS
+    return W, H-pS
 
-def _makeIndexable(V):
+def _flowableSublist(V):
     "if it isn't a list or tuple, wrap it in a list"
     if type(V) not in (ListType, TupleType): V = V is not None and [V] or []
+    from doctemplate import LCActionFlowable
+    assert not [x for x in V if isinstance(x,LCActionFlowable)],'LCActionFlowables not allowed in sublists'
     return V
 
-class KeepTogether(Flowable):
+class _ContainerSpace:  #Abstract some common container like behaviour
+    def getSpaceBefore(self):
+        for c in self._content:
+            if not hasattr(c,'frameAction'):
+                return c.getSpaceBefore()
+        return 0
+
+    def getSpaceAfter(self,content=None):
+        #this needs 2.4
+        #for c in reversed(content or self._content):
+        reverseContent = (content or self._content)[:]
+        reverseContent.reverse()
+        for c in reverseContent:
+            if not hasattr(c,'frameAction'):
+                return c.getSpaceAfter()
+        return 0
+
+class KeepTogether(_ContainerSpace,Flowable):
     def __init__(self,flowables,maxHeight=None):
-        self._flowables = _makeIndexable(flowables)
+        self._content = _flowableSublist(flowables)
         self._maxHeight = maxHeight
 
     def __repr__(self):
-        f = self._flowables
+        f = self._content
         L = map(repr,f)
         import string
         L = "\n"+string.join(L, "\n")
@@ -464,15 +502,35 @@ class KeepTogether(Flowable):
         return "KeepTogether(%s,maxHeight=%s) # end KeepTogether" % (L,self._maxHeight)
 
     def wrap(self, aW, aH):
-        W,H = _listWrapOn(self._flowables,aW,self.canv)
-        self._CPage = (H>aH) and (not self._maxHeight or H<=self._maxHeight)
+        dims = []
+        W,H = _listWrapOn(self._content,aW,self.canv,dims=dims)
+        self._H = H
+        self._H0 = dims and dims[0][1] or 0
+        self._wrapInfo = aW,aH
         return W, 0xffffff  # force a split
 
     def split(self, aW, aH):
-        S = getattr(self,'_CPage',1) and [CondPageBreak(aH+1)] or []
-        for f in self._flowables: S.append(f)
+        if getattr(self,'_wrapInfo',None)!=(aW,aH): self.wrap(aW,aH)
+        S = self._content[:]
+        C0 = self._H>aH and (not self._maxHeight or aH>self._maxHeight)
+        C1 = self._H0>aH
+        if C0 or C1:
+            if C0:
+                from doctemplate import FrameBreak
+                A = FrameBreak
+            else:
+                from doctemplate import NullActionFlowable
+                A = NullActionFlowable
+            S.insert(0,A())
         return S
 
+    def identity(self, maxLen=None):
+        msg = "<KeepTogether at %s%s> containing :%s" % (hex(id(self)),self._frameName(),"\n".join([f.identity() for f in self._content]))
+        if maxLen:
+            return msg[0:maxLen]
+        else:
+            return msg
+
 class Macro(Flowable):
     """This is not actually drawn (i.e. it has zero height)
     but is executed when it would fit in the frame.  Allows direct
@@ -506,22 +564,39 @@ class CallerMacro(Flowable):
 
 class ParagraphAndImage(Flowable):
     '''combine a Paragraph and an Image'''
-    def __init__(self,P,I,xpad=3,ypad=3):
-        self.P, self.style, self.I, self.xpad, self.ypad = P,P.style,I,xpad,ypad
+    def __init__(self,P,I,xpad=3,ypad=3,side='right'):
+        self.P = P
+        self.I = I
+        self.xpad = xpad
+        self.ypad = ypad
+        self._side = side
+
+    def getSpaceBefore(self):
+        return max(self.P.getSpaceBefore(),self.I.getSpaceBefore())
+
+    def getSpaceAfter(self):
+        return max(self.P.getSpaceAfter(),self.I.getSpaceAfter())
 
     def wrap(self,availWidth,availHeight):
         wI, hI = self.I.wrap(availWidth,availHeight)
-        self.wI, self.hI = wI, hI
+        self.wI = wI
+        self.hI = hI
         # work out widths array for breaking
         self.width = availWidth
-        P, style, xpad, ypad = self.P, self.style, self.xpad, self.ypad
+        P = self.P
+        style = P.style
+        xpad = self.xpad
+        ypad = self.ypad
         leading = style.leading
         leftIndent = style.leftIndent
         later_widths = availWidth - leftIndent - style.rightIndent
         intermediate_widths = later_widths - xpad - wI
         first_line_width = intermediate_widths - style.firstLineIndent
         P.width = 0
-        P.blPara = P.breakLines([first_line_width] + int((hI+ypad)/leading)*[intermediate_widths]+[later_widths])
+        nIW = int((hI+ypad)/leading)
+        P.blPara = P.breakLines([first_line_width] + nIW*[intermediate_widths]+[later_widths])
+        if self._side=='left':
+            self._offsets = [wI+xpad]*(1+nIW)+[0]
         P.height = len(P.blPara.lines)*leading
         self.height = max(hI,P.height)
         return (self.width, self.height)
@@ -533,15 +608,23 @@ class ParagraphAndImage(Flowable):
         if not S: return S
         P = self.P = S[0]
         del S[0]
-        style = self.style = P.style
+        style = P.style
         P.height = len(self.P.blPara.lines)*style.leading
         self.height = max(hI,P.height)
         return [self]+S
 
     def draw(self):
         canv = self.canv
-        self.I.drawOn(canv,self.width-self.wI-self.xpad,self.height-self.hI)
-        self.P.drawOn(canv,0,0)
+        if self._side=='left':
+            self.I.drawOn(canv,0,self.height-self.hI)
+            self.P._offsets = self._offsets
+            try:
+                self.P.drawOn(canv,0,0)
+            finally:
+                del self.P._offsets
+        else:
+            self.I.drawOn(canv,self.width-self.wI-self.xpad,self.height-self.hI)
+            self.P.drawOn(canv,0,0)
 
 class FailOnWrap(Flowable):
     def wrap(self, availWidth, availHeight):
@@ -565,7 +648,8 @@ class HRFlowable(Flowable):
             lineCap='round',
             color=lightgrey,
             spaceBefore=1, spaceAfter=1,
-            hAlign='CENTER', vAlign='BOTTOM'):
+            hAlign='CENTER', vAlign='BOTTOM',
+            dash=None):
         Flowable.__init__(self)
         self.width = width
         self.lineWidth = thickness
@@ -575,6 +659,7 @@ class HRFlowable(Flowable):
         self.color = color
         self.hAlign = hAlign
         self.vAlign = vAlign
+        self.dash = dash
 
     def __repr__(self):
         return "HRFlowable(width=%s, height=%s)" % (self.width, self.height)
@@ -595,15 +680,35 @@ class HRFlowable(Flowable):
         canv.setLineWidth(self.lineWidth)
         canv.setLineCap({'butt':0,'round':1, 'square': 2}[self.lineCap.lower()])
         canv.setStrokeColor(self.color)
+        if self.dash: canv.setDash(self.dash)
         canv.line(0, 0, self._width, self.height)
         canv.restoreState()
 
 class _PTOInfo:
     def __init__(self,trailer,header):
-        self.trailer = _makeIndexable(trailer)
-        self.header = _makeIndexable(header)
+        self.trailer = _flowableSublist(trailer)
+        self.header = _flowableSublist(header)
 
-class PTOContainer(Flowable):
+class _Container(_ContainerSpace):  #Abstract some common container like behaviour
+    def drawOn(self, canv, x, y, _sW=0, scale=1.0, content=None, aW=None):
+        '''we simulate being added to a frame'''
+        pS = 0
+        if aW is None: aW = self.width
+        aW = scale*(aW+_sW)
+        if content is None:
+            content = self._content
+        y += self.height*scale
+        for c in content:
+            w, h = c.wrapOn(canv,aW,0xfffffff)
+            if w<_FUZZ or h<_FUZZ: continue
+            if c is not content[0]: h += max(c.getSpaceBefore()-pS,0)
+            y -= h
+            c.drawOn(canv,x,y,_sW=aW-w)
+            if c is not content[-1]:
+                pS = c.getSpaceAfter()
+                y -= pS
+
+class PTOContainer(_Container,Flowable):
     '''PTOContainer(contentList,trailerList,headerList)
     
     A container for flowables decorated with trailer & header lists.
@@ -613,7 +718,7 @@ class PTOContainer(Flowable):
     def __init__(self,content,trailer=None,header=None):
         I = _PTOInfo(trailer,header)
         self._content = C = []
-        for _ in _makeIndexable(content):
+        for _ in _flowableSublist(content):
             if isinstance(_,PTOContainer):
                 C.extend(_._content)
             else:
@@ -624,16 +729,11 @@ class PTOContainer(Flowable):
         self.width, self.height = _listWrapOn(self._content,availWidth,self.canv)
         return self.width,self.height
 
-    def getSpaceBefore(self):
-        return self._content[0].getSpaceBefore()
-
-    def getSpaceAfter(self):
-        return self._content[-1].getSpaceAfter()
-
     def split(self, availWidth, availHeight):
+        if availHeight<0: return []
         canv = self.canv
         C = self._content
-        x = i = H = pS = 0
+        x = i = H = pS = hx = 0
         n = len(C)
         I2W = {}
         for x in xrange(n):
@@ -648,46 +748,304 @@ class PTOContainer(Flowable):
             else:
                 T,tW,tH,tSB = I2W[I]
             _, h = c.wrapOn(canv,availWidth,0xfffffff)
-            if x: h += max(c.getSpaceBefore()-pS,0)
+            if x:
+                hx = max(c.getSpaceBefore()-pS,0)
+                h += hx
             pS = c.getSpaceAfter()
             H += h+pS
-            if H+tH+max(tSB,pS)>=availHeight-_FUZZ: break
+            tHS = tH+max(tSB,pS)
+            if H+tHS>=availHeight-_FUZZ: break
             i += 1
 
         #first retract last thing we tried
         H -= (h+pS)
 
         #attempt a sub split on the last one we have
-        aH = (availHeight - H - max(pS,tSB) - tH)*0.99
+        aH = (availHeight-H-tHS-hx)*0.99999
         if aH>=0.05*availHeight:
             SS = c.splitOn(canv,availWidth,aH)
         else:
             SS = []
-        if SS:
-            from doctemplate import  FrameBreak
-            F = [FrameBreak()]
+        F = [UseUpSpace()]
 
-        if SS:
+        if len(SS)>1:
             R1 = C[:i] + SS[:1] + T + F
             R2 = Hdr + SS[1:]+C[i+1:]
         elif not i:
             return []
         else:
-            R1 = C[:i-1]+T
+            R1 = C[:i]+T+F
             R2 = Hdr + C[i:]
-        return R1 + [PTOContainer(R2,deepcopy(I.trailer),deepcopy(I.header))]
+        T =  R1 + [PTOContainer(R2,deepcopy(I.trailer),deepcopy(I.header))]
+        return T
+
+#utility functions used by KeepInFrame
+def _hmodel(s0,s1,h0,h1):
+    # calculate the parameters in the model
+    # h = a/s**2 + b/s
+    a11 = 1./s0**2
+    a12 = 1./s0
+    a21 = 1./s1**2
+    a22 = 1./s1
+    det = a11*a22-a12*a21
+    b11 = a22/det
+    b12 = -a12/det
+    b21 = -a21/det
+    b22 = a11/det
+    a = b11*h0+b12*h1
+    b = b21*h0+b22*h1
+    return a,b
+
+def _qsolve(h,(a,b)):
+    '''solve the model v = a/s**2 + b/s for an s which gives us v==h'''
+    if abs(a)<=_FUZZ:
+        return b/h
+    t = 0.5*b/a
+    from math import sqrt
+    f = -h/a
+    r = t*t-f
+    if r<0: return None
+    r = sqrt(r)
+    if t>=0:
+        s1 = -t - r 
+    else:
+        s1 = -t + r
+    s2 = f/s1
+    return max(1./s1, 1./s2)
+
+class KeepInFrame(_Container,Flowable):
+    def __init__(self, maxWidth, maxHeight, content=[], mergeSpace=1, mode='shrink', name=''):
+        '''mode describes the action to take when overflowing
+            error       raise an error in the normal way
+            continue    ignore ie just draw it and report maxWidth, maxHeight
+            shrink      shrinkToFit
+            truncate    fit as much as possible
+        '''
+        self.name = name
+        self.maxWidth = maxWidth
+        self.maxHeight = maxHeight
+        self.mode = mode
+        assert mode in ('error','overflow','shrink','truncate'), '%s invalid mode value %s' % (self.identity(),mode)
+        assert maxHeight>=0,  '%s invalid maxHeight value %s' % (self.identity(),maxHeight)
+        if mergeSpace is None: mergeSpace = overlapAttachedSpace
+        self.mergespace = mergeSpace
+        self._content = content
+
+    def _getAvailableWidth(self):
+        return self.maxWidth - self._leftExtraIndent - self._rightExtraIndent
+
+    def identity(self, maxLen=None):
+        return "<%s at %s%s%s> size=%sx%s" % (self.__class__.__name__, hex(id(self)), self._frameName(),
+                getattr(self,'name','') and (' name="%s"'% getattr(self,'name','')) or '',
+                getattr(self,'maxWidth','') and (' maxWidth=%s'%fp_str(getattr(self,'maxWidth',0))) or '',
+                getattr(self,'maxHeight','')and (' maxHeight=%s' % fp_str(getattr(self,'maxHeight')))or '')
+
+    def wrap(self,availWidth,availHeight):
+        from doctemplate import LayoutError
+        mode = self.mode
+        maxWidth = float(self.maxWidth or availWidth)
+        maxHeight = float(self.maxHeight or availHeight)
+        W, H = _listWrapOn(self._content,availWidth,self.canv)
+        if (mode=='error' and (W>availWidth+_FUZZ or H>availHeight+_FUZZ)):
+            ident = 'content %sx%s too large for %s' % (W,H,self.identity(30))
+            #leave to keep apart from the raise
+            raise LayoutError(ident)
+        elif W<=availWidth+_FUZZ and H<=availHeight+_FUZZ:
+            self.width = W-_FUZZ      #we take what we get
+            self.height = H-_FUZZ
+        elif (maxWidth>=availWidth+_FUZZ or maxHeight>=availHeight+_FUZZ):
+            ident = 'Specified size too large for available space %sx%s in %s' % (availWidth,availHeight,self.identity(30))
+            #leave to keep apart from the raise
+            raise LayoutError(ident)
+        elif mode in ('overflow','truncate'):   #we lie
+            self.width = min(maxWidth,W)-_FUZZ
+            self.height = min(maxHeight,H)-_FUZZ
+        else:
+            def func(x):
+                W, H = _listWrapOn(self._content,x*availWidth,self.canv)
+                W /= x
+                H /= x
+                return W, H
+            W0 = W
+            H0 = H
+            s0 = 1
+            if W>maxWidth+_FUZZ:
+                #squeeze out the excess width and or Height
+                s1 = W/maxWidth
+                W, H = func(s1)
+                if H<=maxHeight+_FUZZ:
+                    self.width = W-_FUZZ
+                    self.height = H-_FUZZ
+                    self._scale = s1
+                    return W,H
+                s0 = s1
+                H0 = H
+                W0 = W
+            s1 = H/maxHeight
+            W, H = func(s1)
+            self.width = W-_FUZZ
+            self.height = H-_FUZZ
+            self._scale = s1
+            if H<min(0.95*maxHeight,maxHeight-10) or H>=maxHeight+_FUZZ:
+                #the standard case W should be OK, H is short we want
+                #to find the smallest s with H<=maxHeight
+                H1 = H
+                for f in 0, 0.01, 0.05, 0.10, 0.15:
+                    #apply the quadratic model
+                    s = _qsolve(maxHeight*(1-f),_hmodel(s0,s1,H0,H1))
+                    W, H = func(s)
+                    if H<=maxHeight+_FUZZ and W<=maxWidth+_FUZZ:
+                        self.width = W-_FUZZ
+                        self.height = H-_FUZZ
+                        self._scale = s
+                        break
+
+        return self.width, self.height
 
     def drawOn(self, canv, x, y, _sW=0):
-        '''we simulate being added to a frame'''
-        pS = 0
-        aW = self.width+_sW
-        C = self._content
-        y += self.height
-        for c in C:
-            w, h = c.wrapOn(canv,aW,0xfffffff)
-            if c is not C[0]: h += max(c.getSpaceBefore()-pS,0)
-            y -= h
-            c.drawOn(canv,x,y,_sW=aW-w)
-            if c is not C[-1]:
-                pS = c.getSpaceAfter()
-                y -= pS
+        scale = getattr(self,'_scale',1.0)
+        truncate = self.mode=='truncate'
+        ss = scale!=1.0 or truncate
+        if ss:
+            canv.saveState()
+            if truncate:
+                p = canv.beginPath()
+                p.rect(x, y, self.width,self.height)
+                canv.clipPath(p,stroke=0)
+            else:
+                canv.translate(x,y)
+                x=y=0
+                canv.scale(1.0/scale, 1.0/scale)
+        _Container.drawOn(self, canv, x, y, _sW=_sW, scale=scale)
+        if ss: canv.restoreState()
+
+class ImageAndFlowables(_Container,Flowable):
+    '''combine a list of flowables and an Image'''
+    def __init__(self,I,F,imageLeftPadding=0,imageRightPadding=3,imageTopPadding=0,imageBottomPadding=3,
+                    imageSide='right'):
+        self._content = _flowableSublist(F)
+        self._I = I
+        self._irpad = imageRightPadding
+        self._ilpad = imageLeftPadding
+        self._ibpad = imageBottomPadding
+        self._itpad = imageTopPadding
+        self._side = imageSide
+
+    def getSpaceAfter(self):
+        if hasattr(self,'_C1'):
+            C = self._C1
+        elif hasattr(self,'_C0'):
+            C = self._C0
+        else:
+            C = self._content
+        return _Container.getSpaceAfter(self,C)
+
+    def getSpaceBefore(self):
+        return max(self._I.getSpaceBefore(),_Container.getSpaceBefore(self))
+
+    def _reset(self):
+        for a in ('_wrapArgs','_C0','_C1'):
+            try:
+                delattr(self,a)
+            except:
+                pass
+
+    def wrap(self,availWidth,availHeight):
+        canv = self.canv
+        if hasattr(self,'_wrapArgs'):
+            if self._wrapArgs==(availWidth,availHeight):
+                return self.width,self.height
+            self._reset()
+        self._wrapArgs = availWidth, availHeight
+        wI, hI = self._I.wrap(availWidth,availHeight)
+        self._wI = wI
+        self._hI = hI
+        ilpad = self._ilpad
+        irpad = self._irpad
+        ibpad = self._ibpad
+        itpad = self._itpad
+        self._iW = availWidth - irpad - wI - ilpad
+        aH = itpad + hI + ibpad
+        W,H0,self._C0,self._C1 = self._findSplit(canv,self._iW,aH)
+        self.width = availWidth
+        aH = self._aH = max(aH,H0)
+        if not self._C1:
+            self.height = aH
+        else:
+            W1,H1 = _listWrapOn(self._C1,availWidth,canv)
+            self.height = aH+H1
+        return self.width, self.height
+
+    def split(self,availWidth, availHeight):
+        if hasattr(self,'_wrapArgs'):
+            if self._wrapArgs!=(availWidth,availHeight):
+                self._reset()
+        W,H=self.wrap(availWidth,availHeight)
+        if self._aH>availHeight: return []
+        C1 = self._C1
+        if C1:
+            c0 = C1[0]
+            S = c0.split(availWidth,availHeight-self._aH)
+            if not S:
+                self._C1 = []
+                self.height = self._aH
+            else:
+                self._C1 = [S[0]]
+                self.height = self._aH + S[0].height
+                C1 = S[1:]+C1[1:]
+        else:
+            self._C1 = []
+            self.height = self._aH
+        return [self]+C1
+
+    def drawOn(self, canv, x, y, _sW=0):
+        if self._side=='left':
+            Ix = x + self._ilpad
+            Fx = Ix+ self._irpad + self._wI
+        else:
+            Ix = x + self.width-self._wI-self._irpad - self._ilpad
+            Fx = x
+        self._I.drawOn(canv,Ix,y+self.height-self._itpad-self._hI)
+        _Container.drawOn(self, canv, Fx, y, content=self._C0, aW=self._iW)
+        if self._C1:
+            _Container.drawOn(self, canv, x, y-self._aH,content=self._C1)
+
+    def _findSplit(self,canv,availWidth,availHeight,mergeSpace=1,obj=None):
+        '''return max width, required height for a list of flowables F'''
+        W = 0
+        H = 0
+        pS = sB = 0
+        atTop = 1
+        F = self._content
+        for i,f in enumerate(F):
+            w,h = f.wrapOn(canv,availWidth,0xfffffff)
+            if w<=_FUZZ or h<=_FUZZ: continue
+            W = max(W,w)
+            if not atTop:
+                s = f.getSpaceBefore()
+                if mergeSpace: s = max(s-pS,0)
+                H += s
+            else:
+                if obj is not None: obj._spaceBefore = f.getSpaceBefore()
+                atTop = 0
+            if H>=availHeight:
+                return W, availHeight, F[:i],F[i:]
+            H += h
+            if H>availHeight:
+                from paragraph import Paragraph
+                aH = availHeight-(H-h)
+                if isinstance(f,(Paragraph,Preformatted)):
+                    leading = f.style.leading
+                    nH = leading*int(aH/float(leading))+_FUZZ
+                    if nH<aH: nH += leading
+                    availHeight += nH-aH
+                    aH = nH
+                S = deepcopy(f).split(availWidth,aH)
+                if not S:
+                    return W, availHeight, F[:i],F[i:]
+                else:
+                    return W,availHeight,F[:i]+S[:1],S[1:]+F[i+1:]
+            pS = f.getSpaceAfter()
+            H += pS
+        if obj is not None: obj._spaceAfter = pS
+        return W, H-pS, F, []
