@@ -39,49 +39,71 @@ import sys,os
 
 import re
 
+from mx import DateTime as mdt
+re_from = re.compile('.* from ([a-zA-Z_0-9]+) .*$');
+re_into = re.compile('.* into ([a-zA-Z_0-9]+) .*$');
+
 class fake_cursor:
 	nbr = 0
 	_tables = {}
+	sql_from_log = {}
+	sql_into_log = {}
+	sql_log = False
+	count = 0
+
 	def __init__(self, db, con, dbname):
 		self.db = db
 		self.obj = db.cursor()
 		self.con = con
 		self.dbname = dbname
 
-	def execute_dont_run(self,*args):
-		if not fake_cursor.nbr % 100:
-			print 'sql: ',fake_cursor.nbr
-		res = re.match('^select.* from ([a-zA-Z_]+) .*$', args[0], re.I)
-		if res:
-			fake_cursor._tables.setdefault(res.group(1), 0)
-			fake_cursor._tables[res.group(1)] += 1
+	def execute(self, sql, params=None):
+		if not params:
+			params=()
+		def base_string(s):
+			if isinstance(s, unicode):
+				return s.encode('utf-8')
+			return s
+		p=map(base_string, params)
+		if isinstance(sql, unicode):
+			sql = sql.encode('utf-8')
+		if self.sql_log:
+			now = mdt.now()
+		if p:
+			res = self.obj.execute(sql, p)
+		else:
+			res = self.obj.execute(sql)
+		if self.sql_log:
+			self.count+=1
+			res_from = re_from.match(sql.lower())
+			if res_from:
+				self.sql_from_log.setdefault(res_from.group(1), [0, 0])
+				self.sql_from_log[res_from.group(1)][0] += 1
+				self.sql_from_log[res_from.group(1)][1] += mdt.now() - now
+			res_into = re_into.match(sql.lower())
+			if res_into:
+				self.sql_into_log.setdefault(res_into.group(1), [0, 0])
+				self.sql_into_log[res_into.group(1)][0] += 1
+				self.sql_into_log[res_into.group(1)][1] += mdt.now() - now
+		return res
 
-		#else:
-		#	if len(args)>1:
-		#		print 'sql: ',fake_cursor.nbr, args[0], args[1]
-		#	else:
-		#		print 'sql: ',fake_cursor.nbr, args[0]
-
-		if not fake_cursor.nbr % 5000:
-			ct = []
-			for t,c in fake_cursor._tables.items():
-				ct.append([c,t])
-			ct.sort()
-			ct.reverse()
-			print 'After %d queries' % (fake_cursor.nbr,)
-			for line in ct[:50]:
-				print '    %s: %d' % (line[1], line[0])
-
-		#if len(args)>1:
-		#	print 'sql: ',fake_cursor.nbr, args[0], args[1]
-		#else:
-		#	print 'sql: ',fake_cursor.nbr, args[0]
-
-		fake_cursor.nbr += 1
-		return self.obj.execute(*args)
+	def print_log(self, type='from'):
+		print "SQL LOG %s:" % (type,)
+		if type == 'from':
+			logs = self.sql_from_log.items()
+		else:
+			logs = self.sql_into_log.items()
+		logs.sort(lambda x, y: cmp(x[1][1], y[1][1]))
+		sum=0
+		for r in logs:
+			print "table:", r[0], ":", str(r[1][1]), "/", r[1][0]
+			sum+= r[1][1]
+		print "SUM:%s/%d"% (sum, self.count)
 
 	def close(self):
-#		print "close cursors fno:", [i.fileno() for i in self.db.cursors]
+		if self.sql_log:
+			self.print_log('from')
+			self.print_log('into')
 		self.obj.close()
 
 		# This force the cursor to be freed, and thus, available again. It is 
@@ -90,10 +112,8 @@ class fake_cursor:
 		# collected as fast as they should). The problem is probably due in 
 		# part because browse records keep a reference to the cursor.
 		del self.obj
-#		print "after close cursors fno:", [i.fileno() for i in self.db.cursors]
 
 	def __getattr__(self, name):
-#		print 'LOOK',name
 		return getattr(self.obj, name)
 
 class fakedb:
@@ -116,7 +136,8 @@ def db_connect(db_name):
 	name = "dbname=%s" % db_name
 	user = tools.config['db_user'] and "user=%s" % tools.config['db_user'] or ''
 	password = tools.config['db_password'] and "password=%s" % tools.config['db_password'] or ''
-	tdb = psycopg.connect('%s %s %s %s %s' % (host, port, name, user, password), serialize=0)
+	maxconn = int(tools.config['db_maxconn']) or 64
+	tdb = psycopg.connect('%s %s %s %s %s' % (host, port, name, user, password), serialize=0, maxconn=maxconn)
 	fdb = fakedb(tdb, db_name)
 	return fdb
 
