@@ -502,8 +502,8 @@ def get_languages():
 		'en_EN': 'English (default)',
 		'hu_HU': 'Hungarian',
 		'it_IT': 'Italiano',
-		'pt_BR': 'Portugese (Brasil)',
-		'pt_PT': 'Portugese (Portugal)',
+		'pt_BR': 'Portuguese (Brasil)',
+		'pt_PT': 'Portuguese (Portugal)',
 		'nl_NL': 'Nederlands',
 		'ro_RO': 'Romanian',
 		'ru_RU': 'Russian',
