@@ -70,8 +70,10 @@ def process(cr, workitem, ident, signal=None, force_running=False):
 		for trans in alltrans:
 			if trans['trigger_model']:
 				ids = wkf_expr._eval_expr(cr,ident,workitem,trans['trigger_expr_id'])
-				for id in ids:
-					cr.execute('insert into wkf_triggers (model,res_id,instance_id,workitem_id) values (%s,%d,%d,%d)', (trans['trigger_model'],id,workitem['inst_id'], workitem['id']))
+				for res_id in ids:
+					cr.execute('select nextval(\'wkf_triggers_id_seq\')')
+					id =cr.fetchone()[0]
+					cr.execute('insert into wkf_triggers (model,res_id,instance_id,workitem_id,id) values (%s,%d,%d,%d,%d)', (trans['trigger_model'],res_id,workitem['inst_id'], workitem['id'], id))
 
 	return True
 
@@ -152,8 +154,6 @@ def _split_test(cr, workitem, split_mode, ident, signal=None):
 			cr.execute('select count(*) from wkf_witm_trans where trans_id=%d and inst_id=%d', (transition['id'], workitem['inst_id']))
 			if not cr.fetchone()[0]:
 				transitions.append((transition['id'], workitem['inst_id']))
-	if not test:
-		pass
 	if test and len(transitions):
 		cr.executemany('insert into wkf_witm_trans (trans_id,inst_id) values (%d,%d)', transitions)
 		cr.execute('delete from wkf_workitem where id=%d', (workitem['id'],))
