@@ -72,70 +72,78 @@ password = hasattr(options, 'db_password') and "password=%s" % options.db_passwo
 db = psycopg.connect('%s %s %s %s %s' % (host, port, name, user, password), serialize=0)
 cr = db.cursor()
 
-# ----------------------------- #
-# convert date_planned to delay #
-# ----------------------------- #
+# ------------------------------------------- #
+# convert partner payment terms to properties #
+# ------------------------------------------- #
+
+# setup
+
+cr.execute("select id from ir_model_fields where name='property_payment_term' and model='res.partner'")
+fields_id = cr.fetchone()[0]
+
+cr.execute("select company_id from res_users where company_id is not null limit 1")
+company_id = cr.fetchone()[0]
+
+# get partners
+cr.execute("SELECT c.relname FROM pg_class c, pg_attribute a WHERE c.relname='res_partner' AND a.attname='payment_term' AND c.oid=a.attrelid")
+partners=[]
+drop_payment_term=False
+if cr.rowcount:
+	drop_payment_term=True
+	cr.execute("select id, payment_term from res_partner where payment_term is not null")
+	partners = cr.dictfetchall()
+
+# loop over them
+
+for partner in partners:
+	value = 'account.payment.term,%d' % partner['payment_term']
+	res_id = 'res.partner,%d' % partner['id']
+	cr.execute(
+		"insert into ir_property(name, value, res_id, company_id, fields_id) "\
+		"values(%s, %s, %s, %d, %d)", 
+		('property_payment_term', value, res_id, company_id, fields_id))
+
+# remove the field
+if drop_payment_term:
+	cr.execute("alter table res_partner drop column payment_term")
+cr.execute("delete from ir_model_fields where model = 'res.partner' and name = 'payment_term'")
 
-cr.execute('SELECT c.relname FROM pg_class c WHERE c.relname = \'sale_order_line\'')
-if cr.fetchall():
-	cr.execute("update sale_order_line set delay = pt.sale_delay from product_product as po, product_template as pt where product_id = po.id and po.product_tmpl_id = pt.id")
 cr.commit()
 
-# --------------------------------------------------------------------------- #
-# move account_invoice.project_id to account_invoice_line.account_analytic_id #
-# --------------------------------------------------------------------------- #
+# ------------------------ #
+# remove duplicate reports #
+# ------------------------ #
+
+cr.execute("select model, report_name from ir_act_report_xml group by model, report_name having count(*)>1")
+reports_wh_duplicates = cr.dictfetchall()
+
+cr.execute("select res_id from ir_model_data where model='ir.actions.report.xml'")
+registered_reports = cr.fetchall()
+reg_reports_ids = ','.join([str(id) for (id,) in registered_reports])
+
+for report in reports_wh_duplicates:
+	cr.execute("select id from ir_act_report_xml where model=%s and report_name=%s and id not in ("+reg_reports_ids+")", (report['model'], report['report_name']))
+	(id,) = cr.fetchone()
+	cr.execute("delete from ir_act_report_xml where id=%d", (id,))
+	cr.execute("delete from ir_values where value='ir.actions.report.xml,%d'", (id,))
 
-cr.execute('SELECT a.attname FROM pg_class c, pg_attribute a WHERE c.relname = \'account_invoice\' AND a.attname = \'project_id\' AND c.oid = a.attrelid')
-if cr.fetchall():
-	cr.execute("update account_invoice_line set account_analytic_id = ai.project_id from account_invoice as ai where invoice_id = ai.id")
 cr.commit()
 
-# ------------------------------------------------------------------------- #
-# move purchase_order.project_id to purchase_order_line.account_analytic_id #
-# ------------------------------------------------------------------------- #
+# ------------------------------------- #
+# remove duplicate workflow transitions #
+# ------------------------------------- #
 
-cr.execute('SELECT a.attname FROM pg_class c, pg_attribute a WHERE c.relname = \'purchase_order\' AND a.attname = \'project_id\' AND c.oid = a.attrelid')
-if cr.fetchall():
-	cr.execute("update purchase_order_line set account_analytic_id = po.project_id from purchase_order as po where order_id = po.id")
+# this removes all transitions which are not registered in ir_model_data
+
+cr.execute("delete from wkf_transition where id not in (select res_id from ir_model_data where model='workflow.transition')")
 cr.commit()
 
+# -------------------------------- #
+# remove bad "default" menu action #
+# -------------------------------- #
 
-# --------------- #
-# remove old menu #
-# --------------- #
-
-cr.execute("delete from ir_ui_menu where (id not in (select parent_id from ir_ui_menu where parent_id is not null)) and (id not in (select res_id from ir_values where model='ir.ui.menu'))")
-cr.commit()
-
-# --------------------- #
-# set value for menu_id #
-# --------------------- #
-
-cr.execute('update res_users set menu_id = action_id')
-cr.commit()
-
-# --------------------------------------------------- #
-# set reconciled state to valid for account move line #
-# --------------------------------------------------- #
-
-cr.execute('SELECT c.relname FROM pg_class c WHERE c.relname = \'account_move_line\'')
-if cr.fetchall():
-	cr.execute('UPDATE account_move_line set state = \'valid\' WHERE state = \'reconciled\'')
-cr.commit()
-
-# ------------ #
-# migrate bank #
-# ------------ #
-
-cr.execute('SELECT bank_name, swift, id FROM res_partner_bank WHERE bank IS NULL AND (bank_name IS NOT NULL OR swift IS NOT NULL)')
-res = cr.dictfetchall()
-for line in res:
-	cr.execute('SELECT NEXTVAL(\'res_bank_id_seq\')')
-	bank_id = cr.fetchone()[0]
-	cr.execute('INSERT INTO res_bank (id, name, bic, active) VALUES (%d, %s, %s, True)',
-			(bank_id, line['bank_name'], line['swift']))
-	cr.execute('UPDATE res_partner_bank SET bank = %d WHERE id = %d',
-			(bank_id, line['id']))
+cr.execute("delete from ir_values where key='action' and model='ir.ui.menu' and res_id is null")
 cr.commit()
 
 cr.close()
+
