@@ -90,28 +90,8 @@ def change_column(cr, table, column, new_type, copy):
 	for command in commands:
 		cr.execute(command)
 
-change_column(cr, 'crm_case', 'date_closed', 'timestamp', True)
-cr.commit()
-
-# -------------------- #
-# add module if needed #
-# -------------------- #
-
-cr.execute("SELECT name FROM ir_module_module")
-if not cr.rowcount:
-	for module in ('base', 'marketing', 'subscription', 'account', 'base_partner_relation', 'audittrail', 'account_followup', 'product', 'hr', 'l10n_simple', 'crm', 'stock', 'hr_timesheet', 'purchase', 'report_purchase', 'mrp', 'sale', 'report_sale', 'delivery', 'project', 'sale_crm', 'hr_timesheet_project', 'scrum', 'report_project'):
-		cr.execute("INSERT INTO ir_module_module (name, state) VALUES ('%s', 'installed')" % module)
-	cr.commit()
-
-# --------------- #
-# remove old menu #
-# --------------- #
-
-while True:
-	cr.execute("select id from ir_ui_menu where id not in (select parent_id from ir_ui_menu where parent_id is not null) and id not in (select res_id from ir_model_data where model='ir.ui.menu')")
-	if not cr.rowcount:
-		break
-	cr.execute("delete from ir_ui_menu where id not in (select parent_id from ir_ui_menu where parent_id is not null) and id not in (select res_id from ir_model_data where model='ir.ui.menu')")
+change_column(cr, 'account_account_type', 'code_from', 'varchar(10)', False)
+change_column(cr, 'account_account_type', 'code_to', 'varchar(10)', False)
 cr.commit()
 
 # ----------------------------------------------------- #
@@ -119,12 +99,17 @@ cr.commit()
 # ----------------------------------------------------- #
 
 for line in (
-		"ALTER TABLE ir_module_module ADD demo BOOLEAN",
-		"ALTER TABLE ir_module_module ALTER demo SET DEFAULT False",
-		"DELETE FROM ir_values WHERE VALUE LIKE '%,False'",
-		"""UPDATE ir_ui_view set arch='<?xml version="1.0"?><tree string="Menu" toolbar="1"><field icon="icon" name="name"/></tree>' where name='ir.ui.menu.tree' and type='tree' and field_parent='child_id'""",
+		"alter table ir_model_fields add group_name varchar(64)",
+		"alter table ir_model_fields add view_load boolean",
+		"alter table ir_model_fields alter group_name set default ''",
+		"alter table ir_model_fields alter view_load set default False",
+		"delete from ir_values where value like '%,False'",
 	):
-	cr.execute(line)
+	try:
+		cr.execute(line)
+	except psycopg.ProgrammingError, e:
+		cr.commit()
+		print e
 
 cr.commit()
 cr.close()
